package Sah::Schema::language::code;

use strict;
use Locale::Codes::Language_Codes ();

our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2023-08-07'; # DATE
our $DIST = 'Sah-Schemas-Language'; # DIST
our $VERSION = '0.005'; # VERSION

my $codes = [];
my $names = [];
{
    for my $alpha2 (keys(%{ $Locale::Codes::Data{'language'}{'code2id'}{'alpha-2'} })) {
        push @$codes, $alpha2;
        my $id = $Locale::Codes::Data{'language'}{'code2id'}{'alpha-2'}{$alpha2}[0];
        push @$names, $Locale::Codes::Data{'language'}{'id2names'}{$id}[0];
    }
    for my $alpha3 (keys(%{ $Locale::Codes::Data{'language'}{'code2id'}{'alpha-3'} })) {
        push @$codes, $alpha3;
        my $id = $Locale::Codes::Data{'language'}{'code2id'}{'alpha-3'}{$alpha3}[0];
        push @$names, $Locale::Codes::Data{'language'}{'id2names'}{$id}[0];
    }

    die "Can't extract country codes from Locale::Codes::Language_Codes"
        unless @$codes;
}

our $schema = [str => {
    summary => 'Language code (alpha-2 or alpha-3)',
    description => <<'_',

Accept only current (not retired) codes. Alpha-2 or alpha-3 codes are accepted.

_
    match => '\A[a-z]{2,3}\z',
    in => $codes,
    'x.in.summaries' => $names,
    'x.perl.coerce_rules' => ['From_str::to_lower'],

    examples => [
        {value=>"", valid=>0},
        {value=>"id", valid=>1, summary=>"Indonesian (2 letter)"},
        {value=>"IND", valid=>1, validated_value=>"ind", summary=>"Indonesian (3 letter)"},
        {value=>"qqq", valid=>0, summary=>"Unknown language code"},
    ],

}, {}];

1;
# ABSTRACT: Language code (alpha-2 or alpha-3)

__END__

=pod

=encoding UTF-8

=head1 NAME

Sah::Schema::language::code - Language code (alpha-2 or alpha-3)

=head1 VERSION

This document describes version 0.005 of Sah::Schema::language::code (from Perl distribution Sah-Schemas-Language), released on 2023-08-07.

=head1 SYNOPSIS

=head2 Sample data and validation results against this schema

 ""  # INVALID

 "id"  # valid (Indonesian (2 letter))

 "IND"  # valid (Indonesian (3 letter)), becomes "ind"

 "qqq"  # INVALID (Unknown language code)

=head2 Using with Data::Sah

To check data against this schema (requires L<Data::Sah>):

 use Data::Sah qw(gen_validator);
 my $validator = gen_validator("language::code*");
 say $validator->($data) ? "valid" : "INVALID!";

The above validator returns a boolean result (true if data is valid, false if
otherwise). To return an error message string instead (empty string if data is
valid, a non-empty error message otherwise):

 my $validator = gen_validator("language::code", {return_type=>'str_errmsg'});
 my $errmsg = $validator->($data);
 
 # a sample valid data
 $data = "id";
 my $errmsg = $validator->($data); # => ""
 
 # a sample invalid data
 $data = "";
 my $errmsg = $validator->($data); # => "Must be one of [\"km\",\"nn\",\"vo\",\"fa\",\"rm\",\"en\",\"aa\",\"oc\",\"mr\",\"ab\",\"tn\",\"lu\",\"oj\",\"cy\",\"sv\",\"lt\",\"sl\",\"tr\",\"lb\",\"la\",\"jv\",\"ps\",\"nr\",\"pi\",\"bm\",\"dv\",\"mn\",\"or\",\"af\",\"ny\",\"ff\",\"ht\",\"ie\",\"tk\",\"ty\",\"ha\",\"cr\",\"su\",\"av\",\"lv\",\"ja\",\"st\",\"id\",\"bh\",\"da\",\"be\",\"my\",\"vi\",\"sa\",\"mk\",\"gd\",\"hu\",\"qu\",\"pl\",\"nd\",\"sw\",\"mh\",\"fo\",\"si\",\"zh\",\"ss\",\"ee\",\"sg\",\"ne\",\"lo\",\"kk\",\"ky\",\"ik\",\"th\",\"te\",\"hi\",\"bn\",\"ch\",\"ce\",\"pt\",\"ho\",\"kr\",\"kj\",\"li\",\"pa\",\"gn\",\"lg\",\"as\",\"so\",\"fi\",\"rn\",\"om\",\"br\",\"ug\",\"kn\",\"ms\",\"ku\",\"ng\",\"iu\",\"mi\",\"se\",\"sh\",\"ru\",\"de\",\"ba\",\"tg\",\"gu\",\"ts\",\"sq\",\"co\",\"ti\",\"am\",\"ia\",\"he\",\"ka\",\"tw\",\"it\",\"os\",\"es\",\"ga\",\"sd\",\"mg\",\"eo\",\"no\",\"to\",\"wa\",\"cs\",\"sm\",\"gv\",\"gl\",\"ae\",\"kl\",\"kv\",\"nv\",\"el\",\"hz\",\"bo\",\"nl\",\"sr\",\"ay\",\"cu\",\"tl\",\"ak\",\"fy\",\"yo\",\"xh\",\"sc\",\"sn\",\"dz\",\"ko\",\"io\",\"ml\",\"hr\",\"ca\",\"ro\",\"uk\",\"bg\",\"ks\",\"ii\",\"ur\",\"ki\",\"rw\",\"is\",\"et\",\"zu\",\"hy\",\"wo\",\"nb\",\"ta\",\"kw\",\"tt\",\"az\",\"na\",\"an\",\"ve\",\"ln\",\"fr\",\"za\",\"ar\",\"yi\",\"sk\",\"cv\",\"eu\",\"mt\",\"kg\",\"bs\",\"fj\",\"bi\",\"ig\",\"uz\",\"itb\",\"loj\",\"kjm\",\"bis\",\"rir\",\"vmz\",\"mun\",\"dks\",\"bgv\",\"anr\",\"zrp\",\"cax\",\"mlg\",\"cui\",\"gwr\",\"kro\",\"skv\",\"cat\",\"ach\",\"tkw\",\"kek\",\"kwc\",\"aup\",\"kwh\",\"bin\",\"nuv\",\"mtv\",\"xyl\",\"sxe\",\"mli\",\"yly\",\"nyw\",\"mus\",\"ilb\",\"aeb\",\"yua\",\"ukh\",\"bgi\",\"skg\",\"mnw\",\"nat\",\"tuz\",\"bwg\",\"mtg\",\"brj\",\"cuv\",\"nug\",\"pes\",\"cts\",\"sea\",\"ski\",\"bgg\",\"til\",\"bcd\",\"ctn\",\"mpd\",\"paf\",\"nui\",\"kmw\",\"wwr\",\"mlv\",\"aeu\",\"mti\",\"ilu\",\"bwi\",\"yty\",\"ipk\",\"nax\",\"prl\",\"itz\",\"vmb\",\"ymd\",\"auj\",\"pkr\",\"ylr\",\"pom\",\"aqr\",\"cpe\",\"rga\",\"zcd\",\"xak\",\"tep\",\"ulm\",\"ogg\",\"rwa\",\"owi\",\"dnu\",\"gia\",\"hmv\",\"lop\",\"cco\",\"vmu\",\"igl\",\"any\",\"xdy\",\"aez\",\"zin\",\"mqw\",\"esh\",\"szy\",\"tub\",\"jbj\",\"nco\",\"mox\",\"hmg\",\"der\",\"dza\",\"rmo\",\"tsh\",\"tsc\",\"syb\",\"wgy\",\"brp\",\"fie\",\"mot\",\"tuu\",\"hmi\",\"aya\",\"oin\",\"xun\",\"tbm\",\"bki\",\"sgg\",\"irr\",\"jos\",\"wkr\",\"bea\",\"bzr\",\"dii\",\"ugh\",\"uro\",\"rof\",\"wom\",\"tmh\",\"xlg\",\"swi\",\"tmc\",\"dig\",\"sgi\",\"bkg\",\"cnl\",\"myl\",\"xtv\",\"swg\",\"xli\",\"nok\",\"nfd\",\"byz\",\"adw\",\"ofo\",\"nnl\",\"tra\",\"dgs\",\"fal\",\"bkv\",\"ysd\",\"div\",\"zyb\",\"kkh\",\"kkc\",\"tpx\",\"sin\",\"var\",\"era\",\"tww\",\"cfd\",\"kzx\",\"cok\",\"kiq\",\"bxe\",\"xtg\",\"pia\",\"swv\",\"xpd\",\"gzi\",\"sis\",\"mdr\",\"tgw\",\"dgn\",\"oyb\",\"xti\",\"dws\",\"mxo\",\"kfp\",\"gom\",\"thr\",\"tpt\",\"gbd\",\"srp\",\"bfo\",\"okv\",\"cjn\",\"eve\",\"abt\",\"mak\",\"cda\",\"gaf\",\"yaq\",\"mji\",\"tve\",\"ldo\",\"mfe\",\"klm\",\"wky\",\"bzy\",\"jav\",\"iry\",\"kuf\",\"mjg\",\"ges\",\"zkv\",\"ksw\",\"cjs\",\"byb\",\"abx\",\"nql\",\"mvk\",\"mdy\",\"hrk\",\"tae\",\"kfj\",\"oki\",\"gvf\",\"wes\",\"rka\",\"rei\",\"jut\",\"phw\",\"lbs\",\"njs\",\"yrb\",\"thy\",\"zkg\",\"mjv\",\"ydg\",\"okg\",\"njn\",\"wen\",\"led\",\"lbn\",\"ktm\",\"pwr\",\"apm\",\"zea\",\"wrl\",\"ikl\",\"ram\",\"nda\",\"vay\",\"waf\",\"reg\",\"tod\",\"kfy\",\"ssq\",\"lwm\",\"mdj\",\"mwu\",\"btu\",\"mxx\",\"iin\",\"mgb\",\"ksn\",\"nib\",\"tce\",\"kzo\",\"epo\",\"nmm\",\"khk\",\"mgu\",\"tpo\",\"nbc\",\"lgm\",\"vaj\",\"nbh\",\"niu\",\"gew\",\"mxt\",\"hml\",\"mwb\",\"kss\",\"mck\",\"ahb\",\"bpk\",\"pxm\",\"cmm\",\"gcf\",\"xkb\",\"zmq\",\"knf\",\"wew\",\"cbc\",\"lbw\",\"urt\",\"igg\",\"srr\",\"sha\",\"kym\",\"cib\",\"bzj\",\"dae\",\"urx\",\"omq\",\"kde\",\"mmf\",\"phn\",\"xku\",\"otz\",\"lif\",\"que\",\"owl\",\"blb\",\"luq\",\"bgl\",\"tig\",\"adn\",\"sju\",\"opk\",\"mgz\",\"jie\",\"jow\",\"niz\",\"vap\",\"olu\",\"pdu\",\"zzj\",\"juo\",\"prg\",\"aja\",\"dbm\",\"thp\",\"bmq\",\"tii\",\"xoo\",\"hum\",\"kfr\",\"zpk\",\"ads\",\"pri\",\"mtl\",\"bwl\",\"zlu\",\"nul\",\"gkp\",\"mdp\",\"dmc\",\"sjb\",\"mwz\",\"btz\",\"bft\",\"sbf\",\"umn\",\"abo\",\"xkz\",\"dgw\",\"xns\",\"otu\",\"kqf\",\"tgn\",\"cul\",\"dhr\",\"bzp\",\"sad\",\"ase\",\"twn\",\"dra\",\"tgs\",\"soe\",\"tiv\",\"siw\",\"ums\",\"bfx\",\"sry\",\"otb\",\"ztu\",\"mll\",\"xnn\",\"eiv\",\"dww\",\"blz\",\"dep\",\"xgb\",\"enu\",\"okl\",\"nao\",\"bha\",\"ytp\",\"lva\",\"tnu\",\"ojb\",\"wrg\",\"wdu\",\"grv\",\"rel\",\"ame\",\"slu\",\"spk\",\"ovd\",\"jal\",\"nqg\",\"tnb\",\"xxt\",\"uss\",\"izr\",\"wri\",\"enb\",\"iki\",\"xgu\",\"brr\",\"ala\",\"gdb\",\"mkb\",\"mqs\",\"hih\",\"hbu\",\"auy\",\"duu\",\"zad\",\"aqp\",\"nvo\",\"fon\",\"mjl\",\"yyr\",\"krt\",\"pkp\",\"laa\",\"stb\",\"gri\",\"bjz\",\"ory\",\"lor\",\"ude\",\"stu\",\"xmf\",\"krx\",\"ybk\",\"fos\",\"bsq\",\"cao\",\"eza\",\"ppq\",\"mbq\",\"kpa\",\"anj\",\"dub\",\"grg\",\"mqn\",\"hbb\",\"ikv\",\"ter\",\"ziw\",\"tza\",\"ata\",\"wrv\",\"gdu\",\"mku\",\"gwj\",\"lkm\",\"dsh\",\"nyn\",\"ghn\",\"csm\",\"fai\",\"biw\",\"boe\",\"nni\",\"rmt\",\"clw\",\"llh\",\"bry\",\"llc\",\"tqu\",\"slz\",\"tkn\",\"nct\",\"nng\",\"rmx\",\"pcf\",\"moo\",\"fag\",\"bbf\",\"tnz\",\"msf\",\"ncx\",\"tqb\",\"tks\",\"fut\",\"ghs\",\"nys\",\"qvz\",\"jbr\",\"ntw\",\"yna\",\"szp\",\"bad\",\"myv\",\"xtl\",\"smq\",\"mkz\",\"wsk\",\"mns\",\"dil\",\"bkl\",\"kmn\",\"anp\",\"nnv\",\"cng\",\"loy\",\"swl\",\"myg\",\"lth\",\"tey\",\"lre\",\"ltc\",\"bjb\",\"bju\",\"cni\",\"nxk\",\"nsm\",\"aur\",\"bvd\",\"orr\",\"mnn\",\"duz\",\"aac\",\"zha\",\"aah\",\"nlw\",\"hsl\",\"rhg\",\"kms\",\"seu\",\"huk\",\"etr\",\"pao\",\"ysc\",\"yub\",\"aea\",\"ila\",\"ttr\",\"aki\",\"jrt\",\"zpm\",\"kkd\",\"bdw\",\"ayz\",\"jpa\",\"opm\",\"xph\",\"xpc\",\"akg\",\"kvl\",\"wul\",\"seb\",\"dyr\",\"axe\",\"mzw\",\"typ\",\"tmd\",\"kby\",\"xhv\",\"xmt\",\"inj\",\"akv\",\"gul\",\"ita\",\"tlr\",\"obt\",\"sch\",\"unx\",\"nkp\",\"zbt\",\"dtp\",\"xmx\",\"krf\",\"snr\",\"kal\",\"giz\",\"jmn\",\"khm\",\"sya\",\"nmk\",\"fuf\",\"pni\",\"pyn\",\"tua\",\"wib\",\"bbt\",\"dyy\",\"afo\",\"yuz\",\"lgk\",\"kxs\",\"ayu\",\"bbx\",\"ayb\",\"rmf\",\"kxn\",\"png\",\"ncf\",\"wmm\",\"msx\",\"toc\",\"yil\",\"toh\",\"ptw\",\"wbh\",\"wiu\",\"lec\",\"tty\",\"jms\",\"leh\",\"sez\",\"pys\",\"pnv\",\"sny\",\"lsp\",\"gmm\",\"xrn\",\"kyk\",\"dna\",\"yej\",\"giu\",\"gbh\",\"new\",\"xel\",\"vma\",\"psm\",\"nkj\",\"inp\",\"bpm\",\"mcm\",\"gib\",\"sqr\",\"suj\",\"plw\",\"tly\",\"rgu\",\"tyj\",\"kbr\",\"wyn\",\"yae\",\"pib\",\"tdi\",\"hov\",\"lmp\",\"chn\",\"gsm\",\"wni\",\"bnr\",\"mxf\",\"agv\",\"mip\",\"ngp\",\"mhi\",\"glw\",\"trb\",\"pmm\",\"ndz\",\"wng\",\"tru\",\"awv\",\"bqy\",\"tdg\",\"ain\",\"csk\",\"bup\",\"kcv\",\"fro\",\"uvl\",\"mhg\",\"mph\",\"bcc\",\"mpc\",\"bch\",\"wtw\",\"rnn\",\"pbc\",\"mfq\",\"djj\",\"pbh\",\"piu\",\"lit\",\"gng\",\"tdv\",\"hoi\",\"arj\",\"nsk\",\"nxm\",\"awg\",\"cdz\",\"knx\",\"taq\",\"agi\",\"mmt\",\"kcg\",\"beb\",\"uby\",\"kwd\",\"knt\",\"gyn\",\"ihb\",\"awi\",\"beu\",\"nhn\",\"roo\",\"mmx\",\"gni\",\"urf\",\"lix\",\"kci\",\"acd\",\"wlw\",\"agg\",\"gct\",\"piz\",\"wao\",\"bqr\",\"rkb\",\"spm\",\"yra\",\"tsd\",\"yww\",\"ids\",\"crv\",\"nrg\",\"hlt\",\"buj\",\"ndu\",\"han\",\"trz\",\"bny\",\"ndb\",\"ygw\",\"has\",\"mij\",\"ngj\",\"lmj\",\"gvo\",\"nri\",\"ise\",\"kie\",\"mrn\",\"onr\",\"bxq\",\"kow\",\"sbx\",\"hvn\",\"zch\",\"ybm\",\"kuo\",\"qxr\",\"ymh\",\"vko\",\"mel\",\"kqx\",\"cri\",\"ymc\",\"bez\",\"bya\",\"ibo\",\"gao\",\"htx\",\"xcm\",\"kqt\",\"gqi\",\"och\",\"mrs\",\"ubr\",\"bff\",\"sbt\",\"crg\",\"arp\",\"shu\",\"ulk\",\"qxp\",\"lio\",\"hid\",\"mmo\",\"xka\",\"jiq\",\"lue\",\"zac\",\"aha\",\"gpe\",\"zah\",\"onp\",\"haw\",\"oac\",\"yiv\",\"arr\",\"bla\",\"tjp\",\"kno\",\"ywn\",\"ygs\",\"xam\",\"cia\",\"shb\",\"pok\",\"bme\",\"xeg\",\"ajz\",\"tpf\",\"pnl\",\"kos\",\"yig\",\"mga\",\"bqp\",\"nwy\",\"nia\",\"buy\",\"djr\",\"kzf\",\"hyx\",\"qwm\",\"yii\",\"ngy\",\"mwa\",\"bta\",\"miy\",\"bnj\",\"soq\",\"mrw\",\"asq\",\"kon\",\"lmy\",\"txc\",\"txh\",\"frt\",\"tcq\",\"kem\",\"rnw\",\"dru\",\"ary\",\"ddg\",\"kag\",\"aoe\",\"aiw\",\"gui\",\"sse\",\"kut\",\"tjj\",\"gax\",\"hto\",\"vkt\",\"kai\",\"aad\",\"ddi\",\"onj\",\"zuy\",\"kux\",\"gat\",\"kjk\",\"kqo\",\"ota\",\"wuv\",\"kvv\",\"chw\",\"lhh\",\"abf\",\"gug\",\"shz\",\"drb\",\"bvh\",\"sbo\",\"bvc\",\"run\",\"wln\",\"lld\",\"mir\",\"sds\",\"wat\",\"ngr\",\"zme\",\"lmr\",\"bnp\",\"akl\",\"zla\",\"xso\",\"nhw\",\"kvg\",\"quq\",\"kav\",\"sja\",\"bah\",\"bac\",\"bqj\",\"ola\",\"pda\",\"daq\",\"avd\",\"wls\",\"kvi\",\"bur\",\"rus\",\"kdq\",\"nwr\",\"ypo\",\"xbe\",\"wax\",\"sdn\",\"aju\",\"gjn\",\"lab\",\"sta\",\"ich\",\"cbd\",\"klk\",\"cen\",\"snj\",\"ptn\",\"doh\",\"doc\",\"kpu\",\"obo\",\"lsr\",\"caf\",\"kxw\",\"boq\",\"fss\",\"dty\",\"xmo\",\"mei\",\"atu\",\"mka\",\"gda\",\"crl\",\"yey\",\"atb\",\"hba\",\"kbp\",\"dua\",\"tyy\",\"kpb\",\"lau\",\"tlj\",\"suy\",\"mam\",\"jmw\",\"sla\",\"cky\",\"ktk\",\"pln\",\"pug\",\"ykn\",\"nes\",\"rji\",\"rak\",\"sme\",\"xga\",\"mev\",\"alu\",\"bhb\",\"apk\",\"alb\",\"ena\",\"jek\",\"nbd\",\"hrm\",\"bhu\",\"nen\",\"lvu\",\"qva\",\"xwa\",\"xiy\",\"tna\",\"xrw\",\"uon\",\"nrl\",\"pax\",\"ttj\",\"naf\",\"pls\",\"rjg\",\"azj\",\"pui\",\"bja\",\"tlp\",\"laz\",\"nkr\",\"nzi\",\"hol\",\"sac\",\"sqj\",\"sah\",\"sur\",\"bdn\",\"tyr\",\"kbj\",\"agl\",\"atz\",\"gnl\",\"ohu\",\"bds\",\"tbk\",\"zhb\",\"yer\",\"snp\",\"kpz\",\"lfn\",\"wok\",\"amq\",\"dtr\",\"lsy\",\"ebk\",\"kcl\",\"ttp\",\"xir\",\"iff\",\"tdl\",\"hke\",\"mzs\",\"mso\",\"mhl\",\"vgt\",\"gok\",\"svc\",\"ynu\",\"bbo\",\"bhz\",\"iya\",\"aft\",\"alz\",\"zoq\",\"ckr\",\"dmd\",\"ppe\",\"mbe\",\"com\",\"mzn\",\"bse\",\"nbn\",\"ned\",\"ibb\",\"jbk\",\"gab\",\"kuu\",\"vku\",\"bnm\",\"drt\",\"nqq\",\"lda\",\"byf\",\"pld\",\"cdo\",\"bfa\",\"kub\",\"nbs\",\"bsv\",\"spy\",\"gau\",\"mbv\",\"ksh\",\"pmr\",\"ksc\",\"ibu\",\"ndo\",\"kep\",\"hea\",\"yal\",\"xaj\",\"grq\",\"xcy\",\"nxr\",\"phh\",\"auk\",\"cbs\",\"bsg\",\"wau\",\"cze\",\"wab\",\"cbn\",\"uve\",\"bsi\",\"wsr\",\"ppi\",\"yby\",\"mbi\",\"kil\",\"emw\",\"gaz\",\"brk\",\"rob\",\"dmn\",\"sht\",\"smv\",\"pmy\",\"mee\",\"bqm\",\"tmw\",\"spr\",\"jyy\",\"mzd\",\"nnq\",\"shx\",\"lnl\",\"ura\",\"kuz\",\"adh\",\"rou\",\"vkz\",\"beo\",\"dms\",\"osi\",\"kkw\",\"mxa\",\"tek\",\"xnh\",\"twc\",\"smi\",\"lok\",\"twh\",\"jna\",\"tjm\",\"tgh\",\"pio\",\"waz\",\"tgc\",\"ero\",\"cnq\",\"umc\",\"xap\",\"pue\",\"kej\",\"xcr\",\"bdd\",\"smg\",\"tro\",\"zpr\",\"uuu\",\"axl\",\"zmv\",\"azm\",\"ttm\",\"ssi\",\"rwo\",\"vmf\",\"aoi\",\"gue\",\"lpe\",\"kod\",\"kfk\",\"dym\",\"kae\",\"dde\",\"cop\",\"ush\",\"zhx\",\"khy\",\"aog\",\"ssg\",\"oht\",\"omv\",\"fly\",\"mqh\",\"mqc\",\"maj\",\"iml\",\"ayo\",\"tlm\",\"zmi\",\"dsn\",\"tuf\",\"afu\",\"ssv\",\"omg\",\"psy\",\"elm\",\"tsw\",\"xbg\",\"mcy\",\"nop\",\"bpy\",\"dzo\",\"rma\",\"zmg\",\"nca\",\"paz\",\"snm\",\"hin\",\"afb\",\"xbi\",\"gmy\",\"kve\",\"omi\",\"coj\",\"att\",\"kgw\",\"flr\",\"lui\",\"avn\",\"opy\",\"lls\",\"khr\",\"lax\",\"bmg\",\"kpt\",\"prq\",\"avs\",\"lln\",\"hrp\",\"kra\",\"lat\",\"zpy\",\"kpx\",\"acw\",\"mvp\",\"ghc\",\"ghh\",\"bmi\",\"nyh\",\"nyc\",\"tzx\",\"kww\",\"tiq\",\"atx\",\"lug\",\"aan\",\"gld\",\"rpn\",\"mnc\",\"mnh\",\"bhx\",\"ukw\",\"alt\",\"luv\",\"ium\",\"gmr\",\"pau\",\"lts\",\"seo\",\"ltn\",\"kbm\",\"noj\",\"pab\",\"srk\",\"bpr\",\"mcr\",\"sqm\",\"aas\",\"afz\",\"bht\",\"bmv\",\"alx\",\"map\",\"psr\",\"kmh\",\"kmc\",\"bbu\",\"bih\",\"nhd\",\"yno\",\"gyd\",\"kwn\",\"kgs\",\"kiv\",\"msu\",\"ayt\",\"ifa\",\"lsm\",\"swq\",\"acn\",\"bkq\",\"diq\",\"moa\",\"clc\",\"sml\",\"clh\",\"mar\",\"acs\",\"psp\",\"mcp\",\"noy\",\"bpp\",\"inm\",\"nre\",\"msb\",\"kws\",\"kgn\",\"bbb\",\"muh\",\"muc\",\"pcb\",\"uks\",\"khp\",\"git\",\"unz\",\"bjf\",\"lng\",\"kii\",\"zsl\",\"isi\",\"aid\",\"tfr\",\"yon\",\"jjr\",\"peh\",\"ctc\",\"cre\",\"rnd\",\"cth\",\"zpj\",\"xtq\",\"isg\",\"lyn\",\"kig\",\"lni\",\"chd\",\"nlc\",\"mvr\",\"gix\",\"pgk\",\"xmz\",\"aaw\",\"jdg\",\"alo\",\"yux\",\"slf\",\"wne\",\"nor\",\"yai\",\"jru\",\"tde\",\"mjq\",\"may\",\"set\",\"mcj\",\"mhe\",\"bpj\",\"ffm\",\"nkm\",\"xgf\",\"mrd\",\"enf\",\"dtm\",\"yag\",\"msz\",\"mbl\",\"yut\",\"ppl\",\"bzk\",\"irk\",\"jrb\",\"bsl\",\"bho\",\"naa\",\"ato\",\"idd\",\"xmu\",\"tzo\",\"stf\",\"hiw\",\"thk\",\"yav\",\"obu\",\"ess\",\"hoe\",\"caa\",\"mdk\",\"kpo\",\"zpp\",\"age\",\"mvy\",\"gdf\",\"mkf\",\"tss\",\"lao\",\"dlm\",\"gne\",\"cor\",\"tsn\",\"awe\",\"duf\",\"zbu\",\"ckm\",\"xmb\",\"esn\",\"kce\",\"had\",\"khj\",\"jaq\",\"unu\",\"xsu\",\"rzh\",\"uiv\",\"hlu\",\"ndt\",\"cuq\",\"ems\",\"pne\",\"zum\",\"ypb\",\"tms\",\"rkt\",\"oum\",\"jua\",\"aqk\",\"xar\",\"mlq\",\"tmn\",\"dmw\",\"key\",\"xsb\",\"emn\",\"arm\",\"ndx\",\"ugn\",\"hlb\",\"djm\",\"dgc\",\"dgh\",\"htu\",\"knz\",\"kqb\",\"spp\",\"kks\",\"sbb\",\"dro\",\"lul\",\"bgq\",\"pmj\",\"sih\",\"nuq\",\"sbu\",\"mtq\",\"bwq\",\"uig\",\"dek\",\"liz\",\"skq\",\"mmz\",\"kqu\",\"bml\",\"kkn\",\"aba\",\"vun\",\"trx\",\"xay\",\"pqe\",\"imi\",\"zml\",\"tpa\",\"nif\",\"ake\",\"mgf\",\"ycl\",\"ker\",\"kza\",\"xhe\",\"ypz\",\"vot\",\"ggk\",\"pix\",\"mwf\",\"ank\",\"btf\",\"ert\",\"xrd\",\"pit\",\"gec\",\"oml\",\"geh\",\"ybj\",\"trt\",\"ljw\",\"cjh\",\"nbw\",\"bex\",\"knb\",\"mmu\",\"kqz\",\"kxd\",\"gsp\",\"wec\",\"liu\",\"ngm\",\"cbw\",\"lbc\",\"weh\",\"xkf\",\"mim\",\"aol\",\"njh\",\"sho\",\"xdk\",\"sbz\",\"ssl\",\"blf\",\"hmq\",\"axg\",\"nwm\",\"jmd\",\"bum\",\"lib\",\"mmb\",\"knu\",\"bet\",\"pdt\",\"ijo\",\"wor\",\"nne\",\"boi\",\"jks\",\"gva\",\"dtk\",\"meq\",\"ebr\",\"mxz\",\"kly\",\"sjt\",\"zas\",\"bzm\",\"bxl\",\"olt\",\"tbr\",\"lrv\",\"heb\",\"kyj\",\"suk\",\"bog\",\"nkk\",\"yro\",\"fod\",\"waa\",\"ndf\",\"apy\",\"ldb\",\"bov\",\"ray\",\"vam\",\"txn\",\"dlk\",\"koc\",\"koh\",\"bfu\",\"cdf\",\"kty\",\"nmj\",\"ztx\",\"byo\",\"gaa\",\"lri\",\"iba\",\"ott\",\"gor\",\"thm\",\"ztt\",\"dbp\",\"kua\",\"bfb\",\"hup\",\"vka\",\"otx\",\"puq\",\"urz\",\"lrg\",\"mye\",\"rbl\",\"hch\",\"mdm\",\"txs\",\"lhn\",\"trf\",\"amg\",\"bvn\",\"tal\",\"glh\",\"glc\",\"mxb\",\"mnd\",\"mwt\",\"btt\",\"nix\",\"pif\",\"ofu\",\"hya\",\"mgt\",\"btx\",\"kyp\",\"nit\",\"mxu\",\"woy\",\"ami\",\"lxm\",\"lzl\",\"kmd\",\"tby\",\"wth\",\"ike\",\"bcw\",\"mpw\",\"lhs\",\"klr\",\"bef\",\"udg\",\"xzh\",\"sdh\",\"goy\",\"uru\",\"sdc\",\"ozm\",\"jer\",\"roa\",\"xkx\",\"gre\",\"huj\",\"blt\",\"tvl\",\"ban\",\"dbj\",\"xkt\",\"oog\",\"mfl\",\"blx\",\"nyd\",\"aht\",\"udi\",\"amv\",\"bas\",\"bfz\",\"rar\",\"pwm\",\"apr\",\"nmp\",\"urb\",\"wlh\",\"wlc\",\"ruc\",\"ruh\",\"rsb\",\"tkd\",\"lkj\",\"jii\",\"rmu\",\"gwm\",\"tqt\",\"phd\",\"ncu\",\"anm\",\"sfb\",\"dos\",\"vwa\",\"wif\",\"cgk\",\"uly\",\"fub\",\"iyx\",\"kjr\",\"rim\",\"fuu\",\"poy\",\"sov\",\"tow\",\"asv\",\"jig\",\"nsp\",\"afa\",\"tuo\",\"ncb\",\"pre\",\"rmb\",\"don\",\"pth\",\"lew\",\"syo\",\"wbw\",\"bjx\",\"jiv\",\"csp\",\"asg\",\"buk\",\"lcf\",\"sog\",\"neh\",\"nec\",\"gbw\",\"soi\",\"lmk\",\"ykh\",\"dno\",\"asi\",\"ksd\",\"bjt\",\"yiq\",\"xdm\",\"plc\",\"plh\",\"mik\",\"sxl\",\"ngk\",\"krz\",\"tnt\",\"twd\",\"slx\",\"guq\",\"yuf\",\"aqm\",\"wdt\",\"sas\",\"ylm\",\"por\",\"tci\",\"ark\",\"xxb\",\"xnd\",\"xwt\",\"dav\",\"ilo\",\"umd\",\"enx\",\"sef\",\"bdc\",\"fil\",\"bdh\",\"quv\",\"tnx\",\"kjy\",\"slt\",\"kaq\",\"ncz\",\"tgd\",\"paa\",\"san\",\"rmz\",\"xpw\",\"npl\",\"tcg\",\"kru\",\"dem\",\"kdg\",\"dag\",\"mzc\",\"mzh\",\"scw\",\"svs\",\"dut\",\"mkx\",\"gdx\",\"qug\",\"ito\",\"dai\",\"add\",\"kvq\",\"kdi\",\"dux\",\"mkt\",\"gdt\",\"djk\",\"qui\",\"stt\",\"utr\",\"ige\",\"csj\",\"krb\",\"mfg\",\"tav\",\"saw\",\"xpn\",\"caz\",\"mhq\",\"tdq\",\"tvi\",\"mje\",\"dgd\",\"yss\",\"vmx\",\"dnt\",\"ysn\",\"izm\",\"ixl\",\"udl\",\"brm\",\"mfi\",\"utp\",\"xps\",\"sid\",\"csy\",\"bjo\",\"bqk\",\"tai\",\"agq\",\"syx\",\"yde\",\"oke\",\"tux\",\"tqo\",\"znk\",\"jod\",\"pop\",\"ifb\",\"ree\",\"nsy\",\"scs\",\"bba\",\"kcq\",\"lkr\",\"jae\",\"pca\",\"naz\",\"tem\",\"xud\",\"aml\",\"iyo\",\"scn\",\"mfv\",\"tag\",\"ifu\",\"onk\",\"mou\",\"tut\",\"kjj\",\"lom\",\"gnq\",\"len\",\"wed\",\"njd\",\"vrs\",\"dow\",\"cab\",\"lrl\",\"duo\",\"zba\",\"bxv\",\"swe\",\"hbo\",\"ton\",\"csr\",\"itx\",\"kxc\",\"laf\",\"una\",\"kxh\",\"sge\",\"vel\",\"mko\",\"gdo\",\"xma\",\"kpf\",\"jbm\",\"itt\",\"tos\",\"cau\",\"fer\",\"les\",\"jmc\",\"wbs\",\"xle\",\"sto\",\"bnk\",\"bhf\",\"moz\",\"kjp\",\"crq\",\"bol\",\"tno\",\"lky\",\"uzb\",\"bxi\",\"qvo\",\"xwo\",\"gbs\",\"eno\",\"hdn\",\"nau\",\"ian\",\"fnb\",\"ged\",\"gbn\",\"nab\",\"jpx\",\"jra\",\"orm\",\"hds\",\"nsr\",\"aum\",\"slo\",\"bxg\",\"xte\",\"alf\",\"pze\",\"goj\",\"sba\",\"pbn\",\"akq\",\"ped\",\"oge\",\"ctd\",\"bcn\",\"mpn\",\"byt\",\"dby\",\"oto\",\"kqa\",\"huy\",\"cpi\",\"aic\",\"aih\",\"abu\",\"ktp\",\"byx\",\"abb\",\"lgr\",\"kdl\",\"hnj\",\"dal\",\"rap\",\"cpg\",\"app\",\"nmr\",\"bvw\",\"bcs\",\"mps\",\"pbs\",\"chh\",\"chc\",\"qul\",\"srm\",\"kzz\",\"juu\",\"ypa\",\"pdo\",\"kbk\",\"cld\",\"nhh\",\"bid\",\"nhc\",\"olo\",\"tpz\",\"cmr\",\"sjo\",\"sqk\",\"tcl\",\"npg\",\"klp\",\"ntd\",\"baw\",\"xsa\",\"hla\",\"mud\",\"npi\",\"jub\",\"hme\",\"tbj\",\"kyr\",\"raj\",\"apj\",\"kna\",\"zaw\",\"idc\",\"pnq\",\"ktj\",\"nmy\",\"blo\",\"sxg\",\"aho\",\"abz\",\"asl\",\"xko\",\"hac\",\"ttk\",\"mma\",\"hah\",\"sol\",\"lia\",\"mle\",\"kfm\",\"hur\",\"dbr\",\"gop\",\"kzb\",\"tbp\",\"bge\",\"dhm\",\"jil\",\"kyy\",\"yms\",\"klj\",\"snk\",\"mwo\",\"bto\",\"ymn\",\"tlk\",\"ske\",\"mgo\",\"nio\",\"tpu\",\"wca\",\"mrc\",\"mte\",\"bwe\",\"mrh\",\"kzu\",\"elk\",\"hvc\",\"nue\",\"kwv\",\"dml\",\"hve\",\"siq\",\"nuh\",\"ify\",\"nuc\",\"mre\",\"mtc\",\"bwc\",\"nsb\",\"bwh\",\"yxl\",\"mth\",\"lyg\",\"isn\",\"kin\",\"skc\",\"moy\",\"bek\",\"skh\",\"acv\",\"yoi\",\"nar\",\"lns\",\"ukg\",\"rmp\",\"noa\",\"tdd\",\"bro\",\"lnn\",\"wnd\",\"uki\",\"nsu\",\"kgv\",\"yog\",\"mhd\",\"kis\",\"bjm\",\"bgc\",\"erk\",\"kwi\",\"awd\",\"ant\",\"mlc\",\"mlh\",\"coa\",\"tqm\",\"hhy\",\"bmw\",\"gwt\",\"gnd\",\"kcd\",\"hae\",\"trk\",\"aci\",\"rit\",\"kgg\",\"csu\",\"krj\",\"csb\",\"teo\",\"hod\",\"luw\",\"cuh\",\"cuc\",\"ide\",\"kwg\",\"ukv\",\"agd\",\"car\",\"kgi\",\"anx\",\"ggt\",\"loo\",\"ccp\",\"gwx\",\"tsg\",\"hmc\",\"omw\",\"fuj\",\"maa\",\"hmh\",\"nsz\",\"nbl\",\"ljl\",\"xbw\",\"esg\",\"dum\",\"lkb\",\"gdm\",\"mkm\",\"lku\",\"ncj\",\"jbo\",\"nhe\",\"tsi\",\"gye\",\"lbq\",\"nay\",\"stm\",\"zmw\",\"mor\",\"esi\",\"aqt\",\"ccj\",\"che\",\"tnm\",\"zgh\",\"yan\",\"geq\",\"mva\",\"zro\",\"qvm\",\"enm\",\"krp\",\"cay\",\"pkt\",\"hra\",\"xgm\",\"hhr\",\"aie\",\"nkf\",\"csz\",\"cbl\",\"ssw\",\"ndk\",\"oro\",\"tsv\",\"crd\",\"ogc\",\"hgw\",\"auo\",\"slm\",\"efa\",\"yas\",\"pzh\",\"xtc\",\"kyz\",\"xre\",\"mdx\",\"xth\",\"mxj\",\"nlq\",\"kkg\",\"otm\",\"jnj\",\"avl\",\"tpr\",\"smw\",\"lll\",\"juy\",\"ztm\",\"tht\",\"tmv\",\"mdt\",\"xhd\",\"uis\",\"akd\",\"peq\",\"vls\",\"djf\",\"kki\",\"kea\",\"kzr\",\"xlc\",\"dbb\",\"sro\",\"tmg\",\"pye\",\"hsh\",\"nmz\",\"hub\",\"aal\",\"bfp\",\"bzt\",\"dpp\",\"emg\",\"olm\",\"yuk\",\"lgz\",\"sgc\",\"sgh\",\"sjm\",\"muq\",\"bzx\",\"irx\",\"biq\",\"ldp\",\"swh\",\"tmi\",\"zlm\",\"swc\",\"urj\",\"huu\",\"sek\",\"bkh\",\"aby\",\"bkc\",\"dic\",\"dbu\",\"dih\",\"kkv\",\"emi\",\"kyb\",\"cim\",\"mbw\",\"jur\",\"tpy\",\"zkh\",\"rgk\",\"xaa\",\"bsw\",\"kzy\",\"buf\",\"jah\",\"jac\",\"blm\",\"ahm\",\"pnd\",\"mif\",\"ngf\",\"kfo\",\"lmf\",\"mxp\",\"xor\",\"kyu\",\"rwk\",\"okc\",\"okh\",\"wik\",\"dsl\",\"dho\",\"nmb\",\"huz\",\"imn\",\"pfl\",\"lgb\",\"urp\",\"abr\",\"qwa\",\"lwu\",\"ldj\",\"mwm\",\"btm\",\"mgm\",\"ayk\",\"lgu\",\"nim\",\"mjc\",\"mjh\",\"nmu\",\"bfj\",\"ims\",\"hil\",\"hkn\",\"abp\",\"nbv\",\"cbg\",\"ktb\",\"bss\",\"pps\",\"srt\",\"mbs\",\"urr\",\"apb\",\"rab\",\"jeu\",\"goz\",\"iro\",\"lvk\",\"bzo\",\"mqq\",\"apu\",\"cbi\",\"jeb\",\"rau\",\"alk\",\"uha\",\"spa\",\"bsn\",\"hks\",\"mze\",\"yrm\",\"mbn\",\"bfy\",\"srx\",\"med\",\"ktu\",\"ppn\",\"lji\",\"nbi\",\"klb\",\"vao\",\"tpj\",\"bde\",\"bym\",\"pud\",\"jup\",\"tsl\",\"tbz\",\"cbv\",\"nbg\",\"xop\",\"bnf\",\"mxr\",\"atk\",\"yba\",\"esl\",\"gko\",\"kpk\",\"klu\",\"ple\",\"ktz\",\"hnu\",\"dmv\",\"kwl\",\"apz\",\"jya\",\"oym\",\"smn\",\"bfr\",\"raz\",\"acl\",\"gob\",\"pma\",\"kmq\",\"sms\",\"gou\",\"ury\",\"nzd\",\"mnq\",\"nee\",\"ynk\",\"kgl\",\"abj\",\"wfg\",\"dhx\",\"prc\",\"mxy\",\"prh\",\"klz\",\"kfx\",\"ebu\",\"oss\",\"tkq\",\"wsa\",\"nyq\",\"tbu\",\"tih\",\"yol\",\"tic\",\"dmg\",\"her\",\"kzp\",\"yxg\",\"bqf\",\"fse\",\"pwo\",\"nxa\",\"tpp\",\"jqr\",\"kft\",\"wob\",\"osn\",\"ukl\",\"zmn\",\"krr\",\"lox\",\"snf\",\"wle\",\"ano\",\"rue\",\"dsi\",\"wma\",\"caj\",\"ycn\",\"hig\",\"tex\",\"grh\",\"grc\",\"tet\",\"tum\",\"xbn\",\"hii\",\"xed\",\"sde\",\"utu\",\"zms\",\"fla\",\"lot\",\"omn\",\"phq\",\"tlf\",\"kha\",\"sym\",\"yid\",\"wrh\",\"fuy\",\"ulb\",\"ikh\",\"wgo\",\"mca\",\"ssn\",\"bpa\",\"psa\",\"brx\",\"aon\",\"ksq\",\"sss\",\"kjz\",\"vmm\",\"mop\",\"wwo\",\"xxr\",\"rmy\",\"gle\",\"aos\",\"brt\",\"gma\",\"ulu\",\"yaw\",\"xdo\",\"shk\",\"ttf\",\"naj\",\"kad\",\"ddd\",\"aai\",\"tgq\",\"ltg\",\"aux\",\"myh\",\"myc\",\"ylo\",\"cnc\",\"pko\",\"avv\",\"lnw\",\"cnh\",\"orx\",\"opa\",\"ilm\",\"tml\",\"aem\",\"zpa\",\"lti\",\"ort\",\"ccr\",\"aag\",\"xnq\",\"kry\",\"cap\",\"hpo\",\"twq\",\"kiw\",\"koe\",\"gud\",\"aut\",\"nap\",\"aav\",\"bms\",\"ncr\",\"llg\",\"adq\",\"avi\",\"lun\",\"wud\",\"gpn\",\"kvd\",\"kju\",\"moj\",\"lus\",\"jbt\",\"itm\",\"kjb\",\"lli\",\"fur\",\"kkl\",\"bmn\",\"poz\",\"fah\",\"nnc\",\"nnh\",\"lof\",\"bvl\",\"scg\",\"xpv\",\"xmr\",\"hrz\",\"tan\",\"quw\",\"mvz\",\"sgd\",\"lhl\",\"tlt\",\"obr\",\"snx\",\"lso\",\"xld\",\"cob\",\"daw\",\"tef\",\"kdw\",\"lzn\",\"bkd\",\"did\",\"lbe\",\"cou\",\"ino\",\"sci\",\"elx\",\"tas\",\"nhq\",\"unr\",\"tlx\",\"csa\",\"cje\",\"evn\",\"xpi\",\"nou\",\"jrr\",\"ysg\",\"ttt\",\"azt\",\"xtd\",\"nsa\",\"tvn\",\"afp\",\"bal\",\"fkv\",\"cfg\",\"brf\",\"maz\",\"mfs\",\"chq\",\"ett\",\"msy\",\"xhc\",\"tvs\",\"tcw\",\"mfn\",\"akc\",\"bby\",\"akh\",\"xpg\",\"etx\",\"scv\",\"wxa\",\"nob\",\"aiq\",\"alm\",\"lev\",\"wbv\",\"mvb\",\"sqx\",\"gbg\",\"auf\",\"suo\",\"asw\",\"mrq\",\"bxn\",\"eyo\",\"coz\",\"kbx\",\"yrk\",\"nko\",\"sow\",\"sie\",\"tov\",\"jvd\",\"bxs\",\"dto\",\"gbi\",\"iai\",\"zal\",\"kbt\",\"xmy\",\"ivv\",\"mvu\",\"sqt\",\"mjd\",\"bhm\",\"dge\",\"hru\",\"atm\",\"tzm\",\"lka\",\"wbi\",\"lei\",\"bbr\",\"haq\",\"jad\",\"pap\",\"msr\",\"zkd\",\"uya\",\"kpm\",\"mab\",\"toi\",\"lam\",\"mau\",\"gbv\",\"okd\",\"ydd\",\"tog\",\"cko\",\"pnc\",\"pnh\",\"byk\",\"noz\",\"kcc\",\"nps\",\"kch\",\"bcv\",\"mpv\",\"pbv\",\"aek\",\"mmr\",\"sby\",\"ilk\",\"vkp\",\"awh\",\"lir\",\"awc\",\"kup\",\"dba\",\"gnh\",\"gnc\",\"kqy\",\"gap\",\"agc\",\"agh\",\"fin\",\"gcr\",\"oie\",\"sal\",\"hoh\",\"npn\",\"hoc\",\"knr\",\"aro\",\"srf\",\"wap\",\"djo\",\"skd\",\"mpi\",\"bci\",\"pbi\",\"itk\",\"amw\",\"nud\",\"bnt\",\"bwd\",\"mtd\",\"cpn\",\"keu\",\"pbg\",\"bcg\",\"keb\",\"mpg\",\"bnx\",\"xsy\",\"mhc\",\"cps\",\"bgd\",\"tdc\",\"gvp\",\"tdh\",\"wnc\",\"oci\",\"qxt\",\"nle\",\"ymg\",\"lga\",\"syk\",\"ont\",\"nma\",\"kny\",\"gaj\",\"tuk\",\"sbr\",\"onx\",\"pee\",\"mmy\",\"crc\",\"ymi\",\"cte\",\"crh\",\"vkj\",\"kuj\",\"rop\",\"dol\",\"kqr\",\"liy\",\"icl\",\"lwa\",\"sxs\",\"xsr\",\"mue\",\"dnk\",\"nrc\",\"nte\",\"vmk\",\"gvj\",\"lmo\",\"kff\",\"mio\",\"xab\",\"hmd\",\"shm\",\"kya\",\"bqt\",\"kez\",\"cle\",\"nwo\",\"buo\",\"xau\",\"bqx\",\"frp\",\"sxn\",\"waj\",\"bie\",\"bow\",\"kxq\",\"cma\",\"gvy\",\"kme\",\"xcb\",\"mdf\",\"wbl\",\"lel\",\"gdk\",\"mkk\",\"txi\",\"oav\",\"yih\",\"bno\",\"tol\",\"stk\",\"thf\",\"plq\",\"lrn\",\"kla\",\"xsp\",\"txg\",\"frr\",\"zav\",\"ven\",\"ybb\",\"cdm\",\"neq\",\"way\",\"vaf\",\"ypp\",\"xcu\",\"duk\",\"mne\",\"bon\",\"pmz\",\"ndm\",\"arx\",\"zag\",\"eke\",\"cqd\",\"ghe\",\"apa\",\"sxw\",\"raa\",\"nye\",\"zem\",\"spu\",\"knj\",\"gay\",\"tke\",\"iby\",\"ptq\",\"kta\",\"kuy\",\"tnk\",\"bzf\",\"gbl\",\"spb\",\"zai\",\"rkm\",\"kqp\",\"wdk\",\"mmj\",\"art\",\"lij\",\"bos\",\"ror\",\"xwk\",\"grd\",\"sbp\",\"kvh\",\"kvc\",\"zos\",\"wuh\",\"fry\",\"oon\",\"xpl\",\"bvv\",\"bux\",\"bqo\",\"bjk\",\"fiw\",\"bai\",\"war\",\"ngt\",\"nwx\",\"mit\",\"but\",\"woa\",\"yzk\",\"xsj\",\"ysl\",\"fad\",\"ngx\",\"nnd\",\"mzq\",\"gvr\",\"mix\",\"wsu\",\"lmx\",\"bag\",\"tba\",\"mqe\",\"bem\",\"oos\",\"pmb\",\"rth\",\"cwt\",\"goa\",\"rtc\",\"sbj\",\"lhi\",\"qxo\",\"kur\",\"lip\",\"bvi\",\"bdq\",\"kah\",\"kac\",\"kqj\",\"mmp\",\"ono\",\"ams\",\"bav\",\"nfl\",\"scl\",\"tjo\",\"bvg\",\"guc\",\"pim\",\"guh\",\"hna\",\"knp\",\"amn\",\"ibr\",\"gar\",\"trm\",\"azo\",\"tto\",\"xkk\",\"jwi\",\"ahk\",\"gim\",\"puc\",\"mcb\",\"gmu\",\"par\",\"sos\",\"ula\",\"rwm\",\"eto\",\"msp\",\"pcp\",\"ass\",\"bbp\",\"tge\",\"dyo\",\"son\",\"twe\",\"gmb\",\"koq\",\"asn\",\"bxw\",\"cik\",\"lcm\",\"blk\",\"mcu\",\"bpu\",\"psu\",\"ewe\",\"dov\",\"nik\",\"lst\",\"mgk\",\"aym\",\"tlo\",\"ade\",\"wmb\",\"uta\",\"elo\",\"khu\",\"rif\",\"khb\",\"int\",\"xmj\",\"meh\",\"yml\",\"mec\",\"wim\",\"btk\",\"mwk\",\"anf\",\"sno\",\"ywq\",\"gwf\",\"doi\",\"zpz\",\"vgr\",\"qus\",\"pbl\",\"mpl\",\"bcl\",\"afr\",\"uvh\",\"bpz\",\"mcz\",\"ckx\",\"das\",\"prd\",\"ruq\",\"czh\",\"pcj\",\"kdn\",\"dan\",\"bbj\",\"gmz\",\"sdq\",\"msj\",\"qun\",\"kja\",\"def\",\"otk\",\"taw\",\"sav\",\"ckt\",\"pay\",\"uur\",\"iow\",\"zpu\",\"sut\",\"kse\",\"tyt\",\"sag\",\"nkt\",\"tcn\",\"mfw\",\"sem\",\"xmp\",\"tcs\",\"nkx\",\"kbo\",\"khz\",\"yet\",\"tvw\",\"dtt\",\"ecs\",\"sqo\",\"sjk\",\"sux\",\"yum\",\"olk\",\"tyx\",\"sai\",\"zpb\",\"igs\",\"bsh\",\"phg\",\"bsc\",\"nwb\",\"kfa\",\"bub\",\"lim\",\"ngu\",\"mbc\",\"mbh\",\"miu\",\"ksv\",\"mmm\",\"lmu\",\"dsq\",\"kyf\",\"sve\",\"jid\",\"pip\",\"phi\",\"knm\",\"lmb\",\"ngb\",\"nxx\",\"buu\",\"xao\",\"ign\",\"mib\",\"iws\",\"hkh\",\"trp\",\"ndj\",\"sod\",\"ihp\",\"jcs\",\"ksi\",\"iii\",\"zil\",\"cdj\",\"nmf\",\"arz\",\"rcf\",\"pmx\",\"xvo\",\"ksg\",\"mjw\",\"shy\",\"cwb\",\"pmt\",\"phv\",\"sae\",\"rmk\",\"nck\",\"bep\",\"trj\",\"mul\",\"umi\",\"ndp\",\"keo\",\"buz\",\"xtw\",\"twg\",\"xng\",\"tgi\",\"aaq\",\"dju\",\"pij\",\"smc\",\"djb\",\"cll\",\"twi\",\"smh\",\"kqm\",\"xct\",\"umg\",\"krk\",\"tcd\",\"ybx\",\"ngz\",\"miz\",\"jge\",\"bil\",\"sbm\",\"xni\",\"prs\",\"adi\",\"dad\",\"spt\",\"bej\",\"oub\",\"kdd\",\"shr\",\"osc\",\"nll\",\"sgw\",\"qud\",\"ipo\",\"xxk\",\"huf\",\"tgv\",\"arb\",\"xsm\",\"tis\",\"dbf\",\"aru\",\"ctl\",\"pel\",\"sww\",\"tin\",\"adg\",\"dry\",\"spx\",\"diw\",\"prn\",\"doe\",\"llq\",\"sra\",\"ypm\",\"bkw\",\"ice\",\"xbc\",\"ldk\",\"hmw\",\"omc\",\"gel\",\"bhy\",\"wij\",\"obm\",\"bae\",\"foi\",\"ckb\",\"xmm\",\"sep\",\"grn\",\"mqg\",\"unm\",\"bfk\",\"grs\",\"ych\",\"cku\",\"mqi\",\"aly\",\"zmc\",\"zmh\",\"cbq\",\"mao\",\"xib\",\"yup\",\"usi\",\"nku\",\"dtb\",\"bve\",\"rri\",\"iks\",\"wrs\",\"hwc\",\"inz\",\"aua\",\"nqn\",\"ved\",\"lay\",\"rdb\",\"tfo\",\"zpt\",\"nbq\",\"ora\",\"tyu\",\"ssc\",\"ssh\",\"zpx\",\"hro\",\"kpy\",\"mvo\",\"aoc\",\"njl\",\"lbl\",\"opt\",\"wel\",\"zra\",\"sub\",\"iqu\",\"mqv\",\"aty\",\"yeu\",\"nkb\",\"wrn\",\"dtu\",\"mtw\",\"ayp\",\"wpc\",\"bww\",\"txe\",\"nyv\",\"yuj\",\"xul\",\"kmi\",\"whg\",\"mys\",\"nuw\",\"cns\",\"ckz\",\"nsf\",\"mcx\",\"bpx\",\"mng\",\"skw\",\"alr\",\"bra\",\"tkv\",\"bhr\",\"kmg\",\"mct\",\"sej\",\"bpt\",\"pst\",\"gmx\",\"rsk\",\"bgw\",\"mni\",\"myn\",\"urk\",\"mlw\",\"nyi\",\"bmh\",\"loa\",\"bmc\",\"gip\",\"nnn\",\"kpr\",\"fan\",\"inb\",\"kht\",\"kmv\",\"eki\",\"msm\",\"wmx\",\"sil\",\"bbm\",\"mxk\",\"atr\",\"pcm\",\"tea\",\"wmt\",\"luc\",\"tyz\",\"zae\",\"ekg\",\"cuw\",\"nyg\",\"suz\",\"lsb\",\"ood\",\"tkg\",\"mnv\",\"coo\",\"lcp\",\"csf\",\"nkz\",\"dgl\",\"lar\",\"khx\",\"ctg\",\"kbb\",\"nli\",\"xok\",\"myw\",\"peg\",\"atp\",\"nus\",\"rwr\",\"mts\",\"bws\",\"bgn\",\"adl\",\"pam\",\"muv\",\"sks\",\"gir\",\"mvx\",\"snz\",\"cnw\",\"tft\",\"lnh\",\"hrx\",\"zpo\",\"mvt\",\"lap\",\"skn\",\"opo\",\"squ\",\"pei\",\"pka\",\"fcs\",\"nlg\",\"kkq\",\"yme\",\"hrt\",\"aqa\",\"biv\",\"nun\",\"yla\",\"juk\",\"isc\",\"kih\",\"kbu\",\"ish\",\"kic\",\"mtn\",\"bwn\",\"bgs\",\"rgr\",\"nlv\",\"nti\",\"max\",\"alp\",\"mls\",\"egl\",\"yuy\",\"mui\",\"dkg\",\"tgl\",\"ayr\",\"big\",\"mug\",\"azz\",\"ttz\",\"sey\",\"cus\",\"pev\",\"abk\",\"emq\",\"ntg\",\"cli\",\"twl\",\"wir\",\"yta\",\"nnw\",\"etz\",\"mat\",\"bhp\",\"mln\",\"tmq\",\"ksl\",\"kbz\",\"kho\",\"hms\",\"gga\",\"kzk\",\"cox\",\"elu\",\"tpk\",\"laj\",\"tlu\",\"grw\",\"ria\",\"tlb\",\"wmo\",\"kpj\",\"snu\",\"hmn\",\"ana\",\"giy\",\"cot\",\"tzj\",\"atj\",\"gwa\",\"ikw\",\"bce\",\"dyb\",\"mpe\",\"bhj\",\"wrw\",\"pbe\",\"ser\",\"sza\",\"pzn\",\"xda\",\"etu\",\"not\",\"ulf\",\"vif\",\"wiy\",\"ttu\",\"uum\",\"wwa\",\"pso\",\"fwe\",\"azb\",\"phl\",\"yur\",\"ttb\",\"wga\",\"bpo\",\"mco\",\"pof\",\"etb\",\"yah\",\"zgn\",\"dyu\",\"yac\",\"alj\",\"xts\",\"onu\",\"ekl\",\"gof\",\"nyl\",\"gso\",\"mok\",\"mfd\",\"bey\",\"ghl\",\"tkl\",\"sii\",\"qxu\",\"dgg\",\"ifk\",\"shp\",\"ukq\",\"pmo\",\"gzn\",\"qwt\",\"gbe\",\"cdr\",\"frm\",\"xtn\",\"tvd\",\"dgi\",\"ubu\",\"tju\",\"sig\",\"onb\",\"xls\",\"kml\",\"bkn\",\"hss\",\"acq\",\"prw\",\"nxo\",\"try\",\"din\",\"lee\",\"wbe\",\"bqu\",\"tiw\",\"zza\",\"siv\",\"pwa\",\"piy\",\"sgs\",\"jog\",\"swn\",\"ndr\",\"tad\",\"pga\",\"wof\",\"kgq\",\"rom\",\"xug\",\"sgn\",\"sws\",\"hsn\",\"bnz\",\"bqb\",\"xln\",\"bks\",\"dis\",\"mnl\",\"tbf\",\"xat\",\"gev\",\"wam\",\"cjv\",\"okn\",\"igw\",\"raf\",\"spo\",\"weg\",\"apf\",\"lbg\",\"ren\",\"jas\",\"ktf\",\"cdy\",\"wka\",\"ber\",\"bza\",\"lbi\",\"res\",\"wei\",\"ira\",\"shj\",\"sce\",\"nji\",\"nak\",\"esq\",\"jan\",\"zkn\",\"gvm\",\"oks\",\"tsq\",\"mda\",\"gka\",\"gei\",\"cak\",\"bnu\",\"cji\",\"ybo\",\"vor\",\"xpe\",\"ket\",\"kum\",\"vkm\",\"mjn\",\"klf\",\"tha\",\"trr\",\"lbv\",\"ibm\",\"gam\",\"xco\",\"mjs\",\"geg\",\"ndy\",\"err\",\"vaa\",\"bnb\",\"bqz\",\"bxd\",\"pir\",\"jvn\",\"mql\",\"kex\",\"lwo\",\"gox\",\"cga\",\"otr\",\"jml\",\"aae\",\"kog\",\"yis\",\"xky\",\"dhu\",\"vec\",\"srz\",\"kzm\",\"yin\",\"byj\",\"nmo\",\"bly\",\"crw\",\"lrc\",\"ssd\",\"cwa\",\"ciy\",\"got\",\"tpm\",\"lgo\",\"aod\",\"afk\",\"koi\",\"sjr\",\"cmo\",\"ywg\",\"tbt\",\"olr\",\"bua\",\"kfb\",\"xbd\",\"ave\",\"mgy\",\"nwa\",\"ygi\",\"niy\",\"kyo\",\"xes\",\"zmd\",\"kfu\",\"nga\",\"zyp\",\"bty\",\"doq\",\"lle\",\"mia\",\"lma\",\"boh\",\"tbx\",\"ijj\",\"kov\",\"wli\",\"ara\",\"blr\",\"zoh\",\"dse\",\"jet\",\"zoc\",\"rui\",\"kcw\",\"apx\",\"rax\",\"pfe\",\"sdg\",\"gnw\",\"bmd\",\"oua\",\"jum\",\"ixc\",\"sru\",\"cir\",\"ktx\",\"aww\",\"zty\",\"byp\",\"srb\",\"dbo\",\"pak\",\"glv\",\"oty\",\"rug\",\"agw\",\"huo\",\"xom\",\"ktt\",\"wlg\",\"wun\",\"lud\",\"kvn\",\"xkr\",\"apt\",\"rat\",\"how\",\"ahr\",\"wti\",\"mwr\",\"wlv\",\"rts\",\"btr\",\"abm\",\"glg\",\"gun\",\"lpn\",\"kfz\",\"kas\",\"zyj\",\"dds\",\"amc\",\"amh\",\"klx\",\"ddn\",\"mhw\",\"gus\",\"kan\",\"saq\",\"klt\",\"nir\",\"mgr\",\"wnw\",\"kef\",\"dja\",\"sdv\",\"tez\",\"nam\",\"ptv\",\"sty\",\"pli\",\"neg\",\"pus\",\"soh\",\"soc\",\"mky\",\"mvf\",\"yki\",\"dnj\",\"hal\",\"aub\",\"yea\",\"asc\",\"vmj\",\"ash\",\"loz\",\"txq\",\"dta\",\"duy\",\"nei\",\"nka\",\"plg\",\"wji\",\"bjr\",\"rjs\",\"yyz\",\"wak\",\"eya\",\"auu\",\"sua\",\"oru\",\"tya\",\"ykg\",\"maf\",\"dme\",\"izz\",\"brz\",\"kjt\",\"gak\",\"plv\",\"sly\",\"pti\",\"mrl\",\"tuj\",\"tqr\",\"nvm\",\"mbd\",\"men\",\"ceg\",\"nev\",\"qvy\",\"ilp\",\"cam\",\"mes\",\"jbu\",\"jih\",\"wdy\",\"jic\",\"tny\",\"zaq\",\"kjx\",\"vkk\",\"pnw\",\"lko\",\"kuk\",\"quc\",\"chl\",\"quh\",\"teb\",\"cso\",\"bjy\",\"baq\",\"cof\",\"orz\",\"dur\",\"mzg\",\"dac\",\"yyu\",\"dah\",\"kdc\",\"kdh\",\"ina\",\"auz\",\"frk\",\"lob\",\"bdv\",\"vmp\",\"mom\",\"ail\",\"str\",\"lou\",\"czn\",\"rnl\",\"ifm\",\"lsa\",\"gdr\",\"mkr\",\"teu\",\"mzi\",\"enr\",\"brb\",\"jsl\",\"lje\",\"xwr\",\"nbe\",\"tnr\",\"pot\",\"bvq\",\"bdi\",\"tcc\",\"akw\",\"tch\",\"slr\",\"nof\",\"nso\",\"uan\",\"pox\",\"vit\",\"gyl\",\"bdg\",\"nzs\",\"tup\",\"pqw\",\"xgr\",\"bru\",\"mzv\",\"kjo\",\"kqk\",\"tnp\",\"qvp\",\"lzh\",\"aey\",\"sbk\",\"syr\",\"gll\",\"tur\",\"tah\",\"tac\",\"lkt\",\"ojp\",\"slp\",\"dcc\",\"bjj\",\"sqa\",\"pku\",\"wll\",\"ypk\",\"ity\",\"dup\",\"ylu\",\"bcq\",\"kba\",\"mpq\",\"mfc\",\"mfh\",\"zpf\",\"mkp\",\"aks\",\"anz\",\"ylb\",\"pkb\",\"dnr\",\"sdl\",\"evh\",\"vmr\",\"xxm\",\"stp\",\"jpr\",\"tuy\",\"wwb\",\"knk\",\"slj\",\"puw\",\"wgu\",\"tqp\",\"xri\",\"syy\",\"ygl\",\"mcf\",\"poo\",\"nsx\",\"dya\",\"szb\",\"nst\",\"bxc\",\"xrg\",\"lik\",\"dez\",\"ywl\",\"bxh\",\"qvj\",\"xwj\",\"kxv\",\"wgb\",\"kke\",\"ymq\",\"ccm\",\"tta\",\"mmk\",\"wdj\",\"aza\",\"aer\",\"pnn\",\"jmi\",\"riu\",\"stj\",\"sna\",\"cst\",\"gxx\",\"ggb\",\"vmy\",\"ncm\",\"anu\",\"dny\",\"rmm\",\"mkj\",\"gwu\",\"gdj\",\"mew\",\"gwb\",\"tme\",\"ggu\",\"aqz\",\"sfm\",\"anb\",\"kxi\",\"uge\",\"kol\",\"eme\",\"pns\",\"khf\",\"fum\",\"bjp\",\"itr\",\"tla\",\"csx\",\"rib\",\"kcs\",\"nph\",\"tbo\",\"pgu\",\"thz\",\"gyg\",\"nhg\",\"kvw\",\"chv\",\"cmt\",\"pwb\",\"urm\",\"ebo\",\"zlj\",\"bdl\",\"agn\",\"aws\",\"yha\",\"woo\",\"mwp\",\"btp\",\"rsm\",\"mdz\",\"kyx\",\"oyy\",\"nhi\",\"ags\",\"awn\",\"tse\",\"gyi\",\"gnn\",\"kyt\",\"mgp\",\"bqa\",\"kcn\",\"ese\",\"hos\",\"scq\",\"tdn\",\"aig\",\"rys\",\"byr\",\"cip\",\"wyi\",\"rtw\",\"wnn\",\"eja\",\"lwt\",\"lnd\",\"chi\",\"hno\",\"rng\",\"nmx\",\"xpq\",\"uba\",\"blp\",\"tja\",\"kaw\",\"zna\",\"pmf\",\"ddw\",\"mhn\",\"ona\",\"nhv\",\"nmt\",\"chg\",\"ahp\",\"isd\",\"mhs\",\"kid\",\"guw\",\"xkp\",\"aii\",\"cpc\",\"ryn\",\"pck\",\"mzl\",\"mxm\",\"bbk\",\"tds\",\"qxa\",\"goo\",\"bzz\",\"lgt\",\"msk\",\"cel\",\"yry\",\"bfm\",\"unk\",\"crn\",\"vau\",\"nij\",\"mgj\",\"hav\",\"sjp\",\"mdb\",\"gku\",\"mdu\",\"btj\",\"crs\",\"ldm\",\"klo\",\"bna\",\"yad\",\"fsl\",\"ehu\",\"gqn\",\"mrg\",\"xmk\",\"gbq\",\"pgz\",\"obk\",\"thu\",\"bzu\",\"wku\",\"mrv\",\"sxc\",\"iru\",\"pll\",\"kwe\",\"hvv\",\"toq\",\"hux\",\"leq\",\"xkj\",\"wbq\",\"ykl\",\"hai\",\"ace\",\"hut\",\"blj\",\"idi\",\"kto\",\"ztp\",\"dbt\",\"apo\",\"hag\",\"rao\",\"bzb\",\"wkb\",\"nrn\",\"kge\",\"hem\",\"kqw\",\"gog\",\"hni\",\"cho\",\"csl\",\"frs\",\"sbw\",\"czk\",\"tbv\",\"lrr\",\"hng\",\"lha\",\"goi\",\"bva\",\"kot\",\"xtm\",\"ver\",\"aio\",\"zhd\",\"sif\",\"ore\",\"fij\",\"yxu\",\"swm\",\"bor\",\"woi\",\"amy\",\"dmu\",\"uji\",\"dim\",\"bkm\",\"tbi\",\"ebg\",\"gov\",\"pdc\",\"wog\",\"dmb\",\"tbg\",\"sgm\",\"nzk\",\"gyo\",\"nsl\",\"baa\",\"ywt\",\"ynd\",\"jbe\",\"nho\",\"lbf\",\"apg\",\"hao\",\"xkc\",\"cpp\",\"rag\",\"ahh\",\"rem\",\"knw\",\"oor\",\"zaa\",\"okm\",\"jka\",\"rut\",\"jei\",\"cbb\",\"ktg\",\"sdx\",\"gvs\",\"wan\",\"lad\",\"eee\",\"wlx\",\"loe\",\"kpd\",\"rai\",\"gvn\",\"was\",\"cic\",\"api\",\"sdt\",\"liw\",\"cbu\",\"cih\",\"tee\",\"blh\",\"blc\",\"mmw\",\"oaa\",\"jam\",\"zor\",\"ckq\",\"kti\",\"klv\",\"pjt\",\"atd\",\"ido\",\"mjm\",\"nic\",\"bhd\",\"nih\",\"suq\",\"nbu\",\"mgc\",\"mgh\",\"kun\",\"gas\",\"vkn\",\"nkq\",\"klg\",\"mro\",\"gef\",\"mek\",\"amr\",\"boy\",\"txa\",\"rav\",\"lkl\",\"apv\",\"bth\",\"mwh\",\"mwc\",\"bre\",\"btc\",\"kli\",\"nbb\",\"fip\",\"ktv\",\"ald\",\"kus\",\"ibn\",\"gan\",\"ykt\",\"odt\",\"aau\",\"nex\",\"kvk\",\"kdy\",\"pef\",\"day\",\"mfj\",\"bjh\",\"pan\",\"bjc\",\"saa\",\"rxw\",\"quy\",\"kjv\",\"plt\",\"drd\",\"pas\",\"ltu\",\"sor\",\"mtm\",\"bwm\",\"asr\",\"num\",\"rbp\",\"dsz\",\"dbl\",\"net\",\"hul\",\"aab\",\"skm\",\"sva\",\"bxp\",\"kak\",\"bif\",\"avu\",\"cet\",\"llb\",\"kji\",\"ptt\",\"llu\",\"taj\",\"mlm\",\"kfe\",\"guk\",\"avb\",\"ecy\",\"kjg\",\"msw\",\"arq\",\"pcw\",\"tcy\",\"bbw\",\"lmq\",\"asy\",\"soy\",\"zbw\",\"mkc\",\"gdc\",\"hib\",\"afn\",\"gdh\",\"mkh\",\"pov\",\"yik\",\"ica\",\"lwl\",\"sre\",\"sth\",\"doa\",\"miq\",\"mzx\",\"ltz\",\"ngq\",\"dar\",\"buq\",\"lgl\",\"kdr\",\"shd\",\"hmm\",\"mzt\",\"qur\",\"dsb\",\"kxo\",\"mfp\",\"afs\",\"aaz\",\"nml\",\"viv\",\"xmw\",\"duh\",\"duc\",\"ecr\",\"bdt\",\"poi\",\"vig\",\"slc\",\"cml\",\"slh\",\"tap\",\"ojc\",\"zgm\",\"pog\",\"tnh\",\"tnc\",\"jiy\",\"kyl\",\"enc\",\"bxj\",\"uli\",\"enh\",\"qvh\",\"bdx\",\"qvc\",\"xwc\",\"xmn\",\"tar\",\"grm\",\"tcp\",\"mqf\",\"vra\",\"tsb\",\"syc\",\"pto\",\"tuc\",\"tuh\",\"lki\",\"wba\",\"lea\",\"crk\",\"tsu\",\"kll\",\"xms\",\"esu\",\"unn\",\"ndd\",\"soj\",\"gba\",\"vmc\",\"asj\",\"nrk\",\"yko\",\"vmh\",\"cdd\",\"plo\",\"nqm\",\"bnq\",\"jel\",\"apl\",\"ral\",\"wrm\",\"acz\",\"qup\",\"mfr\",\"dcr\",\"ktl\",\"ior\",\"kwz\",\"neo\",\"kdp\",\"qxq\",\"bdo\",\"kck\",\"trd\",\"wol\",\"aec\",\"tbl\",\"tsz\",\"jhi\",\"bxr\",\"mnf\",\"awk\",\"pid\",\"nsg\",\"csv\",\"vae\",\"gnk\",\"cfa\",\"xrt\",\"agk\",\"mym\",\"xpa\",\"qya\",\"yob\",\"paw\",\"tay\",\"mde\",\"gke\",\"kmf\",\"the\",\"hok\",\"nsi\",\"uku\",\"bed\",\"daj\",\"bbn\",\"mfy\",\"pcn\",\"kwu\",\"kdj\",\"msn\",\"kgb\",\"kxx\",\"itc\",\"ire\",\"bze\",\"acu\",\"nnm\",\"fam\",\"bqq\",\"csi\",\"jmx\",\"mzo\",\"nyf\",\"mhk\",\"pyx\",\"gol\",\"acb\",\"kgu\",\"asp\",\"nfa\",\"csg\",\"nsv\",\"bbs\",\"tdk\",\"sop\",\"tvy\",\"kxt\",\"kwb\",\"mss\",\"wnk\",\"tkf\",\"pol\",\"kkz\",\"nhx\",\"kns\",\"mmn\",\"tlq\",\"kyg\",\"lin\",\"dee\",\"cmi\",\"iwm\",\"dzd\",\"fir\",\"waw\",\"boj\",\"lis\",\"ijc\",\"igm\",\"gcn\",\"vil\",\"knn\",\"amp\",\"nmv\",\"nht\",\"cmg\",\"snq\",\"ull\",\"ayd\",\"kyi\",\"emz\",\"gaw\",\"lgg\",\"ttq\",\"aix\",\"lcd\",\"tmz\",\"nmg\",\"cht\",\"yle\",\"zca\",\"koo\",\"oca\",\"lgi\",\"chx\",\"ksf\",\"lwg\",\"gid\",\"kuw\",\"byc\",\"pnk\",\"byh\",\"nmi\",\"ait\",\"kyv\",\"twf\",\"kqs\",\"kkb\",\"yud\",\"amj\",\"fwa\",\"kjl\",\"sbs\",\"mrt\",\"npy\",\"bop\",\"sbn\",\"mrx\",\"sed\",\"sze\",\"mpa\",\"kbq\",\"bca\",\"hts\",\"kqn\",\"huv\",\"tgf\",\"kku\",\"dbv\",\"glo\",\"hyw\",\"sqq\",\"xsn\",\"emb\",\"ugb\",\"udj\",\"row\",\"hat\",\"dbg\",\"gwe\",\"tmb\",\"ane\",\"ruo\",\"hug\",\"wlo\",\"adf\",\"ypn\",\"gge\",\"sdo\",\"tmu\",\"akk\",\"prm\",\"cpy\",\"sxr\",\"hui\",\"vep\",\"idt\",\"tim\",\"emu\",\"hax\",\"dbi\",\"rsn\",\"nhf\",\"gyf\",\"nol\",\"urn\",\"myk\",\"agm\",\"xow\",\"cnk\",\"nxi\",\"wsg\",\"xuo\",\"hom\",\"bau\",\"yxa\",\"bab\",\"teq\",\"dma\",\"xqt\",\"kcm\",\"loq\",\"gnm\",\"wsi\",\"bsr\",\"awm\",\"nxg\",\"mbr\",\"juw\",\"pht\",\"smy\",\"lhu\",\"mhm\",\"bvu\",\"sio\",\"nke\",\"aif\",\"jns\",\"wnm\",\"tye\",\"zaz\",\"tdm\",\"kst\",\"sue\",\"mxs\",\"abw\",\"col\",\"ksx\",\"dgo\",\"brq\",\"mxn\",\"ofs\",\"chf\",\"drc\",\"bvb\",\"wsv\",\"yee\",\"gsg\",\"fak\",\"nnk\",\"pmi\",\"tgt\",\"sld\",\"kzw\",\"twx\",\"crm\",\"txu\",\"smr\",\"bfn\",\"mby\",\"cjo\",\"ybi\",\"xcg\",\"spv\",\"bsy\",\"geo\",\"xgd\",\"mrf\",\"nba\",\"egx\",\"lja\",\"end\",\"bfs\",\"grk\",\"xwd\",\"xnt\",\"mal\",\"txb\",\"tgx\",\"tnd\",\"twt\",\"auq\",\"wdd\",\"ldn\",\"zsr\",\"std\",\"wrk\",\"ikk\",\"cba\",\"zab\",\"adt\",\"spi\",\"vlp\",\"lse\",\"gdd\",\"nrm\",\"adx\",\"zau\",\"jku\",\"bvz\",\"shh\",\"shc\",\"ine\",\"haf\",\"njo\",\"nqk\",\"spg\",\"vbk\",\"mvl\",\"xcv\",\"weo\",\"lbo\",\"lla\",\"mie\",\"zpv\",\"xyk\",\"zmr\",\"nge\",\"mow\",\"krn\",\"srq\",\"yap\",\"muo\",\"mqx\",\"svb\",\"kel\",\"ycr\",\"lme\",\"nto\",\"sjd\",\"mqt\",\"old\",\"bio\",\"clo\",\"fox\",\"nwe\",\"krs\",\"xbr\",\"ava\",\"bue\",\"omr\",\"zpi\",\"sab\",\"doz\",\"prk\",\"akm\",\"lnj\",\"xhm\",\"ssr\",\"tik\",\"aor\",\"rrt\",\"nlo\",\"pqm\",\"cwe\",\"aaa\",\"kij\",\"otd\",\"zpg\",\"cto\",\"peo\",\"sau\",\"luy\",\"jgb\",\"bhc\",\"nid\",\"psg\",\"mnx\",\"omy\",\"bhh\",\"mgd\",\"dje\",\"ccs\",\"kmt\",\"khv\",\"xby\",\"bpg\",\"mcg\",\"naw\",\"btd\",\"ccn\",\"xal\",\"yaj\",\"kmx\",\"zmy\",\"alc\",\"alh\",\"mci\",\"bpi\",\"iwk\",\"gmg\",\"psi\",\"saz\",\"sfs\",\"rmn\",\"dob\",\"tkt\",\"kxf\",\"caw\",\"lah\",\"lac\",\"ncn\",\"nyt\",\"kip\",\"lur\",\"fli\",\"xkd\",\"gmv\",\"hia\",\"ght\",\"pnm\",\"khi\",\"wmg\",\"nyx\",\"fun\",\"bld\",\"atc\",\"tzh\",\"are\",\"ath\",\"bpv\",\"mcv\",\"wmi\",\"khg\",\"bmr\",\"oue\",\"rms\",\"tkx\",\"kpc\",\"ncs\",\"kph\",\"qtz\",\"pfa\",\"cid\",\"kfq\",\"ssy\",\"uzs\",\"nef\",\"zkk\",\"isr\",\"lup\",\"kir\",\"one\",\"pex\",\"tfi\",\"rro\",\"jak\",\"mvg\",\"spl\",\"gbb\",\"nan\",\"lcc\",\"lch\",\"efi\",\"mav\",\"gbu\",\"gih\",\"gic\",\"pum\",\"byd\",\"cvn\",\"nlx\",\"nas\",\"thq\",\"plf\",\"bmp\",\"ssj\",\"uzn\",\"zne\",\"mdq\",\"mvi\",\"aoj\",\"ctt\",\"ydk\",\"okk\",\"wih\",\"leu\",\"wic\",\"cas\",\"mem\",\"sfw\",\"tsa\",\"mag\",\"ivb\",\"bix\",\"bzq\",\"xbj\",\"ybl\",\"mut\",\"tou\",\"dkx\",\"tob\",\"mai\",\"bit\",\"yay\",\"ntx\",\"zmj\",\"mux\",\"ayh\",\"ayc\",\"mjk\",\"mqo\",\"can\",\"xcl\",\"mvv\",\"hrv\",\"leb\",\"wbb\",\"clt\",\"rmw\",\"xpz\",\"cog\",\"nov\",\"ssp\",\"mos\",\"eko\",\"kga\",\"scb\",\"nfu\",\"bmj\",\"gsl\",\"nyo\",\"gbz\",\"gho\",\"kiy\",\"kwa\",\"scu\",\"fqs\",\"lqr\",\"aca\",\"luj\",\"zqe\",\"pml\",\"mon\",\"krw\",\"ycp\",\"noi\",\"xpu\",\"bne\",\"lya\",\"yar\",\"kmo\",\"yuc\",\"uka\",\"nzm\",\"nxl\",\"sgk\",\"zmp\",\"toz\",\"omp\",\"mno\",\"cov\",\"nog\",\"swk\",\"oyd\",\"xbp\",\"uam\",\"bkk\",\"seh\",\"sec\",\"cyo\",\"lez\",\"dik\",\"bdf\",\"xpb\",\"cdh\",\"sit\",\"cdc\",\"ocu\",\"hmk\",\"kzs\",\"dgx\",\"kso\",\"dnd\",\"khl\",\"vmd\",\"ymb\",\"kof\",\"fll\",\"dgt\",\"osp\",\"xav\",\"tpn\",\"bfw\",\"six\",\"kbe\",\"kzn\",\"xvi\",\"yim\",\"xdq\",\"anq\",\"syd\",\"rkh\",\"xag\",\"smp\",\"tud\",\"xem\",\"xut\",\"bsj\",\"gml\",\"mbj\",\"eud\",\"zeh\",\"imy\",\"xai\",\"ndh\",\"ndc\",\"psl\",\"bcz\",\"mpz\",\"pho\",\"mcl\",\"bpl\",\"ele\",\"jun\",\"njx\",\"lbx\",\"nuk\",\"tma\",\"bwk\",\"deq\",\"mtk\",\"zpl\",\"ipi\",\"bec\",\"beh\",\"kev\",\"ema\",\"ymz\",\"sdf\",\"itd\",\"uga\",\"ado\",\"sne\",\"ruf\",\"wum\",\"uwa\",\"kvm\",\"xon\",\"njt\",\"urw\",\"lbt\",\"jus\",\"wet\",\"bgk\",\"rsw\",\"erh\",\"gum\",\"imr\",\"pic\",\"pbu\",\"pih\",\"mlk\",\"tgo\",\"mpu\",\"bcu\",\"aze\",\"trh\",\"trc\",\"tte\",\"mxw\",\"kka\",\"ego\",\"kei\",\"umo\",\"abn\",\"aed\",\"ppp\",\"xno\",\"wtf\",\"mbp\",\"gex\",\"abs\",\"cuk\",\"bsp\",\"rtm\",\"ewo\",\"pbb\",\"smj\",\"bcb\",\"keg\",\"mpb\",\"two\",\"kam\",\"zbc\",\"sng\",\"mkw\",\"scx\",\"kxa\",\"eli\",\"hif\",\"ctz\",\"sys\",\"pez\",\"tli\",\"zmm\",\"ttv\",\"stw\",\"tus\",\"jma\",\"doo\",\"syn\",\"tun\",\"xbm\",\"sni\",\"kyq\",\"xmc\",\"tlg\",\"duw\",\"xmh\",\"sct\",\"mej\",\"jle\",\"zib\",\"nlz\",\"sxk\",\"ngl\",\"xgw\",\"aom\",\"mil\",\"vms\",\"ssm\",\"biz\",\"hgm\",\"kee\",\"dyg\",\"xhr\",\"ojw\",\"xpt\",\"tti\",\"akr\",\"tlv\",\"slw\",\"lml\",\"puj\",\"kzd\",\"jgo\",\"xpx\",\"nmq\",\"snv\",\"pny\",\"xra\",\"tnw\",\"dnn\",\"qvw\",\"muz\",\"xww\",\"azg\",\"ttg\",\"lgq\",\"bul\",\"ntz\",\"enw\",\"dyi\",\"gbx\",\"mep\",\"jpn\",\"nlu\",\"zul\",\"ils\",\"aes\",\"kbg\",\"peb\",\"bjw\",\"sao\",\"aaf\",\"arl\",\"ctu\",\"xve\",\"qwe\",\"abd\",\"aen\",\"ziz\",\"kbi\",\"pnr\",\"llf\",\"ntu\",\"muu\",\"lum\",\"its\",\"bib\",\"wbt\",\"let\",\"xod\",\"eot\",\"xae\",\"lex\",\"vrt\",\"jud\",\"pup\",\"huq\",\"tqw\",\"clu\",\"bmm\",\"dbq\",\"pch\",\"mub\",\"pcc\",\"biu\",\"bbh\",\"bbc\",\"aky\",\"tox\",\"msc\",\"msh\",\"kbv\",\"ida\",\"hor\",\"knc\",\"dgz\",\"xkw\",\"gup\",\"bnv\",\"agr\",\"tyl\",\"cad\",\"njb\",\"gcc\",\"yrs\",\"ijn\",\"lbb\",\"mbm\",\"coe\",\"nju\",\"ciw\",\"lbu\",\"lih\",\"lic\",\"weu\",\"ppm\",\"awr\",\"bxk\",\"ijs\",\"siz\",\"bsm\",\"jko\",\"yrn\",\"gnr\",\"kcr\",\"zao\",\"kap\",\"blw\",\"cry\",\"mmc\",\"haa\",\"mmh\",\"yel\",\"dwz\",\"yij\",\"byn\",\"tdr\",\"rbk\",\"niw\",\"bni\",\"mgw\",\"xil\",\"noe\",\"txo\",\"pbt\",\"mhr\",\"mpt\",\"bct\",\"bng\",\"mra\",\"ckl\",\"kvp\",\"mpx\",\"bys\",\"mww\",\"btw\",\"geb\",\"ymx\",\"kaj\",\"kqc\",\"kqh\",\"dwu\",\"ddj\",\"oni\",\"xep\",\"awy\",\"dgb\",\"bqv\",\"aia\",\"smm\",\"zng\",\"sbc\",\"tjg\",\"bvo\",\"siu\",\"sbh\",\"efe\",\"crr\",\"njz\",\"lsl\",\"kcy\",\"lbz\",\"jje\",\"nsq\",\"cha\",\"hoy\",\"ong\",\"sib\",\"gqr\",\"tak\",\"hre\",\"otw\",\"zyn\",\"ubi\",\"guj\",\"tji\",\"mve\",\"inl\",\"agy\",\"xuu\",\"gya\",\"nha\",\"bao\",\"job\",\"csq\",\"zlw\",\"bqi\",\"yph\",\"kvj\",\"zsm\",\"mod\",\"mfk\",\"wny\",\"bqg\",\"tvk\",\"mae\",\"tdy\",\"dmf\",\"mhy\",\"xsc\",\"xsh\",\"nrr\",\"sjw\",\"yip\",\"xub\",\"gez\",\"zts\",\"xnb\",\"quk\",\"gla\",\"umu\",\"ext\",\"otn\",\"pbo\",\"tgu\",\"dak\",\"bco\",\"mpo\",\"kdk\",\"kvy\",\"frc\",\"wuy\",\"txt\",\"twb\",\"kkf\",\"gme\",\"dji\",\"nrp\",\"yir\",\"arv\",\"tdj\",\"twu\",\"tgb\",\"txx\",\"pse\",\"ztn\",\"bpe\",\"ots\",\"mhj\",\"xnu\",\"mce\",\"umb\",\"zls\",\"adu\",\"sjn\",\"gta\",\"roc\",\"roh\",\"arg\",\"gnj\",\"kay\",\"zax\",\"kjq\",\"crp\",\"tmf\",\"khe\",\"xer\",\"pdn\",\"kcj\",\"sda\",\"krd\",\"tck\",\"hoj\",\"wla\",\"ari\",\"sjs\",\"kbl\",\"ksz\",\"agj\",\"zln\",\"oui\",\"wme\",\"zat\",\"adb\",\"xnz\",\"mhp\",\"xks\",\"bln\",\"fud\",\"mii\",\"bat\",\"ywa\",\"ngi\",\"ahs\",\"lmi\",\"sok\",\"yrw\",\"ttl\",\"wnp\",\"axm\",\"nwg\",\"cin\",\"bug\",\"ask\",\"gvc\",\"ngg\",\"mig\",\"rmd\",\"wah\",\"wac\",\"ncd\",\"tgz\",\"phu\",\"xkn\",\"bls\",\"wur\",\"kvr\",\"bui\",\"nwi\",\"yga\",\"ahn\",\"bax\",\"lmg\",\"nis\",\"mgs\",\"mwn\",\"ccd\",\"btn\",\"ksu\",\"gah\",\"ibh\",\"zpe\",\"gac\",\"ngv\",\"cgg\",\"lmv\",\"lht\",\"gur\",\"tll\",\"bvt\",\"koa\",\"hop\",\"oco\",\"poq\",\"kar\",\"snl\",\"ddr\",\"bvx\",\"kcp\",\"hca\",\"mgn\",\"ksb\",\"mws\",\"bts\",\"buv\",\"nin\",\"crj\",\"cwg\",\"ymo\",\"kuc\",\"byw\",\"adz\",\"kuh\",\"bql\",\"aew\",\"mnb\",\"kmu\",\"mqz\",\"mxd\",\"lsv\",\"lfa\",\"ism\",\"kim\",\"cyb\",\"bjs\",\"ybe\",\"jnd\",\"xpo\",\"pnp\",\"pac\",\"bjn\",\"pah\",\"whu\",\"mnu\",\"xce\",\"kmb\",\"goq\",\"lnm\",\"yso\",\"bda\",\"akj\",\"pur\",\"yhl\",\"nyu\",\"itw\",\"tbq\",\"lsi\",\"tqn\",\"kgf\",\"dox\",\"tku\",\"amk\",\"mer\",\"mza\",\"ing\",\"qxl\",\"spe\",\"kwf\",\"tkb\",\"dot\",\"nzy\",\"urd\",\"tjl\",\"sco\",\"acf\",\"nyb\",\"ubl\",\"bnl\",\"too\",\"hbn\",\"xii\",\"akp\",\"hed\",\"nkv\",\"mnz\",\"dun\",\"mks\",\"gds\",\"svx\",\"mqb\",\"puy\",\"suv\",\"sts\",\"tuw\",\"dlg\",\"leo\",\"syw\",\"tyv\",\"nxe\",\"stn\",\"pnj\",\"cea\",\"vro\",\"ktq\",\"lrk\",\"pta\",\"yev\",\"kmz\",\"mqu\",\"gdn\",\"mkn\",\"afh\",\"raq\",\"dus\",\"apq\",\"wja\",\"xiv\",\"enn\",\"vmw\",\"nzr\",\"nea\",\"ffi\",\"usu\",\"nki\",\"dnw\",\"qvn\",\"sui\",\"tnn\",\"sls\",\"yam\",\"ldd\",\"sax\",\"ojs\",\"tyi\",\"gse\",\"sln\",\"tkz\",\"tns\",\"ckv\",\"nkg\",\"sat\",\"klq\",\"uar\",\"pla\",\"oda\",\"yka\",\"yei\",\"pme\",\"mey\",\"bfd\",\"bok\",\"xgn\",\"qvs\",\"dti\",\"gbo\",\"jye\",\"sug\",\"yot\",\"wgi\",\"szg\",\"bgr\",\"rgs\",\"xru\",\"anv\",\"fap\",\"gin\",\"nnp\",\"bhw\",\"hmy\",\"lcs\",\"kfl\",\"skr\",\"gis\",\"wgg\",\"alw\",\"nac\",\"nah\",\"yox\",\"xrb\",\"asm\",\"nur\",\"mtr\",\"axk\",\"bwr\",\"rgn\",\"som\",\"act\",\"ggg\",\"cac\",\"law\",\"cah\",\"pyu\",\"gcd\",\"rhp\",\"kgx\",\"kxb\",\"dso\",\"ani\",\"dhl\",\"knd\",\"kwt\",\"cur\",\"gwi\",\"ayn\",\"oia\",\"dof\",\"jim\",\"kgt\",\"szv\",\"atw\",\"myp\",\"kwx\",\"zia\",\"ays\",\"hio\",\"mmd\",\"acx\",\"kpw\",\"gwg\",\"dbe\",\"win\",\"lid\",\"dzn\",\"xaq\",\"grj\",\"mlr\",\"jmb\",\"cnp\",\"ang\",\"hue\",\"nvh\",\"tsx\",\"cla\",\"moh\",\"avo\",\"sky\",\"moc\",\"cme\",\"bia\",\"mty\",\"bwy\",\"dei\",\"nuy\",\"dka\",\"esx\",\"bgy\",\"hld\",\"wrp\",\"ikp\",\"xsd\",\"mua\",\"faj\",\"est\",\"qum\",\"nnj\",\"hmr\",\"kye\",\"deg\",\"tst\",\"kdm\",\"dam\",\"sbd\",\"sen\",\"keq\",\"cta\",\"tcm\",\"pea\",\"kxz\",\"kqd\",\"yli\",\"srl\",\"lwe\",\"aao\",\"dev\",\"saf\",\"myj\",\"pkg\",\"yun\",\"nla\",\"zhw\",\"cuy\",\"ses\",\"lto\",\"aqg\",\"zgr\",\"nme\",\"ylg\",\"cds\",\"lrm\",\"jay\",\"mru\",\"bzv\",\"mdg\",\"kzc\",\"vai\",\"tpc\",\"nbo\",\"gyz\",\"kkx\",\"vjk\",\"nhz\",\"eip\",\"mdi\",\"mvq\",\"gea\",\"kkt\",\"vut\",\"cja\",\"tip\",\"xtr\",\"vem\",\"cdn\",\"vag\",\"thi\",\"nse\",\"mrb\",\"rey\",\"nja\",\"bzi\",\"wea\",\"bkr\",\"iri\",\"dir\",\"zaf\",\"cse\",\"vav\",\"zen\",\"bom\",\"und\",\"ndn\",\"emx\",\"idb\",\"hau\",\"swr\",\"aul\",\"xmd\",\"idu\",\"hab\",\"mdv\",\"sgr\",\"bzg\",\"nds\",\"maq\",\"chz\",\"mjy\",\"cbo\",\"yak\",\"thv\",\"tmt\",\"shw\",\"ajw\",\"xoc\",\"pgi\",\"tij\",\"okr\",\"bes\",\"dmo\",\"rer\",\"gyb\",\"nhb\",\"pwg\",\"xty\",\"xua\",\"pcd\",\"ben\",\"nhu\",\"bbd\",\"pgg\",\"lke\",\"zom\",\"msd\",\"pwi\",\"brl\",\"zkr\",\"juc\",\"juh\",\"udm\",\"kik\",\"isk\",\"baf\",\"coq\",\"mrz\",\"ers\",\"rnb\",\"rxd\",\"sgy\",\"chu\",\"pis\",\"aib\",\"trs\",\"dga\",\"xyj\",\"noq\",\"mjr\",\"xly\",\"haz\",\"dwa\",\"diy\",\"trn\",\"bky\",\"abc\",\"abh\",\"bvf\",\"lol\",\"swy\",\"sia\",\"chb\",\"amm\",\"tel\",\"wyb\",\"pin\",\"koz\",\"ruu\",\"wlu\",\"sdb\",\"mjp\",\"ajs\",\"sgj\",\"aql\",\"bew\",\"xyy\",\"yll\",\"sri\",\"fmu\",\"ada\",\"ugo\",\"bkj\",\"gtu\",\"ihw\",\"dij\",\"srg\",\"tmo\",\"rub\",\"ute\",\"swj\",\"urc\",\"sdu\",\"ajn\",\"urh\",\"cbt\",\"piw\",\"tiy\",\"bmk\",\"drn\",\"twa\",\"poe\",\"erw\",\"zpq\",\"nbt\",\"xtj\",\"del\",\"xna\",\"trw\",\"ytl\",\"mxh\",\"srv\",\"mxc\",\"glb\",\"ega\",\"kko\",\"luk\",\"glu\",\"ljx\",\"uma\",\"bcf\",\"pbf\",\"zkp\",\"pad\",\"drs\",\"tga\",\"vie\",\"ule\",\"wtb\",\"swp\",\"gmq\",\"sdz\",\"ldh\",\"ril\",\"omk\",\"dip\",\"dhi\",\"bkp\",\"anl\",\"ggl\",\"bpq\",\"mcq\",\"bfh\",\"bfc\",\"xlp\",\"psq\",\"kfv\",\"sgp\",\"zmk\",\"dhg\",\"ruz\",\"kou\",\"mjj\",\"shn\",\"kfg\",\"khq\",\"jaj\",\"pha\",\"rkw\",\"heh\",\"kje\",\"ygu\",\"dhv\",\"xqa\",\"gft\",\"ssk\",\"ndw\",\"aok\",\"afd\",\"tir\",\"xtp\",\"dmx\",\"kfi\",\"sxm\",\"ywu\",\"szl\",\"okj\",\"prr\",\"rej\",\"shs\",\"mzb\",\"nxq\",\"log\",\"scf\",\"myy\",\"rod\",\"teg\",\"hix\",\"jee\",\"ynn\",\"mlj\",\"grr\",\"kwo\",\"nez\",\"tam\",\"nya\",\"loi\",\"smk\",\"hit\",\"gha\",\"ape\",\"eka\",\"kgo\",\"yns\",\"krh\",\"krc\",\"kte\",\"plz\",\"cuj\",\"tka\",\"tei\",\"mzu\",\"brv\",\"skj\",\"wha\",\"frd\",\"tvm\",\"pgl\",\"cya\",\"mna\",\"nuj\",\"brg\",\"zhn\",\"mtj\",\"fbl\",\"bdu\",\"bwj\",\"yuw\",\"bdb\",\"lov\",\"xpf\",\"nny\",\"fay\",\"kma\",\"bgj\",\"pmq\",\"hmp\",\"mfm\",\"kle\",\"zsk\",\"ikr\",\"bri\",\"tev\",\"wrr\",\"sew\",\"cup\",\"mzz\",\"bsk\",\"hne\",\"aug\",\"gbf\",\"bzl\",\"wkl\",\"bxm\",\"ppk\",\"plu\",\"mbk\",\"bhs\",\"odu\",\"yku\",\"aat\",\"cch\",\"rpt\",\"ccc\",\"ibd\",\"aln\",\"lvs\",\"gad\",\"org\",\"fvr\",\"neb\",\"cnr\",\"aax\",\"mlp\",\"bhn\",\"neu\",\"zrg\",\"usa\",\"aui\",\"giw\",\"goe\",\"kud\",\"als\",\"ori\",\"plb\",\"gry\",\"hkk\",\"myr\",\"gju\",\"llx\",\"avt\",\"spq\",\"far\",\"woe\",\"nnr\",\"kpn\",\"ptu\",\"las\",\"bgp\",\"mqa\",\"fuh\",\"wry\",\"tbe\",\"fuc\",\"hmj\",\"atn\",\"val\",\"tzn\",\"eso\",\"bdz\",\"mtp\",\"ufi\",\"bwp\",\"lef\",\"nqy\",\"wbf\",\"mdl\",\"ats\",\"nup\",\"orv\",\"tso\",\"ceb\",\"rmh\",\"jbi\",\"rmc\",\"thl\",\"nch\",\"kps\",\"skp\",\"lan\",\"ncc\",\"wad\",\"tof\",\"nce\",\"zun\",\"gly\",\"rme\",\"sak\",\"kva\",\"wua\",\"bot\",\"ars\",\"pdi\",\"tib\",\"mrp\",\"ygr\",\"arn\",\"emm\",\"box\",\"ywr\",\"tbh\",\"tiu\",\"sjg\",\"fue\",\"tbc\",\"npo\",\"bqw\",\"ael\",\"tmm\",\"woc\",\"pru\",\"sfe\",\"ebc\",\"paq\",\"dda\",\"hap\",\"cpo\",\"kaa\",\"tjw\",\"lrt\",\"xyb\",\"oti\",\"aij\",\"goc\",\"goh\",\"djn\",\"svk\",\"ztg\",\"kkm\",\"zpd\",\"vum\",\"qxw\",\"ruy\",\"wly\",\"chj\",\"cce\",\"gua\",\"lpa\",\"itl\",\"kor\",\"onw\",\"hnh\",\"mwi\",\"bti\",\"blv\",\"yia\",\"klh\",\"klc\",\"prz\",\"dok\",\"amx\",\"bpd\",\"mcd\",\"tiz\",\"mgg\",\"nig\",\"psd\",\"glr\",\"mrj\",\"tul\",\"syl\",\"btg\",\"mwg\",\"nii\",\"xkv\",\"bnw\",\"mgi\",\"gmd\",\"nhp\",\"amt\",\"bli\",\"mwv\",\"wlr\",\"btv\",\"bus\",\"kth\",\"ngn\",\"kre\",\"ktc\",\"jgk\",\"min\",\"igb\",\"koy\",\"ahg\",\"chp\",\"lmn\",\"wmd\",\"rah\",\"xkg\",\"rac\",\"aph\",\"apc\",\"jeh\",\"rnp\",\"sxo\",\"bun\",\"niv\",\"mgv\",\"vml\",\"xki\",\"aip\",\"mis\",\"khd\",\"haj\",\"sdr\",\"ngs\",\"ahi\",\"ins\",\"ikz\",\"gjr\",\"wrz\",\"ptr\",\"acm\",\"nnu\",\"fau\",\"jit\",\"lsn\",\"kwm\",\"tao\",\"lss\",\"bjv\",\"kjh\",\"kjc\",\"nnb\",\"bdy\",\"inn\",\"fab\",\"nxd\",\"vsl\",\"bvk\",\"myb\",\"sox\",\"kbw\",\"plr\",\"yom\",\"tvo\",\"bjg\",\"asx\",\"mzy\",\"ykr\",\"pmd\",\"cnb\",\"bfe\",\"ner\",\"nza\",\"bji\",\"cnu\",\"grz\",\"bak\",\"kif\",\"mfo\",\"sot\",\"kxp\",\"myu\",\"frq\",\"yzg\",\"ast\",\"yes\",\"nkn\",\"ojg\",\"ulc\",\"wrb\",\"tct\",\"qvi\",\"bxo\",\"slg\",\"dts\",\"vic\",\"duv\",\"xgg\",\"tni\",\"sun\",\"tyn\",\"pua\",\"zak\",\"kuq\",\"bdr\",\"cey\",\"ttw\",\"sus\",\"yrl\",\"stv\",\"sli\",\"xwg\",\"pty\",\"mxe\",\"eng\",\"gaq\",\"yaf\",\"tys\",\"oji\",\"wdg\",\"nnz\",\"vbb\",\"faz\",\"nks\",\"tcx\",\"tng\",\"poh\",\"wru\",\"dtn\",\"mkv\",\"xgi\",\"iku\",\"poc\",\"kdx\",\"myz\",\"env\",\"xin\",\"ure\",\"uth\",\"dax\",\"stg\",\"waq\",\"byl\",\"dui\",\"ney\",\"kxj\",\"snw\",\"gdg\",\"mkg\",\"qux\",\"tnv\",\"cks\",\"gru\",\"esm\",\"dug\",\"slv\",\"grb\",\"sti\",\"ply\",\"ckn\",\"ojv\",\"xis\",\"tsm\",\"kdt\",\"spd\",\"yky\",\"dln\",\"mzr\",\"mea\",\"mki\",\"gdi\",\"zxx\",\"zgb\",\"tvt\",\"nej\",\"ogu\",\"kxy\",\"mlz\",\"mfx\",\"bll\",\"zdj\",\"etn\",\"iqw\",\"abe\",\"vmg\",\"dng\",\"azn\",\"ttn\",\"pyy\",\"mzp\",\"xjb\",\"tts\",\"suw\",\"tuv\",\"tvx\",\"ahl\",\"ssf\",\"aso\",\"dni\",\"ogb\",\"ets\",\"xkl\",\"vmi\",\"mft\",\"aof\",\"dyn\",\"nkw\",\"soo\",\"tug\",\"bwz\",\"liq\",\"bdp\",\"xad\",\"hmb\",\"nuz\",\"mwl\",\"lkh\",\"lkc\",\"ymk\",\"tat\",\"sns\",\"skz\",\"mmq\",\"jio\",\"tln\",\"syi\",\"pna\",\"tax\",\"zmf\",\"knq\",\"tls\",\"snn\",\"him\",\"tui\",\"bgz\",\"hmu\",\"dnv\",\"vmv\",\"mgl\",\"nil\",\"mlb\",\"jmr\",\"otl\",\"avm\",\"xsq\",\"bmf\",\"cuu\",\"mzj\",\"cub\",\"csc\",\"csh\",\"lsw\",\"kxr\",\"luf\",\"mpk\",\"nep\",\"kdo\",\"llm\",\"bck\",\"dao\",\"ilv\",\"ztl\",\"iti\",\"mlu\",\"bwb\",\"nsc\",\"mtb\",\"nsh\",\"hmz\",\"bgu\",\"xrr\",\"nub\",\"tco\",\"sjl\",\"kbs\",\"ilg\",\"ptp\",\"skb\",\"sqs\",\"tpe\",\"kqq\",\"pqa\",\"sku\",\"sqn\",\"nuu\",\"sbq\",\"bgb\",\"xha\",\"kze\",\"ked\",\"kbn\",\"itv\",\"mtu\",\"ili\",\"bdj\",\"bwu\",\"aka\",\"dul\",\"lro\",\"wyr\",\"nmc\",\"nmh\",\"byi\",\"bnn\",\"chr\",\"gqa\",\"sgz\",\"mvd\",\"nbm\",\"lgh\",\"mju\",\"rup\",\"mjb\",\"sdp\",\"byg\",\"stl\",\"air\",\"lwh\",\"xzp\",\"tok\",\"veo\",\"mkl\",\"gdl\",\"cra\",\"diz\",\"wbk\",\"hay\",\"lek\",\"cpx\",\"bkz\",\"bns\",\"fmp\",\"rnr\",\"mad\",\"nww\",\"qvl\",\"byv\",\"xwl\",\"oku\",\"enl\",\"buw\",\"kyc\",\"kyh\",\"gbk\",\"xmq\",\"npx\",\"jab\",\"bsf\",\"boo\",\"tnl\",\"mbf\",\"nra\",\"ife\",\"nhr\",\"gyr\",\"cmc\",\"lmw\",\"sll\",\"reb\",\"jau\",\"mry\",\"okb\",\"miw\",\"xgl\",\"zku\",\"ngw\",\"fit\",\"moe\",\"bku\",\"awa\",\"yhs\",\"aiy\",\"huh\",\"huc\",\"gna\",\"hsb\",\"cod\",\"bqn\",\"xlb\",\"diu\",\"dmm\",\"sgb\",\"bbq\",\"kca\",\"har\",\"swu\",\"yxm\",\"sdj\",\"vnm\",\"arw\",\"msq\",\"hoa\",\"chy\",\"mjz\",\"swb\",\"idr\",\"kop\",\"bjl\",\"wyy\",\"zoo\",\"sgu\",\"xpk\",\"dib\",\"xlu\",\"aga\",\"vsv\",\"bqs\",\"cae\",\"onn\",\"mrr\",\"glj\",\"ygp\",\"zkz\",\"gyy\",\"xtb\",\"zns\",\"nhy\",\"tjs\",\"nae\",\"qxn\",\"jaz\",\"zyg\",\"sck\",\"tql\",\"qxs\",\"tda\",\"djw\",\"nod\",\"fkk\",\"smf\",\"okz\",\"ons\",\"amo\",\"xtu\",\"tjn\",\"vsi\",\"mha\",\"dri\",\"cxh\",\"iir\",\"pmh\",\"ksr\",\"pbm\",\"xya\",\"guu\",\"llk\",\"bcm\",\"mpm\",\"kab\",\"avk\",\"kau\",\"gub\",\"ady\",\"drg\",\"sij\",\"wep\",\"cpf\",\"fif\",\"afe\",\"sel\",\"wub\",\"pww\",\"kvb\",\"tia\",\"twy\",\"bsx\",\"vto\",\"xny\",\"cjp\",\"ajg\",\"pra\",\"srn\",\"mbx\",\"yiz\",\"bst\",\"xuj\",\"mbt\",\"egy\",\"yul\",\"phr\",\"ppt\",\"srs\",\"kvu\",\"wuu\",\"aak\",\"tgy\",\"aji\",\"kjd\",\"lbj\",\"adr\",\"shi\",\"upi\",\"rwl\",\"njj\",\"sip\",\"xeu\",\"rmq\",\"ncq\",\"osx\",\"gil\",\"kaz\",\"kfs\",\"ksy\",\"lcl\",\"spc\",\"guz\",\"kfn\",\"ost\",\"shg\",\"xeb\",\"fuq\",\"wkw\",\"bzw\",\"umr\",\"hik\",\"smt\",\"shv\",\"upv\",\"xch\",\"kvz\",\"xcc\",\"imo\",\"ayl\",\"tgr\",\"pae\",\"xup\",\"vid\",\"mdw\",\"dhn\",\"ocm\",\"twr\",\"gej\",\"ybh\",\"yiu\",\"dhs\",\"dzl\",\"smx\",\"ymm\",\"xnr\",\"wil\",\"dsk\",\"nzu\",\"kue\",\"gfk\",\"god\",\"cna\",\"sst\",\"pej\",\"brs\",\"mff\",\"aot\",\"zhi\",\"iso\",\"kio\",\"xpm\",\"eky\",\"nlj\",\"cfm\",\"mez\",\"nyy\",\"hnd\",\"bel\",\"zph\",\"ibe\",\"gae\",\"zpc\",\"vnk\",\"ssx\",\"nzb\",\"ysm\",\"aox\",\"mxq\",\"brn\",\"dmk\",\"vol\",\"tes\",\"clj\",\"los\",\"wae\",\"zmt\",\"for\",\"mny\",\"omx\",\"kmy\",\"zmx\",\"ntj\",\"taf\",\"lon\",\"wod\",\"gve\",\"ytw\",\"omt\",\"nna\",\"trl\",\"muj\",\"faa\",\"yng\",\"mqr\",\"pil\",\"ten\",\"tbd\",\"ghr\",\"tom\",\"wmh\",\"wmc\",\"lut\",\"nyr\",\"rad\",\"kpi\",\"ekr\",\"apd\",\"bmx\",\"lag\",\"ggw\",\"lem\",\"ktd\",\"wbm\",\"meu\",\"alv\",\"ati\",\"tkr\",\"bhv\",\"bmt\",\"gww\",\"flh\",\"kpg\",\"meb\",\"khc\",\"anw\",\"khh\",\"pep\",\"atg\",\"jbn\",\"nbk\",\"ctp\",\"nzz\",\"lai\",\"roe\",\"gra\",\"www\",\"ors\",\"mch\",\"bpc\",\"pub\",\"bph\",\"mcc\",\"bfq\",\"kpv\",\"zrn\",\"psc\",\"bhg\",\"kmr\",\"psh\",\"ndl\",\"mup\",\"yao\",\"szw\",\"cbk\",\"ntp\",\"kld\",\"aus\",\"mqy\",\"atv\",\"ali\",\"fre\",\"bhi\",\"gmh\",\"bip\",\"lvi\",\"ldq\",\"aun\",\"alg\",\"bxf\",\"orn\",\"mnr\",\"zrs\",\"puu\",\"gbm\",\"lav\",\"ntr\",\"pnu\",\"rin\",\"hle\",\"xse\",\"jbw\",\"mur\",\"mqj\",\"mac\",\"dzg\",\"hma\",\"mah\",\"kmp\",\"ann\",\"ayi\",\"zmo\",\"kyd\",\"gwn\",\"wig\",\"omo\",\"ans\",\"akz\",\"xbo\",\"mnp\",\"wii\",\"bir\",\"ayg\",\"pnb\",\"dkr\",\"tkp\",\"mvh\",\"asf\",\"shl\",\"orw\",\"hrc\",\"sso\",\"szs\",\"gii\",\"auw\",\"nmd\",\"ekp\",\"zga\",\"kit\",\"nyp\",\"ist\",\"szn\",\"hwo\",\"sbe\",\"per\",\"zwa\",\"kix\",\"kzq\",\"wiv\",\"tpq\",\"kqe\",\"gig\",\"cly\",\"brw\",\"hps\",\"sei\",\"ska\",\"yug\",\"abq\",\"pkn\",\"emk\",\"nua\",\"aqn\",\"xhu\",\"mnj\",\"yln\",\"biy\",\"yat\",\"mta\",\"aku\",\"bwa\",\"sam\",\"muy\",\"akb\",\"yui\",\"bga\",\"mqp\",\"seg\",\"kmj\",\"yax\",\"nty\",\"tcf\",\"noh\",\"noc\",\"pnz\",\"jdt\",\"pks\",\"tew\",\"cty\",\"pey\",\"mme\",\"kdf\",\"sev\",\"yvt\",\"luo\",\"kkk\",\"drl\",\"quf\",\"hud\",\"low\",\"coh\",\"coc\",\"usp\",\"dbd\",\"lie\",\"den\",\"mla\",\"gce\",\"nyj\",\"bmo\",\"des\",\"nly\",\"svm\",\"cua\",\"kne\",\"irn\",\"bzn\",\"lkd\",\"mse\",\"nrb\",\"xur\",\"phj\",\"pce\",\"tgp\",\"jaa\",\"bbe\",\"all\",\"zka\",\"kfw\",\"xac\",\"ump\",\"ndi\",\"mbo\",\"lvl\",\"bof\",\"caq\",\"ppo\",\"cjy\",\"mhz\",\"oka\",\"yda\",\"bso\",\"jkm\",\"gey\",\"bhl\",\"ndg\",\"zam\",\"rki\",\"jor\",\"bzs\",\"rea\",\"twp\",\"zeg\",\"nru\",\"dwr\",\"kpl\",\"naq\",\"ths\",\"njy\",\"qwc\",\"lby\",\"kcz\",\"esk\",\"qwh\",\"van\",\"tzl\",\"txm\",\"atl\",\"gnz\",\"sir\",\"mds\",\"ehs\",\"tsk\",\"ndv\",\"gkn\",\"mdn\",\"dhw\",\"vas\",\"crb\",\"agz\",\"ksj\",\"gqu\",\"lal\",\"cdi\",\"dgr\",\"hoz\",\"thn\",\"gza\",\"ger\",\"nrz\",\"kgk\",\"wnu\",\"amf\",\"smo\",\"trg\",\"ynl\",\"bvm\",\"pig\",\"nsd\",\"xta\",\"lhm\",\"erg\",\"mhu\",\"xnj\",\"imt\",\"bev\",\"tri\",\"ack\",\"mhb\",\"kec\",\"keh\",\"ryu\",\"kwk\",\"eri\",\"wnb\",\"tgj\",\"tdb\",\"pgn\",\"ukk\",\"bam\",\"kcb\",\"xme\",\"beg\",\"sga\",\"gnb\",\"ksp\",\"xla\",\"agu\",\"awb\",\"ihi\",\"wer\",\"adj\",\"bka\",\"awu\",\"oso\",\"trv\",\"bei\",\"lbr\",\"njr\",\"srw\",\"yok\",\"gnu\",\"crz\",\"axx\",\"agb\",\"dwy\",\"dia\",\"moq\",\"une\",\"pgs\",\"pwn\",\"kcu\",\"csd\",\"piv\",\"swa\",\"siy\",\"hob\",\"zbe\",\"xpr\",\"xht\",\"xmv\",\"mdd\",\"gkd\",\"ulw\",\"yau\",\"akt\",\"wal\",\"byq\",\"thd\",\"gvl\",\"puo\",\"hji\",\"ysr\",\"azc\",\"tth\",\"ttc\",\"tre\",\"ley\",\"hak\",\"pie\",\"bxa\",\"toy\",\"yab\",\"pow\",\"etc\",\"eth\",\"akx\",\"kul\",\"vkl\",\"meo\",\"gby\",\"fom\",\"obi\",\"lks\",\"nfr\",\"ung\",\"lnz\",\"snc\",\"lkn\",\"bzd\",\"wkd\",\"tlh\",\"bee\",\"xmg\",\"tlc\",\"mqm\",\"hdy\",\"mrk\",\"gal\",\"kiz\",\"ibl\",\"uni\",\"elh\",\"hvk\",\"pci\",\"tkm\",\"ler\",\"bbi\",\"wbr\",\"ysy\",\"msi\",\"ekm\",\"pnt\",\"nde\",\"nym\",\"csn\",\"tor\",\"aik\",\"msg\",\"pnx\",\"pgd\",\"taa\",\"faf\",\"css\",\"bbg\",\"xpy\",\"nnf\",\"pcg\",\"czo\",\"eaa\",\"yaz\",\"chk\",\"nss\",\"bbv\",\"mfa\",\"kbh\",\"msv\",\"kiu\",\"isu\",\"kbc\",\"nhk\",\"kmm\",\"lnb\",\"sqh\",\"saj\",\"myf\",\"jhs\",\"dop\",\"iar\",\"gbr\",\"mnm\",\"lnu\",\"nsn\",\"tva\",\"kib\",\"rol\",\"cde\",\"bap\",\"klw\",\"nms\",\"bcr\",\"mpr\",\"wci\",\"pbr\",\"mht\",\"mmv\",\"see\",\"tdt\",\"ksm\",\"liv\",\"lgs\",\"fgr\",\"kxk\",\"tdx\",\"vmq\",\"yue\",\"lws\",\"lgn\",\"txj\",\"nmn\",\"knv\",\"mhx\",\"bnc\",\"awx\",\"ktw\",\"agt\",\"zaj\",\"kys\",\"mmi\",\"ggd\",\"hot\",\"raw\",\"bvp\",\"kcx\",\"apw\",\"kng\",\"cmn\",\"lhp\",\"mmg\",\"kct\",\"cms\",\"euq\",\"and\",\"awt\",\"kni\",\"axb\",\"agx\",\"phm\",\"gwd\",\"gnt\",\"kyn\",\"lig\",\"tuq\",\"fia\",\"hus\",\"xsi\",\"tif\",\"ypg\",\"prf\",\"aye\",\"npa\",\"pal\",\"nrx\",\"sbv\",\"pby\",\"mpy\",\"tbw\",\"bcy\",\"kvo\",\"nrt\",\"kqv\",\"hun\",\"dze\",\"baj\",\"dbn\",\"bqc\",\"bqh\",\"wie\",\"ded\",\"wow\",\"xfa\",\"hti\",\"crt\",\"xsv\",\"guo\",\"lpo\",\"sbg\",\"egm\",\"gie\",\"umm\",\"kqg\",\"gow\",\"qxc\",\"xnm\",\"qxh\",\"sbi\",\"lce\",\"ymr\",\"bvj\",\"jkp\",\"aeq\",\"crx\",\"zap\",\"kqi\",\"rge\",\"aqd\",\"cpa\",\"ddo\",\"twm\",\"kao\",\"tqq\",\"won\",\"kpe\",\"zsu\",\"xet\",\"eze\",\"dbw\",\"uda\",\"ago\",\"huw\",\"oar\",\"ate\",\"arc\",\"arh\",\"rog\",\"ted\",\"tbn\",\"xum\",\"hoo\",\"bvy\",\"bkf\",\"dif\",\"zar\",\"kco\",\"tbs\",\"bdk\",\"gno\",\"zuh\",\"wos\",\"lae\",\"jkr\",\"awo\",\"osu\",\"ymp\",\"swf\",\"mbz\",\"msl\",\"yix\",\"mho\",\"hnn\",\"sim\",\"bbl\",\"pcl\",\"txr\",\"mzk\",\"ama\",\"wno\",\"gos\",\"ale\",\"smu\",\"tdo\",\"brd\",\"gon\",\"mpj\",\"bhe\",\"bcj\",\"djc\",\"smb\",\"bay\",\"yit\",\"hns\",\"gut\",\"obl\",\"kax\",\"tnq\",\"zay\",\"cro\",\"kls\",\"xml\",\"aud\",\"nmw\",\"kui\",\"bvr\",\"cgc\",\"ibg\",\"enq\",\"cjm\",\"gag\",\"wav\",\"gem\",\"kug\",\"lww\",\"lpx\",\"gux\",\"gjk\",\"gai\",\"lra\",\"zbl\",\"cek\",\"kln\",\"mbb\",\"buc\",\"kyw\",\"buh\",\"nwc\",\"bsb\",\"kts\",\"jaf\",\"kuv\",\"nek\",\"mpp\",\"kvx\",\"wux\",\"bcp\",\"jen\",\"pbp\",\"yne\",\"wai\",\"bar\",\"ras\",\"aps\",\"duq\",\"kvt\",\"lmh\",\"wut\",\"boa\",\"lmc\",\"txy\",\"odk\",\"ykk\",\"apn\",\"smz\",\"mkq\",\"gdq\",\"ran\",\"bsu\",\"stq\",\"wag\",\"njm\",\"mic\",\"plk\",\"mbu\",\"mih\",\"ppu\",\"wem\",\"ngc\",\"ngh\",\"ktn\",\"lbm\",\"inc\",\"kda\",\"inh\",\"daa\",\"bmb\",\"met\",\"she\",\"pav\",\"glk\",\"say\",\"mum\",\"qua\",\"luu\",\"mlf\",\"kjn\",\"ntm\",\"lsh\",\"zlq\",\"csw\",\"lsc\",\"aoz\",\"lub\",\"ssz\",\"kjs\",\"bim\",\"sbl\",\"clm\",\"wtk\",\"bmu\",\"icr\",\"srd\",\"gbp\",\"dor\",\"kql\",\"skf\",\"ako\",\"top\",\"sdk\",\"xsl\",\"nsw\",\"xho\",\"dva\",\"pai\",\"nuf\",\"pux\",\"mtf\",\"otq\",\"bwf\",\"nlm\",\"wbp\",\"lep\",\"ztq\",\"xpj\",\"put\",\"wxw\",\"bgf\",\"wlk\",\"ctm\",\"tca\",\"ruk\",\"pem\",\"zmz\",\"pag\",\"yec\",\"mml\",\"soa\",\"gbj\",\"dre\",\"bmz\",\"dth\",\"pon\",\"vis\",\"asa\",\"doy\",\"mwq\",\"btq\",\"ssu\",\"lil\",\"aou\",\"niq\",\"suc\",\"mgq\",\"aob\",\"gcl\",\"ssb\",\"luz\",\"tyh\",\"uln\",\"nkc\",\"scp\",\"nkh\",\"vin\",\"knl\",\"dhd\",\"sar\",\"pos\",\"omb\",\"zmu\",\"xbb\",\"afi\",\"kok\",\"kfd\",\"blq\",\"hmf\",\"xpp\",\"pno\",\"qyp\",\"ckh\",\"zim\",\"xkq\",\"afg\",\"jia\",\"wbj\",\"lej\",\"zmb\",\"omu\",\"ysp\",\"czt\",\"toj\",\"kwr\",\"gro\",\"rbb\",\"cut\",\"ndq\",\"wmw\",\"mfz\",\"mas\",\"hij\",\"acr\",\"bfl\",\"cag\",\"nav\",\"gjm\",\"mlx\",\"aap\",\"khw\",\"anc\",\"anh\",\"man\",\"gwc\",\"mlt\",\"gek\",\"cai\",\"kgr\",\"ldl\",\"yva\",\"cjk\",\"mef\",\"cux\",\"tsy\",\"psw\",\"mvs\",\"skx\",\"bpw\",\"jda\",\"mcw\",\"tfn\",\"ykm\",\"cvg\",\"lbk\",\"szc\",\"nux\",\"bxu\",\"esy\",\"yor\",\"nqo\",\"bgt\",\"nai\",\"fni\",\"lzz\",\"bwx\",\"xdc\",\"mtx\",\"puf\",\"nut\",\"iko\",\"wro\",\"bxb\",\"bgx\",\"yaa\",\"mtt\",\"gmw\",\"bnd\",\"bwt\",\"ukr\",\"mvn\",\"fng\",\"cav\",\"nag\",\"skt\",\"llp\",\"nem\",\"taz\",\"bdm\",\"xjt\",\"url\",\"non\",\"rsl\",\"lna\",\"bye\",\"mfb\",\"tvu\",\"kgy\",\"znd\",\"hhi\",\"myo\",\"ues\",\"aqc\",\"mfu\",\"kia\",\"kwy\",\"isa\",\"trq\",\"nos\",\"acy\",\"zpw\",\"iou\",\"cno\",\"pkc\",\"pkh\",\"mov\",\"llj\",\"tau\",\"dgk\",\"yhd\",\"ije\",\"nno\",\"mog\",\"hmx\",\"fao\",\"cos\",\"uky\",\"mzm\",\"mxl\",\"yre\",\"hmt\",\"sik\",\"bqd\",\"bxz\",\"deh\",\"dec\",\"con\",\"tsr\",\"beq\",\"dwk\",\"jnl\",\"yoy\",\"moi\",\"tab\",\"ygm\",\"snd\",\"swt\",\"ayq\",\"cbj\",\"sxb\",\"sgx\",\"bkt\",\"xas\",\"iwo\",\"bkx\",\"tpv\",\"ccl\",\"igo\",\"xan\",\"tmr\",\"dix\",\"sxu\",\"ite\",\"bzh\",\"bzc\",\"irh\",\"swx\",\"tld\",\"kzv\",\"sgt\",\"giq\",\"yif\",\"xcw\",\"ncl\",\"tpg\",\"rml\",\"thh\",\"xvs\",\"qws\",\"kzg\",\"mdh\",\"mdc\",\"vnp\",\"dyd\",\"kom\",\"nbj\",\"xtt\",\"vah\",\"ful\",\"tpi\",\"ima\",\"zik\",\"ile\",\"aee\",\"azd\",\"xvn\",\"epi\",\"ttd\",\"kzi\",\"lcq\",\"kkr\",\"tmy\",\"bik\",\"kaf\",\"pmw\",\"mjt\",\"krl\",\"dkk\",\"emy\",\"seq\",\"ugy\",\"cpu\",\"abi\",\"ken\",\"kbd\",\"clk\",\"wtm\",\"cpb\",\"kes\",\"gsw\",\"vme\",\"ntk\",\"guf\",\"yuq\",\"abg\",\"mjx\",\"dne\",\"muk\",\"jls\",\"jui\",\"ret\",\"jax\",\"npb\",\"kky\",\"kvf\",\"abv\",\"pek\",\"ljp\",\"rum\",\"xog\",\"nbp\",\"wlm\",\"zkt\",\"tue\",\"fiu\",\"xzm\",\"okx\",\"pro\",\"nlk\",\"npu\",\"xoi\",\"tio\",\"jat\",\"mxg\",\"qve\",\"xwe\",\"nbr\",\"urv\",\"pms\",\"jng\",\"chm\",\"gsn\",\"amb\",\"wym\",\"kkj\",\"kew\",\"dmy\",\"tdf\",\"gss\",\"yxy\",\"sle\",\"xto\",\"mxi\",\"jct\",\"amu\",\"aim\",\"ghk\",\"nyk\",\"sma\",\"boz\",\"pmn\",\"mhf\",\"jni\",\"ekk\",\"swo\",\"due\",\"mnk\",\"wss\",\"tmj\",\"uri\",\"ynq\",\"ind\",\"kcf\",\"whk\",\"nxn\",\"dio\",\"bko\",\"osa\",\"urg\",\"zsa\",\"lrz\",\"cbr\",\"xlo\",\"udu\",\"ste\",\"src\",\"srh\",\"mxv\",\"agf\",\"lsd\",\"mke\",\"gde\",\"gym\",\"kmk\",\"nhm\",\"bou\",\"ham\",\"dmr\",\"tix\",\"atq\",\"heg\",\"xaw\",\"oko\",\"bsa\",\"uhn\",\"spn\",\"amz\",\"nal\",\"prx\",\"nrf\",\"kpq\",\"mba\",\"kfh\",\"kfc\",\"hei\",\"hka\",\"kkp\",\"prt\",\"sps\",\"zko\",\"nby\",\"laq\",\"eit\",\"usk\",\"jao\",\"bob\",\"tit\",\"mjo\",\"mqk\",\"cby\",\"alq\",\"ybn\",\"bfi\",\"ldg\",\"dtd\",\"mrm\",\"xyt\",\"nkd\",\"emp\",\"bje\",\"bhq\",\"crf\",\"cal\",\"bfg\",\"ldi\",\"xcn\",\"krv\",\"zpn\",\"brh\",\"mwe\",\"bte\",\"jul\",\"izh\",\"brc\",\"drq\",\"bgo\",\"tcu\",\"wrx\",\"nqt\",\"ikx\",\"mto\",\"bwo\",\"fpe\",\"nuo\",\"tcb\",\"ikt\",\"zps\",\"tsp\",\"nie\",\"akf\",\"sko\",\"djd\",\"mge\",\"now\",\"kri\",\"ard\",\"cuo\",\"tec\",\"ble\",\"teh\",\"grt\",\"qub\",\"xnk\",\"bma\",\"loc\",\"cow\",\"loh\",\"kgj\",\"cie\",\"asz\",\"abl\",\"soz\",\"kwj\",\"kdu\",\"tgk\",\"mlo\",\"dau\",\"hir\",\"lua\",\"gpa\",\"xke\",\"grx\",\"ole\",\"jiu\",\"rmi\",\"wms\",\"sje\",\"nci\",\"tsj\",\"pym\",\"fln\",\"bud\",\"shq\",\"maw\",\"oma\",\"nnt\",\"khn\",\"fat\",\"zma\",\"fui\",\"tpl\",\"ncg\",\"mid\",\"khs\",\"hmo\",\"rmg\",\"ukp\",\"tcz\",\"ngd\",\"fax\",\"kxm\",\"kzl\",\"wmn\",\"lmd\",\"zle\",\"ksk\",\"jib\",\"ote\",\"orc\",\"hrw\",\"bps\",\"mcs\",\"orh\",\"aar\",\"cnx\",\"rmv\",\"ccg\",\"quz\",\"pss\",\"acp\",\"mvw\",\"gmn\",\"asb\",\"xrm\",\"phk\",\"auh\",\"ogo\",\"myx\",\"kdz\",\"auc\",\"kwp\",\"daz\",\"hwa\",\"sob\",\"sou\",\"fuv\",\"kgp\",\"asu\",\"bpn\",\"mcn\",\"cwd\",\"cnt\",\"ssa\",\"zte\",\"psn\",\"aoa\"]"

Often a schema has coercion rule or default value rules, so after validation the
validated value will be different from the original. To return the validated
(set-as-default, coerced, prefiltered) value:

 my $validator = gen_validator("language::code", {return_type=>'str_errmsg+val'});
 my $res = $validator->($data); # [$errmsg, $validated_val]
 
 # a sample valid data
 $data = "id";
 my $res = $validator->($data); # => ["","id"]
 
 # a sample invalid data
 $data = "";
 my $res = $validator->($data); # => ["Must be one of [\"km\",\"nn\",\"vo\",\"fa\",\"rm\",\"en\",\"aa\",\"oc\",\"mr\",\"ab\",\"tn\",\"lu\",\"oj\",\"cy\",\"sv\",\"lt\",\"sl\",\"tr\",\"lb\",\"la\",\"jv\",\"ps\",\"nr\",\"pi\",\"bm\",\"dv\",\"mn\",\"or\",\"af\",\"ny\",\"ff\",\"ht\",\"ie\",\"tk\",\"ty\",\"ha\",\"cr\",\"su\",\"av\",\"lv\",\"ja\",\"st\",\"id\",\"bh\",\"da\",\"be\",\"my\",\"vi\",\"sa\",\"mk\",\"gd\",\"hu\",\"qu\",\"pl\",\"nd\",\"sw\",\"mh\",\"fo\",\"si\",\"zh\",\"ss\",\"ee\",\"sg\",\"ne\",\"lo\",\"kk\",\"ky\",\"ik\",\"th\",\"te\",\"hi\",\"bn\",\"ch\",\"ce\",\"pt\",\"ho\",\"kr\",\"kj\",\"li\",\"pa\",\"gn\",\"lg\",\"as\",\"so\",\"fi\",\"rn\",\"om\",\"br\",\"ug\",\"kn\",\"ms\",\"ku\",\"ng\",\"iu\",\"mi\",\"se\",\"sh\",\"ru\",\"de\",\"ba\",\"tg\",\"gu\",\"ts\",\"sq\",\"co\",\"ti\",\"am\",\"ia\",\"he\",\"ka\",\"tw\",\"it\",\"os\",\"es\",\"ga\",\"sd\",\"mg\",\"eo\",\"no\",\"to\",\"wa\",\"cs\",\"sm\",\"gv\",\"gl\",\"ae\",\"kl\",\"kv\",\"nv\",\"el\",\"hz\",\"bo\",\"nl\",\"sr\",\"ay\",\"cu\",\"tl\",\"ak\",\"fy\",\"yo\",\"xh\",\"sc\",\"sn\",\"dz\",\"ko\",\"io\",\"ml\",\"hr\",\"ca\",\"ro\",\"uk\",\"bg\",\"ks\",\"ii\",\"ur\",\"ki\",\"rw\",\"is\",\"et\",\"zu\",\"hy\",\"wo\",\"nb\",\"ta\",\"kw\",\"tt\",\"az\",\"na\",\"an\",\"ve\",\"ln\",\"fr\",\"za\",\"ar\",\"yi\",\"sk\",\"cv\",\"eu\",\"mt\",\"kg\",\"bs\",\"fj\",\"bi\",\"ig\",\"uz\",\"itb\",\"loj\",\"kjm\",\"bis\",\"rir\",\"vmz\",\"mun\",\"dks\",\"bgv\",\"anr\",\"zrp\",\"cax\",\"mlg\",\"cui\",\"gwr\",\"kro\",\"skv\",\"cat\",\"ach\",\"tkw\",\"kek\",\"kwc\",\"aup\",\"kwh\",\"bin\",\"nuv\",\"mtv\",\"xyl\",\"sxe\",\"mli\",\"yly\",\"nyw\",\"mus\",\"ilb\",\"aeb\",\"yua\",\"ukh\",\"bgi\",\"skg\",\"mnw\",\"nat\",\"tuz\",\"bwg\",\"mtg\",\"brj\",\"cuv\",\"nug\",\"pes\",\"cts\",\"sea\",\"ski\",\"bgg\",\"til\",\"bcd\",\"ctn\",\"mpd\",\"paf\",\"nui\",\"kmw\",\"wwr\",\"mlv\",\"aeu\",\"mti\",\"ilu\",\"bwi\",\"yty\",\"ipk\",\"nax\",\"prl\",\"itz\",\"vmb\",\"ymd\",\"auj\",\"pkr\",\"ylr\",\"pom\",\"aqr\",\"cpe\",\"rga\",\"zcd\",\"xak\",\"tep\",\"ulm\",\"ogg\",\"rwa\",\"owi\",\"dnu\",\"gia\",\"hmv\",\"lop\",\"cco\",\"vmu\",\"igl\",\"any\",\"xdy\",\"aez\",\"zin\",\"mqw\",\"esh\",\"szy\",\"tub\",\"jbj\",\"nco\",\"mox\",\"hmg\",\"der\",\"dza\",\"rmo\",\"tsh\",\"tsc\",\"syb\",\"wgy\",\"brp\",\"fie\",\"mot\",\"tuu\",\"hmi\",\"aya\",\"oin\",\"xun\",\"tbm\",\"bki\",\"sgg\",\"irr\",\"jos\",\"wkr\",\"bea\",\"bzr\",\"dii\",\"ugh\",\"uro\",\"rof\",\"wom\",\"tmh\",\"xlg\",\"swi\",\"tmc\",\"dig\",\"sgi\",\"bkg\",\"cnl\",\"myl\",\"xtv\",\"swg\",\"xli\",\"nok\",\"nfd\",\"byz\",\"adw\",\"ofo\",\"nnl\",\"tra\",\"dgs\",\"fal\",\"bkv\",\"ysd\",\"div\",\"zyb\",\"kkh\",\"kkc\",\"tpx\",\"sin\",\"var\",\"era\",\"tww\",\"cfd\",\"kzx\",\"cok\",\"kiq\",\"bxe\",\"xtg\",\"pia\",\"swv\",\"xpd\",\"gzi\",\"sis\",\"mdr\",\"tgw\",\"dgn\",\"oyb\",\"xti\",\"dws\",\"mxo\",\"kfp\",\"gom\",\"thr\",\"tpt\",\"gbd\",\"srp\",\"bfo\",\"okv\",\"cjn\",\"eve\",\"abt\",\"mak\",\"cda\",\"gaf\",\"yaq\",\"mji\",\"tve\",\"ldo\",\"mfe\",\"klm\",\"wky\",\"bzy\",\"jav\",\"iry\",\"kuf\",\"mjg\",\"ges\",\"zkv\",\"ksw\",\"cjs\",\"byb\",\"abx\",\"nql\",\"mvk\",\"mdy\",\"hrk\",\"tae\",\"kfj\",\"oki\",\"gvf\",\"wes\",\"rka\",\"rei\",\"jut\",\"phw\",\"lbs\",\"njs\",\"yrb\",\"thy\",\"zkg\",\"mjv\",\"ydg\",\"okg\",\"njn\",\"wen\",\"led\",\"lbn\",\"ktm\",\"pwr\",\"apm\",\"zea\",\"wrl\",\"ikl\",\"ram\",\"nda\",\"vay\",\"waf\",\"reg\",\"tod\",\"kfy\",\"ssq\",\"lwm\",\"mdj\",\"mwu\",\"btu\",\"mxx\",\"iin\",\"mgb\",\"ksn\",\"nib\",\"tce\",\"kzo\",\"epo\",\"nmm\",\"khk\",\"mgu\",\"tpo\",\"nbc\",\"lgm\",\"vaj\",\"nbh\",\"niu\",\"gew\",\"mxt\",\"hml\",\"mwb\",\"kss\",\"mck\",\"ahb\",\"bpk\",\"pxm\",\"cmm\",\"gcf\",\"xkb\",\"zmq\",\"knf\",\"wew\",\"cbc\",\"lbw\",\"urt\",\"igg\",\"srr\",\"sha\",\"kym\",\"cib\",\"bzj\",\"dae\",\"urx\",\"omq\",\"kde\",\"mmf\",\"phn\",\"xku\",\"otz\",\"lif\",\"que\",\"owl\",\"blb\",\"luq\",\"bgl\",\"tig\",\"adn\",\"sju\",\"opk\",\"mgz\",\"jie\",\"jow\",\"niz\",\"vap\",\"olu\",\"pdu\",\"zzj\",\"juo\",\"prg\",\"aja\",\"dbm\",\"thp\",\"bmq\",\"tii\",\"xoo\",\"hum\",\"kfr\",\"zpk\",\"ads\",\"pri\",\"mtl\",\"bwl\",\"zlu\",\"nul\",\"gkp\",\"mdp\",\"dmc\",\"sjb\",\"mwz\",\"btz\",\"bft\",\"sbf\",\"umn\",\"abo\",\"xkz\",\"dgw\",\"xns\",\"otu\",\"kqf\",\"tgn\",\"cul\",\"dhr\",\"bzp\",\"sad\",\"ase\",\"twn\",\"dra\",\"tgs\",\"soe\",\"tiv\",\"siw\",\"ums\",\"bfx\",\"sry\",\"otb\",\"ztu\",\"mll\",\"xnn\",\"eiv\",\"dww\",\"blz\",\"dep\",\"xgb\",\"enu\",\"okl\",\"nao\",\"bha\",\"ytp\",\"lva\",\"tnu\",\"ojb\",\"wrg\",\"wdu\",\"grv\",\"rel\",\"ame\",\"slu\",\"spk\",\"ovd\",\"jal\",\"nqg\",\"tnb\",\"xxt\",\"uss\",\"izr\",\"wri\",\"enb\",\"iki\",\"xgu\",\"brr\",\"ala\",\"gdb\",\"mkb\",\"mqs\",\"hih\",\"hbu\",\"auy\",\"duu\",\"zad\",\"aqp\",\"nvo\",\"fon\",\"mjl\",\"yyr\",\"krt\",\"pkp\",\"laa\",\"stb\",\"gri\",\"bjz\",\"ory\",\"lor\",\"ude\",\"stu\",\"xmf\",\"krx\",\"ybk\",\"fos\",\"bsq\",\"cao\",\"eza\",\"ppq\",\"mbq\",\"kpa\",\"anj\",\"dub\",\"grg\",\"mqn\",\"hbb\",\"ikv\",\"ter\",\"ziw\",\"tza\",\"ata\",\"wrv\",\"gdu\",\"mku\",\"gwj\",\"lkm\",\"dsh\",\"nyn\",\"ghn\",\"csm\",\"fai\",\"biw\",\"boe\",\"nni\",\"rmt\",\"clw\",\"llh\",\"bry\",\"llc\",\"tqu\",\"slz\",\"tkn\",\"nct\",\"nng\",\"rmx\",\"pcf\",\"moo\",\"fag\",\"bbf\",\"tnz\",\"msf\",\"ncx\",\"tqb\",\"tks\",\"fut\",\"ghs\",\"nys\",\"qvz\",\"jbr\",\"ntw\",\"yna\",\"szp\",\"bad\",\"myv\",\"xtl\",\"smq\",\"mkz\",\"wsk\",\"mns\",\"dil\",\"bkl\",\"kmn\",\"anp\",\"nnv\",\"cng\",\"loy\",\"swl\",\"myg\",\"lth\",\"tey\",\"lre\",\"ltc\",\"bjb\",\"bju\",\"cni\",\"nxk\",\"nsm\",\"aur\",\"bvd\",\"orr\",\"mnn\",\"duz\",\"aac\",\"zha\",\"aah\",\"nlw\",\"hsl\",\"rhg\",\"kms\",\"seu\",\"huk\",\"etr\",\"pao\",\"ysc\",\"yub\",\"aea\",\"ila\",\"ttr\",\"aki\",\"jrt\",\"zpm\",\"kkd\",\"bdw\",\"ayz\",\"jpa\",\"opm\",\"xph\",\"xpc\",\"akg\",\"kvl\",\"wul\",\"seb\",\"dyr\",\"axe\",\"mzw\",\"typ\",\"tmd\",\"kby\",\"xhv\",\"xmt\",\"inj\",\"akv\",\"gul\",\"ita\",\"tlr\",\"obt\",\"sch\",\"unx\",\"nkp\",\"zbt\",\"dtp\",\"xmx\",\"krf\",\"snr\",\"kal\",\"giz\",\"jmn\",\"khm\",\"sya\",\"nmk\",\"fuf\",\"pni\",\"pyn\",\"tua\",\"wib\",\"bbt\",\"dyy\",\"afo\",\"yuz\",\"lgk\",\"kxs\",\"ayu\",\"bbx\",\"ayb\",\"rmf\",\"kxn\",\"png\",\"ncf\",\"wmm\",\"msx\",\"toc\",\"yil\",\"toh\",\"ptw\",\"wbh\",\"wiu\",\"lec\",\"tty\",\"jms\",\"leh\",\"sez\",\"pys\",\"pnv\",\"sny\",\"lsp\",\"gmm\",\"xrn\",\"kyk\",\"dna\",\"yej\",\"giu\",\"gbh\",\"new\",\"xel\",\"vma\",\"psm\",\"nkj\",\"inp\",\"bpm\",\"mcm\",\"gib\",\"sqr\",\"suj\",\"plw\",\"tly\",\"rgu\",\"tyj\",\"kbr\",\"wyn\",\"yae\",\"pib\",\"tdi\",\"hov\",\"lmp\",\"chn\",\"gsm\",\"wni\",\"bnr\",\"mxf\",\"agv\",\"mip\",\"ngp\",\"mhi\",\"glw\",\"trb\",\"pmm\",\"ndz\",\"wng\",\"tru\",\"awv\",\"bqy\",\"tdg\",\"ain\",\"csk\",\"bup\",\"kcv\",\"fro\",\"uvl\",\"mhg\",\"mph\",\"bcc\",\"mpc\",\"bch\",\"wtw\",\"rnn\",\"pbc\",\"mfq\",\"djj\",\"pbh\",\"piu\",\"lit\",\"gng\",\"tdv\",\"hoi\",\"arj\",\"nsk\",\"nxm\",\"awg\",\"cdz\",\"knx\",\"taq\",\"agi\",\"mmt\",\"kcg\",\"beb\",\"uby\",\"kwd\",\"knt\",\"gyn\",\"ihb\",\"awi\",\"beu\",\"nhn\",\"roo\",\"mmx\",\"gni\",\"urf\",\"lix\",\"kci\",\"acd\",\"wlw\",\"agg\",\"gct\",\"piz\",\"wao\",\"bqr\",\"rkb\",\"spm\",\"yra\",\"tsd\",\"yww\",\"ids\",\"crv\",\"nrg\",\"hlt\",\"buj\",\"ndu\",\"han\",\"trz\",\"bny\",\"ndb\",\"ygw\",\"has\",\"mij\",\"ngj\",\"lmj\",\"gvo\",\"nri\",\"ise\",\"kie\",\"mrn\",\"onr\",\"bxq\",\"kow\",\"sbx\",\"hvn\",\"zch\",\"ybm\",\"kuo\",\"qxr\",\"ymh\",\"vko\",\"mel\",\"kqx\",\"cri\",\"ymc\",\"bez\",\"bya\",\"ibo\",\"gao\",\"htx\",\"xcm\",\"kqt\",\"gqi\",\"och\",\"mrs\",\"ubr\",\"bff\",\"sbt\",\"crg\",\"arp\",\"shu\",\"ulk\",\"qxp\",\"lio\",\"hid\",\"mmo\",\"xka\",\"jiq\",\"lue\",\"zac\",\"aha\",\"gpe\",\"zah\",\"onp\",\"haw\",\"oac\",\"yiv\",\"arr\",\"bla\",\"tjp\",\"kno\",\"ywn\",\"ygs\",\"xam\",\"cia\",\"shb\",\"pok\",\"bme\",\"xeg\",\"ajz\",\"tpf\",\"pnl\",\"kos\",\"yig\",\"mga\",\"bqp\",\"nwy\",\"nia\",\"buy\",\"djr\",\"kzf\",\"hyx\",\"qwm\",\"yii\",\"ngy\",\"mwa\",\"bta\",\"miy\",\"bnj\",\"soq\",\"mrw\",\"asq\",\"kon\",\"lmy\",\"txc\",\"txh\",\"frt\",\"tcq\",\"kem\",\"rnw\",\"dru\",\"ary\",\"ddg\",\"kag\",\"aoe\",\"aiw\",\"gui\",\"sse\",\"kut\",\"tjj\",\"gax\",\"hto\",\"vkt\",\"kai\",\"aad\",\"ddi\",\"onj\",\"zuy\",\"kux\",\"gat\",\"kjk\",\"kqo\",\"ota\",\"wuv\",\"kvv\",\"chw\",\"lhh\",\"abf\",\"gug\",\"shz\",\"drb\",\"bvh\",\"sbo\",\"bvc\",\"run\",\"wln\",\"lld\",\"mir\",\"sds\",\"wat\",\"ngr\",\"zme\",\"lmr\",\"bnp\",\"akl\",\"zla\",\"xso\",\"nhw\",\"kvg\",\"quq\",\"kav\",\"sja\",\"bah\",\"bac\",\"bqj\",\"ola\",\"pda\",\"daq\",\"avd\",\"wls\",\"kvi\",\"bur\",\"rus\",\"kdq\",\"nwr\",\"ypo\",\"xbe\",\"wax\",\"sdn\",\"aju\",\"gjn\",\"lab\",\"sta\",\"ich\",\"cbd\",\"klk\",\"cen\",\"snj\",\"ptn\",\"doh\",\"doc\",\"kpu\",\"obo\",\"lsr\",\"caf\",\"kxw\",\"boq\",\"fss\",\"dty\",\"xmo\",\"mei\",\"atu\",\"mka\",\"gda\",\"crl\",\"yey\",\"atb\",\"hba\",\"kbp\",\"dua\",\"tyy\",\"kpb\",\"lau\",\"tlj\",\"suy\",\"mam\",\"jmw\",\"sla\",\"cky\",\"ktk\",\"pln\",\"pug\",\"ykn\",\"nes\",\"rji\",\"rak\",\"sme\",\"xga\",\"mev\",\"alu\",\"bhb\",\"apk\",\"alb\",\"ena\",\"jek\",\"nbd\",\"hrm\",\"bhu\",\"nen\",\"lvu\",\"qva\",\"xwa\",\"xiy\",\"tna\",\"xrw\",\"uon\",\"nrl\",\"pax\",\"ttj\",\"naf\",\"pls\",\"rjg\",\"azj\",\"pui\",\"bja\",\"tlp\",\"laz\",\"nkr\",\"nzi\",\"hol\",\"sac\",\"sqj\",\"sah\",\"sur\",\"bdn\",\"tyr\",\"kbj\",\"agl\",\"atz\",\"gnl\",\"ohu\",\"bds\",\"tbk\",\"zhb\",\"yer\",\"snp\",\"kpz\",\"lfn\",\"wok\",\"amq\",\"dtr\",\"lsy\",\"ebk\",\"kcl\",\"ttp\",\"xir\",\"iff\",\"tdl\",\"hke\",\"mzs\",\"mso\",\"mhl\",\"vgt\",\"gok\",\"svc\",\"ynu\",\"bbo\",\"bhz\",\"iya\",\"aft\",\"alz\",\"zoq\",\"ckr\",\"dmd\",\"ppe\",\"mbe\",\"com\",\"mzn\",\"bse\",\"nbn\",\"ned\",\"ibb\",\"jbk\",\"gab\",\"kuu\",\"vku\",\"bnm\",\"drt\",\"nqq\",\"lda\",\"byf\",\"pld\",\"cdo\",\"bfa\",\"kub\",\"nbs\",\"bsv\",\"spy\",\"gau\",\"mbv\",\"ksh\",\"pmr\",\"ksc\",\"ibu\",\"ndo\",\"kep\",\"hea\",\"yal\",\"xaj\",\"grq\",\"xcy\",\"nxr\",\"phh\",\"auk\",\"cbs\",\"bsg\",\"wau\",\"cze\",\"wab\",\"cbn\",\"uve\",\"bsi\",\"wsr\",\"ppi\",\"yby\",\"mbi\",\"kil\",\"emw\",\"gaz\",\"brk\",\"rob\",\"dmn\",\"sht\",\"smv\",\"pmy\",\"mee\",\"bqm\",\"tmw\",\"spr\",\"jyy\",\"mzd\",\"nnq\",\"shx\",\"lnl\",\"ura\",\"kuz\",\"adh\",\"rou\",\"vkz\",\"beo\",\"dms\",\"osi\",\"kkw\",\"mxa\",\"tek\",\"xnh\",\"twc\",\"smi\",\"lok\",\"twh\",\"jna\",\"tjm\",\"tgh\",\"pio\",\"waz\",\"tgc\",\"ero\",\"cnq\",\"umc\",\"xap\",\"pue\",\"kej\",\"xcr\",\"bdd\",\"smg\",\"tro\",\"zpr\",\"uuu\",\"axl\",\"zmv\",\"azm\",\"ttm\",\"ssi\",\"rwo\",\"vmf\",\"aoi\",\"gue\",\"lpe\",\"kod\",\"kfk\",\"dym\",\"kae\",\"dde\",\"cop\",\"ush\",\"zhx\",\"khy\",\"aog\",\"ssg\",\"oht\",\"omv\",\"fly\",\"mqh\",\"mqc\",\"maj\",\"iml\",\"ayo\",\"tlm\",\"zmi\",\"dsn\",\"tuf\",\"afu\",\"ssv\",\"omg\",\"psy\",\"elm\",\"tsw\",\"xbg\",\"mcy\",\"nop\",\"bpy\",\"dzo\",\"rma\",\"zmg\",\"nca\",\"paz\",\"snm\",\"hin\",\"afb\",\"xbi\",\"gmy\",\"kve\",\"omi\",\"coj\",\"att\",\"kgw\",\"flr\",\"lui\",\"avn\",\"opy\",\"lls\",\"khr\",\"lax\",\"bmg\",\"kpt\",\"prq\",\"avs\",\"lln\",\"hrp\",\"kra\",\"lat\",\"zpy\",\"kpx\",\"acw\",\"mvp\",\"ghc\",\"ghh\",\"bmi\",\"nyh\",\"nyc\",\"tzx\",\"kww\",\"tiq\",\"atx\",\"lug\",\"aan\",\"gld\",\"rpn\",\"mnc\",\"mnh\",\"bhx\",\"ukw\",\"alt\",\"luv\",\"ium\",\"gmr\",\"pau\",\"lts\",\"seo\",\"ltn\",\"kbm\",\"noj\",\"pab\",\"srk\",\"bpr\",\"mcr\",\"sqm\",\"aas\",\"afz\",\"bht\",\"bmv\",\"alx\",\"map\",\"psr\",\"kmh\",\"kmc\",\"bbu\",\"bih\",\"nhd\",\"yno\",\"gyd\",\"kwn\",\"kgs\",\"kiv\",\"msu\",\"ayt\",\"ifa\",\"lsm\",\"swq\",\"acn\",\"bkq\",\"diq\",\"moa\",\"clc\",\"sml\",\"clh\",\"mar\",\"acs\",\"psp\",\"mcp\",\"noy\",\"bpp\",\"inm\",\"nre\",\"msb\",\"kws\",\"kgn\",\"bbb\",\"muh\",\"muc\",\"pcb\",\"uks\",\"khp\",\"git\",\"unz\",\"bjf\",\"lng\",\"kii\",\"zsl\",\"isi\",\"aid\",\"tfr\",\"yon\",\"jjr\",\"peh\",\"ctc\",\"cre\",\"rnd\",\"cth\",\"zpj\",\"xtq\",\"isg\",\"lyn\",\"kig\",\"lni\",\"chd\",\"nlc\",\"mvr\",\"gix\",\"pgk\",\"xmz\",\"aaw\",\"jdg\",\"alo\",\"yux\",\"slf\",\"wne\",\"nor\",\"yai\",\"jru\",\"tde\",\"mjq\",\"may\",\"set\",\"mcj\",\"mhe\",\"bpj\",\"ffm\",\"nkm\",\"xgf\",\"mrd\",\"enf\",\"dtm\",\"yag\",\"msz\",\"mbl\",\"yut\",\"ppl\",\"bzk\",\"irk\",\"jrb\",\"bsl\",\"bho\",\"naa\",\"ato\",\"idd\",\"xmu\",\"tzo\",\"stf\",\"hiw\",\"thk\",\"yav\",\"obu\",\"ess\",\"hoe\",\"caa\",\"mdk\",\"kpo\",\"zpp\",\"age\",\"mvy\",\"gdf\",\"mkf\",\"tss\",\"lao\",\"dlm\",\"gne\",\"cor\",\"tsn\",\"awe\",\"duf\",\"zbu\",\"ckm\",\"xmb\",\"esn\",\"kce\",\"had\",\"khj\",\"jaq\",\"unu\",\"xsu\",\"rzh\",\"uiv\",\"hlu\",\"ndt\",\"cuq\",\"ems\",\"pne\",\"zum\",\"ypb\",\"tms\",\"rkt\",\"oum\",\"jua\",\"aqk\",\"xar\",\"mlq\",\"tmn\",\"dmw\",\"key\",\"xsb\",\"emn\",\"arm\",\"ndx\",\"ugn\",\"hlb\",\"djm\",\"dgc\",\"dgh\",\"htu\",\"knz\",\"kqb\",\"spp\",\"kks\",\"sbb\",\"dro\",\"lul\",\"bgq\",\"pmj\",\"sih\",\"nuq\",\"sbu\",\"mtq\",\"bwq\",\"uig\",\"dek\",\"liz\",\"skq\",\"mmz\",\"kqu\",\"bml\",\"kkn\",\"aba\",\"vun\",\"trx\",\"xay\",\"pqe\",\"imi\",\"zml\",\"tpa\",\"nif\",\"ake\",\"mgf\",\"ycl\",\"ker\",\"kza\",\"xhe\",\"ypz\",\"vot\",\"ggk\",\"pix\",\"mwf\",\"ank\",\"btf\",\"ert\",\"xrd\",\"pit\",\"gec\",\"oml\",\"geh\",\"ybj\",\"trt\",\"ljw\",\"cjh\",\"nbw\",\"bex\",\"knb\",\"mmu\",\"kqz\",\"kxd\",\"gsp\",\"wec\",\"liu\",\"ngm\",\"cbw\",\"lbc\",\"weh\",\"xkf\",\"mim\",\"aol\",\"njh\",\"sho\",\"xdk\",\"sbz\",\"ssl\",\"blf\",\"hmq\",\"axg\",\"nwm\",\"jmd\",\"bum\",\"lib\",\"mmb\",\"knu\",\"bet\",\"pdt\",\"ijo\",\"wor\",\"nne\",\"boi\",\"jks\",\"gva\",\"dtk\",\"meq\",\"ebr\",\"mxz\",\"kly\",\"sjt\",\"zas\",\"bzm\",\"bxl\",\"olt\",\"tbr\",\"lrv\",\"heb\",\"kyj\",\"suk\",\"bog\",\"nkk\",\"yro\",\"fod\",\"waa\",\"ndf\",\"apy\",\"ldb\",\"bov\",\"ray\",\"vam\",\"txn\",\"dlk\",\"koc\",\"koh\",\"bfu\",\"cdf\",\"kty\",\"nmj\",\"ztx\",\"byo\",\"gaa\",\"lri\",\"iba\",\"ott\",\"gor\",\"thm\",\"ztt\",\"dbp\",\"kua\",\"bfb\",\"hup\",\"vka\",\"otx\",\"puq\",\"urz\",\"lrg\",\"mye\",\"rbl\",\"hch\",\"mdm\",\"txs\",\"lhn\",\"trf\",\"amg\",\"bvn\",\"tal\",\"glh\",\"glc\",\"mxb\",\"mnd\",\"mwt\",\"btt\",\"nix\",\"pif\",\"ofu\",\"hya\",\"mgt\",\"btx\",\"kyp\",\"nit\",\"mxu\",\"woy\",\"ami\",\"lxm\",\"lzl\",\"kmd\",\"tby\",\"wth\",\"ike\",\"bcw\",\"mpw\",\"lhs\",\"klr\",\"bef\",\"udg\",\"xzh\",\"sdh\",\"goy\",\"uru\",\"sdc\",\"ozm\",\"jer\",\"roa\",\"xkx\",\"gre\",\"huj\",\"blt\",\"tvl\",\"ban\",\"dbj\",\"xkt\",\"oog\",\"mfl\",\"blx\",\"nyd\",\"aht\",\"udi\",\"amv\",\"bas\",\"bfz\",\"rar\",\"pwm\",\"apr\",\"nmp\",\"urb\",\"wlh\",\"wlc\",\"ruc\",\"ruh\",\"rsb\",\"tkd\",\"lkj\",\"jii\",\"rmu\",\"gwm\",\"tqt\",\"phd\",\"ncu\",\"anm\",\"sfb\",\"dos\",\"vwa\",\"wif\",\"cgk\",\"uly\",\"fub\",\"iyx\",\"kjr\",\"rim\",\"fuu\",\"poy\",\"sov\",\"tow\",\"asv\",\"jig\",\"nsp\",\"afa\",\"tuo\",\"ncb\",\"pre\",\"rmb\",\"don\",\"pth\",\"lew\",\"syo\",\"wbw\",\"bjx\",\"jiv\",\"csp\",\"asg\",\"buk\",\"lcf\",\"sog\",\"neh\",\"nec\",\"gbw\",\"soi\",\"lmk\",\"ykh\",\"dno\",\"asi\",\"ksd\",\"bjt\",\"yiq\",\"xdm\",\"plc\",\"plh\",\"mik\",\"sxl\",\"ngk\",\"krz\",\"tnt\",\"twd\",\"slx\",\"guq\",\"yuf\",\"aqm\",\"wdt\",\"sas\",\"ylm\",\"por\",\"tci\",\"ark\",\"xxb\",\"xnd\",\"xwt\",\"dav\",\"ilo\",\"umd\",\"enx\",\"sef\",\"bdc\",\"fil\",\"bdh\",\"quv\",\"tnx\",\"kjy\",\"slt\",\"kaq\",\"ncz\",\"tgd\",\"paa\",\"san\",\"rmz\",\"xpw\",\"npl\",\"tcg\",\"kru\",\"dem\",\"kdg\",\"dag\",\"mzc\",\"mzh\",\"scw\",\"svs\",\"dut\",\"mkx\",\"gdx\",\"qug\",\"ito\",\"dai\",\"add\",\"kvq\",\"kdi\",\"dux\",\"mkt\",\"gdt\",\"djk\",\"qui\",\"stt\",\"utr\",\"ige\",\"csj\",\"krb\",\"mfg\",\"tav\",\"saw\",\"xpn\",\"caz\",\"mhq\",\"tdq\",\"tvi\",\"mje\",\"dgd\",\"yss\",\"vmx\",\"dnt\",\"ysn\",\"izm\",\"ixl\",\"udl\",\"brm\",\"mfi\",\"utp\",\"xps\",\"sid\",\"csy\",\"bjo\",\"bqk\",\"tai\",\"agq\",\"syx\",\"yde\",\"oke\",\"tux\",\"tqo\",\"znk\",\"jod\",\"pop\",\"ifb\",\"ree\",\"nsy\",\"scs\",\"bba\",\"kcq\",\"lkr\",\"jae\",\"pca\",\"naz\",\"tem\",\"xud\",\"aml\",\"iyo\",\"scn\",\"mfv\",\"tag\",\"ifu\",\"onk\",\"mou\",\"tut\",\"kjj\",\"lom\",\"gnq\",\"len\",\"wed\",\"njd\",\"vrs\",\"dow\",\"cab\",\"lrl\",\"duo\",\"zba\",\"bxv\",\"swe\",\"hbo\",\"ton\",\"csr\",\"itx\",\"kxc\",\"laf\",\"una\",\"kxh\",\"sge\",\"vel\",\"mko\",\"gdo\",\"xma\",\"kpf\",\"jbm\",\"itt\",\"tos\",\"cau\",\"fer\",\"les\",\"jmc\",\"wbs\",\"xle\",\"sto\",\"bnk\",\"bhf\",\"moz\",\"kjp\",\"crq\",\"bol\",\"tno\",\"lky\",\"uzb\",\"bxi\",\"qvo\",\"xwo\",\"gbs\",\"eno\",\"hdn\",\"nau\",\"ian\",\"fnb\",\"ged\",\"gbn\",\"nab\",\"jpx\",\"jra\",\"orm\",\"hds\",\"nsr\",\"aum\",\"slo\",\"bxg\",\"xte\",\"alf\",\"pze\",\"goj\",\"sba\",\"pbn\",\"akq\",\"ped\",\"oge\",\"ctd\",\"bcn\",\"mpn\",\"byt\",\"dby\",\"oto\",\"kqa\",\"huy\",\"cpi\",\"aic\",\"aih\",\"abu\",\"ktp\",\"byx\",\"abb\",\"lgr\",\"kdl\",\"hnj\",\"dal\",\"rap\",\"cpg\",\"app\",\"nmr\",\"bvw\",\"bcs\",\"mps\",\"pbs\",\"chh\",\"chc\",\"qul\",\"srm\",\"kzz\",\"juu\",\"ypa\",\"pdo\",\"kbk\",\"cld\",\"nhh\",\"bid\",\"nhc\",\"olo\",\"tpz\",\"cmr\",\"sjo\",\"sqk\",\"tcl\",\"npg\",\"klp\",\"ntd\",\"baw\",\"xsa\",\"hla\",\"mud\",\"npi\",\"jub\",\"hme\",\"tbj\",\"kyr\",\"raj\",\"apj\",\"kna\",\"zaw\",\"idc\",\"pnq\",\"ktj\",\"nmy\",\"blo\",\"sxg\",\"aho\",\"abz\",\"asl\",\"xko\",\"hac\",\"ttk\",\"mma\",\"hah\",\"sol\",\"lia\",\"mle\",\"kfm\",\"hur\",\"dbr\",\"gop\",\"kzb\",\"tbp\",\"bge\",\"dhm\",\"jil\",\"kyy\",\"yms\",\"klj\",\"snk\",\"mwo\",\"bto\",\"ymn\",\"tlk\",\"ske\",\"mgo\",\"nio\",\"tpu\",\"wca\",\"mrc\",\"mte\",\"bwe\",\"mrh\",\"kzu\",\"elk\",\"hvc\",\"nue\",\"kwv\",\"dml\",\"hve\",\"siq\",\"nuh\",\"ify\",\"nuc\",\"mre\",\"mtc\",\"bwc\",\"nsb\",\"bwh\",\"yxl\",\"mth\",\"lyg\",\"isn\",\"kin\",\"skc\",\"moy\",\"bek\",\"skh\",\"acv\",\"yoi\",\"nar\",\"lns\",\"ukg\",\"rmp\",\"noa\",\"tdd\",\"bro\",\"lnn\",\"wnd\",\"uki\",\"nsu\",\"kgv\",\"yog\",\"mhd\",\"kis\",\"bjm\",\"bgc\",\"erk\",\"kwi\",\"awd\",\"ant\",\"mlc\",\"mlh\",\"coa\",\"tqm\",\"hhy\",\"bmw\",\"gwt\",\"gnd\",\"kcd\",\"hae\",\"trk\",\"aci\",\"rit\",\"kgg\",\"csu\",\"krj\",\"csb\",\"teo\",\"hod\",\"luw\",\"cuh\",\"cuc\",\"ide\",\"kwg\",\"ukv\",\"agd\",\"car\",\"kgi\",\"anx\",\"ggt\",\"loo\",\"ccp\",\"gwx\",\"tsg\",\"hmc\",\"omw\",\"fuj\",\"maa\",\"hmh\",\"nsz\",\"nbl\",\"ljl\",\"xbw\",\"esg\",\"dum\",\"lkb\",\"gdm\",\"mkm\",\"lku\",\"ncj\",\"jbo\",\"nhe\",\"tsi\",\"gye\",\"lbq\",\"nay\",\"stm\",\"zmw\",\"mor\",\"esi\",\"aqt\",\"ccj\",\"che\",\"tnm\",\"zgh\",\"yan\",\"geq\",\"mva\",\"zro\",\"qvm\",\"enm\",\"krp\",\"cay\",\"pkt\",\"hra\",\"xgm\",\"hhr\",\"aie\",\"nkf\",\"csz\",\"cbl\",\"ssw\",\"ndk\",\"oro\",\"tsv\",\"crd\",\"ogc\",\"hgw\",\"auo\",\"slm\",\"efa\",\"yas\",\"pzh\",\"xtc\",\"kyz\",\"xre\",\"mdx\",\"xth\",\"mxj\",\"nlq\",\"kkg\",\"otm\",\"jnj\",\"avl\",\"tpr\",\"smw\",\"lll\",\"juy\",\"ztm\",\"tht\",\"tmv\",\"mdt\",\"xhd\",\"uis\",\"akd\",\"peq\",\"vls\",\"djf\",\"kki\",\"kea\",\"kzr\",\"xlc\",\"dbb\",\"sro\",\"tmg\",\"pye\",\"hsh\",\"nmz\",\"hub\",\"aal\",\"bfp\",\"bzt\",\"dpp\",\"emg\",\"olm\",\"yuk\",\"lgz\",\"sgc\",\"sgh\",\"sjm\",\"muq\",\"bzx\",\"irx\",\"biq\",\"ldp\",\"swh\",\"tmi\",\"zlm\",\"swc\",\"urj\",\"huu\",\"sek\",\"bkh\",\"aby\",\"bkc\",\"dic\",\"dbu\",\"dih\",\"kkv\",\"emi\",\"kyb\",\"cim\",\"mbw\",\"jur\",\"tpy\",\"zkh\",\"rgk\",\"xaa\",\"bsw\",\"kzy\",\"buf\",\"jah\",\"jac\",\"blm\",\"ahm\",\"pnd\",\"mif\",\"ngf\",\"kfo\",\"lmf\",\"mxp\",\"xor\",\"kyu\",\"rwk\",\"okc\",\"okh\",\"wik\",\"dsl\",\"dho\",\"nmb\",\"huz\",\"imn\",\"pfl\",\"lgb\",\"urp\",\"abr\",\"qwa\",\"lwu\",\"ldj\",\"mwm\",\"btm\",\"mgm\",\"ayk\",\"lgu\",\"nim\",\"mjc\",\"mjh\",\"nmu\",\"bfj\",\"ims\",\"hil\",\"hkn\",\"abp\",\"nbv\",\"cbg\",\"ktb\",\"bss\",\"pps\",\"srt\",\"mbs\",\"urr\",\"apb\",\"rab\",\"jeu\",\"goz\",\"iro\",\"lvk\",\"bzo\",\"mqq\",\"apu\",\"cbi\",\"jeb\",\"rau\",\"alk\",\"uha\",\"spa\",\"bsn\",\"hks\",\"mze\",\"yrm\",\"mbn\",\"bfy\",\"srx\",\"med\",\"ktu\",\"ppn\",\"lji\",\"nbi\",\"klb\",\"vao\",\"tpj\",\"bde\",\"bym\",\"pud\",\"jup\",\"tsl\",\"tbz\",\"cbv\",\"nbg\",\"xop\",\"bnf\",\"mxr\",\"atk\",\"yba\",\"esl\",\"gko\",\"kpk\",\"klu\",\"ple\",\"ktz\",\"hnu\",\"dmv\",\"kwl\",\"apz\",\"jya\",\"oym\",\"smn\",\"bfr\",\"raz\",\"acl\",\"gob\",\"pma\",\"kmq\",\"sms\",\"gou\",\"ury\",\"nzd\",\"mnq\",\"nee\",\"ynk\",\"kgl\",\"abj\",\"wfg\",\"dhx\",\"prc\",\"mxy\",\"prh\",\"klz\",\"kfx\",\"ebu\",\"oss\",\"tkq\",\"wsa\",\"nyq\",\"tbu\",\"tih\",\"yol\",\"tic\",\"dmg\",\"her\",\"kzp\",\"yxg\",\"bqf\",\"fse\",\"pwo\",\"nxa\",\"tpp\",\"jqr\",\"kft\",\"wob\",\"osn\",\"ukl\",\"zmn\",\"krr\",\"lox\",\"snf\",\"wle\",\"ano\",\"rue\",\"dsi\",\"wma\",\"caj\",\"ycn\",\"hig\",\"tex\",\"grh\",\"grc\",\"tet\",\"tum\",\"xbn\",\"hii\",\"xed\",\"sde\",\"utu\",\"zms\",\"fla\",\"lot\",\"omn\",\"phq\",\"tlf\",\"kha\",\"sym\",\"yid\",\"wrh\",\"fuy\",\"ulb\",\"ikh\",\"wgo\",\"mca\",\"ssn\",\"bpa\",\"psa\",\"brx\",\"aon\",\"ksq\",\"sss\",\"kjz\",\"vmm\",\"mop\",\"wwo\",\"xxr\",\"rmy\",\"gle\",\"aos\",\"brt\",\"gma\",\"ulu\",\"yaw\",\"xdo\",\"shk\",\"ttf\",\"naj\",\"kad\",\"ddd\",\"aai\",\"tgq\",\"ltg\",\"aux\",\"myh\",\"myc\",\"ylo\",\"cnc\",\"pko\",\"avv\",\"lnw\",\"cnh\",\"orx\",\"opa\",\"ilm\",\"tml\",\"aem\",\"zpa\",\"lti\",\"ort\",\"ccr\",\"aag\",\"xnq\",\"kry\",\"cap\",\"hpo\",\"twq\",\"kiw\",\"koe\",\"gud\",\"aut\",\"nap\",\"aav\",\"bms\",\"ncr\",\"llg\",\"adq\",\"avi\",\"lun\",\"wud\",\"gpn\",\"kvd\",\"kju\",\"moj\",\"lus\",\"jbt\",\"itm\",\"kjb\",\"lli\",\"fur\",\"kkl\",\"bmn\",\"poz\",\"fah\",\"nnc\",\"nnh\",\"lof\",\"bvl\",\"scg\",\"xpv\",\"xmr\",\"hrz\",\"tan\",\"quw\",\"mvz\",\"sgd\",\"lhl\",\"tlt\",\"obr\",\"snx\",\"lso\",\"xld\",\"cob\",\"daw\",\"tef\",\"kdw\",\"lzn\",\"bkd\",\"did\",\"lbe\",\"cou\",\"ino\",\"sci\",\"elx\",\"tas\",\"nhq\",\"unr\",\"tlx\",\"csa\",\"cje\",\"evn\",\"xpi\",\"nou\",\"jrr\",\"ysg\",\"ttt\",\"azt\",\"xtd\",\"nsa\",\"tvn\",\"afp\",\"bal\",\"fkv\",\"cfg\",\"brf\",\"maz\",\"mfs\",\"chq\",\"ett\",\"msy\",\"xhc\",\"tvs\",\"tcw\",\"mfn\",\"akc\",\"bby\",\"akh\",\"xpg\",\"etx\",\"scv\",\"wxa\",\"nob\",\"aiq\",\"alm\",\"lev\",\"wbv\",\"mvb\",\"sqx\",\"gbg\",\"auf\",\"suo\",\"asw\",\"mrq\",\"bxn\",\"eyo\",\"coz\",\"kbx\",\"yrk\",\"nko\",\"sow\",\"sie\",\"tov\",\"jvd\",\"bxs\",\"dto\",\"gbi\",\"iai\",\"zal\",\"kbt\",\"xmy\",\"ivv\",\"mvu\",\"sqt\",\"mjd\",\"bhm\",\"dge\",\"hru\",\"atm\",\"tzm\",\"lka\",\"wbi\",\"lei\",\"bbr\",\"haq\",\"jad\",\"pap\",\"msr\",\"zkd\",\"uya\",\"kpm\",\"mab\",\"toi\",\"lam\",\"mau\",\"gbv\",\"okd\",\"ydd\",\"tog\",\"cko\",\"pnc\",\"pnh\",\"byk\",\"noz\",\"kcc\",\"nps\",\"kch\",\"bcv\",\"mpv\",\"pbv\",\"aek\",\"mmr\",\"sby\",\"ilk\",\"vkp\",\"awh\",\"lir\",\"awc\",\"kup\",\"dba\",\"gnh\",\"gnc\",\"kqy\",\"gap\",\"agc\",\"agh\",\"fin\",\"gcr\",\"oie\",\"sal\",\"hoh\",\"npn\",\"hoc\",\"knr\",\"aro\",\"srf\",\"wap\",\"djo\",\"skd\",\"mpi\",\"bci\",\"pbi\",\"itk\",\"amw\",\"nud\",\"bnt\",\"bwd\",\"mtd\",\"cpn\",\"keu\",\"pbg\",\"bcg\",\"keb\",\"mpg\",\"bnx\",\"xsy\",\"mhc\",\"cps\",\"bgd\",\"tdc\",\"gvp\",\"tdh\",\"wnc\",\"oci\",\"qxt\",\"nle\",\"ymg\",\"lga\",\"syk\",\"ont\",\"nma\",\"kny\",\"gaj\",\"tuk\",\"sbr\",\"onx\",\"pee\",\"mmy\",\"crc\",\"ymi\",\"cte\",\"crh\",\"vkj\",\"kuj\",\"rop\",\"dol\",\"kqr\",\"liy\",\"icl\",\"lwa\",\"sxs\",\"xsr\",\"mue\",\"dnk\",\"nrc\",\"nte\",\"vmk\",\"gvj\",\"lmo\",\"kff\",\"mio\",\"xab\",\"hmd\",\"shm\",\"kya\",\"bqt\",\"kez\",\"cle\",\"nwo\",\"buo\",\"xau\",\"bqx\",\"frp\",\"sxn\",\"waj\",\"bie\",\"bow\",\"kxq\",\"cma\",\"gvy\",\"kme\",\"xcb\",\"mdf\",\"wbl\",\"lel\",\"gdk\",\"mkk\",\"txi\",\"oav\",\"yih\",\"bno\",\"tol\",\"stk\",\"thf\",\"plq\",\"lrn\",\"kla\",\"xsp\",\"txg\",\"frr\",\"zav\",\"ven\",\"ybb\",\"cdm\",\"neq\",\"way\",\"vaf\",\"ypp\",\"xcu\",\"duk\",\"mne\",\"bon\",\"pmz\",\"ndm\",\"arx\",\"zag\",\"eke\",\"cqd\",\"ghe\",\"apa\",\"sxw\",\"raa\",\"nye\",\"zem\",\"spu\",\"knj\",\"gay\",\"tke\",\"iby\",\"ptq\",\"kta\",\"kuy\",\"tnk\",\"bzf\",\"gbl\",\"spb\",\"zai\",\"rkm\",\"kqp\",\"wdk\",\"mmj\",\"art\",\"lij\",\"bos\",\"ror\",\"xwk\",\"grd\",\"sbp\",\"kvh\",\"kvc\",\"zos\",\"wuh\",\"fry\",\"oon\",\"xpl\",\"bvv\",\"bux\",\"bqo\",\"bjk\",\"fiw\",\"bai\",\"war\",\"ngt\",\"nwx\",\"mit\",\"but\",\"woa\",\"yzk\",\"xsj\",\"ysl\",\"fad\",\"ngx\",\"nnd\",\"mzq\",\"gvr\",\"mix\",\"wsu\",\"lmx\",\"bag\",\"tba\",\"mqe\",\"bem\",\"oos\",\"pmb\",\"rth\",\"cwt\",\"goa\",\"rtc\",\"sbj\",\"lhi\",\"qxo\",\"kur\",\"lip\",\"bvi\",\"bdq\",\"kah\",\"kac\",\"kqj\",\"mmp\",\"ono\",\"ams\",\"bav\",\"nfl\",\"scl\",\"tjo\",\"bvg\",\"guc\",\"pim\",\"guh\",\"hna\",\"knp\",\"amn\",\"ibr\",\"gar\",\"trm\",\"azo\",\"tto\",\"xkk\",\"jwi\",\"ahk\",\"gim\",\"puc\",\"mcb\",\"gmu\",\"par\",\"sos\",\"ula\",\"rwm\",\"eto\",\"msp\",\"pcp\",\"ass\",\"bbp\",\"tge\",\"dyo\",\"son\",\"twe\",\"gmb\",\"koq\",\"asn\",\"bxw\",\"cik\",\"lcm\",\"blk\",\"mcu\",\"bpu\",\"psu\",\"ewe\",\"dov\",\"nik\",\"lst\",\"mgk\",\"aym\",\"tlo\",\"ade\",\"wmb\",\"uta\",\"elo\",\"khu\",\"rif\",\"khb\",\"int\",\"xmj\",\"meh\",\"yml\",\"mec\",\"wim\",\"btk\",\"mwk\",\"anf\",\"sno\",\"ywq\",\"gwf\",\"doi\",\"zpz\",\"vgr\",\"qus\",\"pbl\",\"mpl\",\"bcl\",\"afr\",\"uvh\",\"bpz\",\"mcz\",\"ckx\",\"das\",\"prd\",\"ruq\",\"czh\",\"pcj\",\"kdn\",\"dan\",\"bbj\",\"gmz\",\"sdq\",\"msj\",\"qun\",\"kja\",\"def\",\"otk\",\"taw\",\"sav\",\"ckt\",\"pay\",\"uur\",\"iow\",\"zpu\",\"sut\",\"kse\",\"tyt\",\"sag\",\"nkt\",\"tcn\",\"mfw\",\"sem\",\"xmp\",\"tcs\",\"nkx\",\"kbo\",\"khz\",\"yet\",\"tvw\",\"dtt\",\"ecs\",\"sqo\",\"sjk\",\"sux\",\"yum\",\"olk\",\"tyx\",\"sai\",\"zpb\",\"igs\",\"bsh\",\"phg\",\"bsc\",\"nwb\",\"kfa\",\"bub\",\"lim\",\"ngu\",\"mbc\",\"mbh\",\"miu\",\"ksv\",\"mmm\",\"lmu\",\"dsq\",\"kyf\",\"sve\",\"jid\",\"pip\",\"phi\",\"knm\",\"lmb\",\"ngb\",\"nxx\",\"buu\",\"xao\",\"ign\",\"mib\",\"iws\",\"hkh\",\"trp\",\"ndj\",\"sod\",\"ihp\",\"jcs\",\"ksi\",\"iii\",\"zil\",\"cdj\",\"nmf\",\"arz\",\"rcf\",\"pmx\",\"xvo\",\"ksg\",\"mjw\",\"shy\",\"cwb\",\"pmt\",\"phv\",\"sae\",\"rmk\",\"nck\",\"bep\",\"trj\",\"mul\",\"umi\",\"ndp\",\"keo\",\"buz\",\"xtw\",\"twg\",\"xng\",\"tgi\",\"aaq\",\"dju\",\"pij\",\"smc\",\"djb\",\"cll\",\"twi\",\"smh\",\"kqm\",\"xct\",\"umg\",\"krk\",\"tcd\",\"ybx\",\"ngz\",\"miz\",\"jge\",\"bil\",\"sbm\",\"xni\",\"prs\",\"adi\",\"dad\",\"spt\",\"bej\",\"oub\",\"kdd\",\"shr\",\"osc\",\"nll\",\"sgw\",\"qud\",\"ipo\",\"xxk\",\"huf\",\"tgv\",\"arb\",\"xsm\",\"tis\",\"dbf\",\"aru\",\"ctl\",\"pel\",\"sww\",\"tin\",\"adg\",\"dry\",\"spx\",\"diw\",\"prn\",\"doe\",\"llq\",\"sra\",\"ypm\",\"bkw\",\"ice\",\"xbc\",\"ldk\",\"hmw\",\"omc\",\"gel\",\"bhy\",\"wij\",\"obm\",\"bae\",\"foi\",\"ckb\",\"xmm\",\"sep\",\"grn\",\"mqg\",\"unm\",\"bfk\",\"grs\",\"ych\",\"cku\",\"mqi\",\"aly\",\"zmc\",\"zmh\",\"cbq\",\"mao\",\"xib\",\"yup\",\"usi\",\"nku\",\"dtb\",\"bve\",\"rri\",\"iks\",\"wrs\",\"hwc\",\"inz\",\"aua\",\"nqn\",\"ved\",\"lay\",\"rdb\",\"tfo\",\"zpt\",\"nbq\",\"ora\",\"tyu\",\"ssc\",\"ssh\",\"zpx\",\"hro\",\"kpy\",\"mvo\",\"aoc\",\"njl\",\"lbl\",\"opt\",\"wel\",\"zra\",\"sub\",\"iqu\",\"mqv\",\"aty\",\"yeu\",\"nkb\",\"wrn\",\"dtu\",\"mtw\",\"ayp\",\"wpc\",\"bww\",\"txe\",\"nyv\",\"yuj\",\"xul\",\"kmi\",\"whg\",\"mys\",\"nuw\",\"cns\",\"ckz\",\"nsf\",\"mcx\",\"bpx\",\"mng\",\"skw\",\"alr\",\"bra\",\"tkv\",\"bhr\",\"kmg\",\"mct\",\"sej\",\"bpt\",\"pst\",\"gmx\",\"rsk\",\"bgw\",\"mni\",\"myn\",\"urk\",\"mlw\",\"nyi\",\"bmh\",\"loa\",\"bmc\",\"gip\",\"nnn\",\"kpr\",\"fan\",\"inb\",\"kht\",\"kmv\",\"eki\",\"msm\",\"wmx\",\"sil\",\"bbm\",\"mxk\",\"atr\",\"pcm\",\"tea\",\"wmt\",\"luc\",\"tyz\",\"zae\",\"ekg\",\"cuw\",\"nyg\",\"suz\",\"lsb\",\"ood\",\"tkg\",\"mnv\",\"coo\",\"lcp\",\"csf\",\"nkz\",\"dgl\",\"lar\",\"khx\",\"ctg\",\"kbb\",\"nli\",\"xok\",\"myw\",\"peg\",\"atp\",\"nus\",\"rwr\",\"mts\",\"bws\",\"bgn\",\"adl\",\"pam\",\"muv\",\"sks\",\"gir\",\"mvx\",\"snz\",\"cnw\",\"tft\",\"lnh\",\"hrx\",\"zpo\",\"mvt\",\"lap\",\"skn\",\"opo\",\"squ\",\"pei\",\"pka\",\"fcs\",\"nlg\",\"kkq\",\"yme\",\"hrt\",\"aqa\",\"biv\",\"nun\",\"yla\",\"juk\",\"isc\",\"kih\",\"kbu\",\"ish\",\"kic\",\"mtn\",\"bwn\",\"bgs\",\"rgr\",\"nlv\",\"nti\",\"max\",\"alp\",\"mls\",\"egl\",\"yuy\",\"mui\",\"dkg\",\"tgl\",\"ayr\",\"big\",\"mug\",\"azz\",\"ttz\",\"sey\",\"cus\",\"pev\",\"abk\",\"emq\",\"ntg\",\"cli\",\"twl\",\"wir\",\"yta\",\"nnw\",\"etz\",\"mat\",\"bhp\",\"mln\",\"tmq\",\"ksl\",\"kbz\",\"kho\",\"hms\",\"gga\",\"kzk\",\"cox\",\"elu\",\"tpk\",\"laj\",\"tlu\",\"grw\",\"ria\",\"tlb\",\"wmo\",\"kpj\",\"snu\",\"hmn\",\"ana\",\"giy\",\"cot\",\"tzj\",\"atj\",\"gwa\",\"ikw\",\"bce\",\"dyb\",\"mpe\",\"bhj\",\"wrw\",\"pbe\",\"ser\",\"sza\",\"pzn\",\"xda\",\"etu\",\"not\",\"ulf\",\"vif\",\"wiy\",\"ttu\",\"uum\",\"wwa\",\"pso\",\"fwe\",\"azb\",\"phl\",\"yur\",\"ttb\",\"wga\",\"bpo\",\"mco\",\"pof\",\"etb\",\"yah\",\"zgn\",\"dyu\",\"yac\",\"alj\",\"xts\",\"onu\",\"ekl\",\"gof\",\"nyl\",\"gso\",\"mok\",\"mfd\",\"bey\",\"ghl\",\"tkl\",\"sii\",\"qxu\",\"dgg\",\"ifk\",\"shp\",\"ukq\",\"pmo\",\"gzn\",\"qwt\",\"gbe\",\"cdr\",\"frm\",\"xtn\",\"tvd\",\"dgi\",\"ubu\",\"tju\",\"sig\",\"onb\",\"xls\",\"kml\",\"bkn\",\"hss\",\"acq\",\"prw\",\"nxo\",\"try\",\"din\",\"lee\",\"wbe\",\"bqu\",\"tiw\",\"zza\",\"siv\",\"pwa\",\"piy\",\"sgs\",\"jog\",\"swn\",\"ndr\",\"tad\",\"pga\",\"wof\",\"kgq\",\"rom\",\"xug\",\"sgn\",\"sws\",\"hsn\",\"bnz\",\"bqb\",\"xln\",\"bks\",\"dis\",\"mnl\",\"tbf\",\"xat\",\"gev\",\"wam\",\"cjv\",\"okn\",\"igw\",\"raf\",\"spo\",\"weg\",\"apf\",\"lbg\",\"ren\",\"jas\",\"ktf\",\"cdy\",\"wka\",\"ber\",\"bza\",\"lbi\",\"res\",\"wei\",\"ira\",\"shj\",\"sce\",\"nji\",\"nak\",\"esq\",\"jan\",\"zkn\",\"gvm\",\"oks\",\"tsq\",\"mda\",\"gka\",\"gei\",\"cak\",\"bnu\",\"cji\",\"ybo\",\"vor\",\"xpe\",\"ket\",\"kum\",\"vkm\",\"mjn\",\"klf\",\"tha\",\"trr\",\"lbv\",\"ibm\",\"gam\",\"xco\",\"mjs\",\"geg\",\"ndy\",\"err\",\"vaa\",\"bnb\",\"bqz\",\"bxd\",\"pir\",\"jvn\",\"mql\",\"kex\",\"lwo\",\"gox\",\"cga\",\"otr\",\"jml\",\"aae\",\"kog\",\"yis\",\"xky\",\"dhu\",\"vec\",\"srz\",\"kzm\",\"yin\",\"byj\",\"nmo\",\"bly\",\"crw\",\"lrc\",\"ssd\",\"cwa\",\"ciy\",\"got\",\"tpm\",\"lgo\",\"aod\",\"afk\",\"koi\",\"sjr\",\"cmo\",\"ywg\",\"tbt\",\"olr\",\"bua\",\"kfb\",\"xbd\",\"ave\",\"mgy\",\"nwa\",\"ygi\",\"niy\",\"kyo\",\"xes\",\"zmd\",\"kfu\",\"nga\",\"zyp\",\"bty\",\"doq\",\"lle\",\"mia\",\"lma\",\"boh\",\"tbx\",\"ijj\",\"kov\",\"wli\",\"ara\",\"blr\",\"zoh\",\"dse\",\"jet\",\"zoc\",\"rui\",\"kcw\",\"apx\",\"rax\",\"pfe\",\"sdg\",\"gnw\",\"bmd\",\"oua\",\"jum\",\"ixc\",\"sru\",\"cir\",\"ktx\",\"aww\",\"zty\",\"byp\",\"srb\",\"dbo\",\"pak\",\"glv\",\"oty\",\"rug\",\"agw\",\"huo\",\"xom\",\"ktt\",\"wlg\",\"wun\",\"lud\",\"kvn\",\"xkr\",\"apt\",\"rat\",\"how\",\"ahr\",\"wti\",\"mwr\",\"wlv\",\"rts\",\"btr\",\"abm\",\"glg\",\"gun\",\"lpn\",\"kfz\",\"kas\",\"zyj\",\"dds\",\"amc\",\"amh\",\"klx\",\"ddn\",\"mhw\",\"gus\",\"kan\",\"saq\",\"klt\",\"nir\",\"mgr\",\"wnw\",\"kef\",\"dja\",\"sdv\",\"tez\",\"nam\",\"ptv\",\"sty\",\"pli\",\"neg\",\"pus\",\"soh\",\"soc\",\"mky\",\"mvf\",\"yki\",\"dnj\",\"hal\",\"aub\",\"yea\",\"asc\",\"vmj\",\"ash\",\"loz\",\"txq\",\"dta\",\"duy\",\"nei\",\"nka\",\"plg\",\"wji\",\"bjr\",\"rjs\",\"yyz\",\"wak\",\"eya\",\"auu\",\"sua\",\"oru\",\"tya\",\"ykg\",\"maf\",\"dme\",\"izz\",\"brz\",\"kjt\",\"gak\",\"plv\",\"sly\",\"pti\",\"mrl\",\"tuj\",\"tqr\",\"nvm\",\"mbd\",\"men\",\"ceg\",\"nev\",\"qvy\",\"ilp\",\"cam\",\"mes\",\"jbu\",\"jih\",\"wdy\",\"jic\",\"tny\",\"zaq\",\"kjx\",\"vkk\",\"pnw\",\"lko\",\"kuk\",\"quc\",\"chl\",\"quh\",\"teb\",\"cso\",\"bjy\",\"baq\",\"cof\",\"orz\",\"dur\",\"mzg\",\"dac\",\"yyu\",\"dah\",\"kdc\",\"kdh\",\"ina\",\"auz\",\"frk\",\"lob\",\"bdv\",\"vmp\",\"mom\",\"ail\",\"str\",\"lou\",\"czn\",\"rnl\",\"ifm\",\"lsa\",\"gdr\",\"mkr\",\"teu\",\"mzi\",\"enr\",\"brb\",\"jsl\",\"lje\",\"xwr\",\"nbe\",\"tnr\",\"pot\",\"bvq\",\"bdi\",\"tcc\",\"akw\",\"tch\",\"slr\",\"nof\",\"nso\",\"uan\",\"pox\",\"vit\",\"gyl\",\"bdg\",\"nzs\",\"tup\",\"pqw\",\"xgr\",\"bru\",\"mzv\",\"kjo\",\"kqk\",\"tnp\",\"qvp\",\"lzh\",\"aey\",\"sbk\",\"syr\",\"gll\",\"tur\",\"tah\",\"tac\",\"lkt\",\"ojp\",\"slp\",\"dcc\",\"bjj\",\"sqa\",\"pku\",\"wll\",\"ypk\",\"ity\",\"dup\",\"ylu\",\"bcq\",\"kba\",\"mpq\",\"mfc\",\"mfh\",\"zpf\",\"mkp\",\"aks\",\"anz\",\"ylb\",\"pkb\",\"dnr\",\"sdl\",\"evh\",\"vmr\",\"xxm\",\"stp\",\"jpr\",\"tuy\",\"wwb\",\"knk\",\"slj\",\"puw\",\"wgu\",\"tqp\",\"xri\",\"syy\",\"ygl\",\"mcf\",\"poo\",\"nsx\",\"dya\",\"szb\",\"nst\",\"bxc\",\"xrg\",\"lik\",\"dez\",\"ywl\",\"bxh\",\"qvj\",\"xwj\",\"kxv\",\"wgb\",\"kke\",\"ymq\",\"ccm\",\"tta\",\"mmk\",\"wdj\",\"aza\",\"aer\",\"pnn\",\"jmi\",\"riu\",\"stj\",\"sna\",\"cst\",\"gxx\",\"ggb\",\"vmy\",\"ncm\",\"anu\",\"dny\",\"rmm\",\"mkj\",\"gwu\",\"gdj\",\"mew\",\"gwb\",\"tme\",\"ggu\",\"aqz\",\"sfm\",\"anb\",\"kxi\",\"uge\",\"kol\",\"eme\",\"pns\",\"khf\",\"fum\",\"bjp\",\"itr\",\"tla\",\"csx\",\"rib\",\"kcs\",\"nph\",\"tbo\",\"pgu\",\"thz\",\"gyg\",\"nhg\",\"kvw\",\"chv\",\"cmt\",\"pwb\",\"urm\",\"ebo\",\"zlj\",\"bdl\",\"agn\",\"aws\",\"yha\",\"woo\",\"mwp\",\"btp\",\"rsm\",\"mdz\",\"kyx\",\"oyy\",\"nhi\",\"ags\",\"awn\",\"tse\",\"gyi\",\"gnn\",\"kyt\",\"mgp\",\"bqa\",\"kcn\",\"ese\",\"hos\",\"scq\",\"tdn\",\"aig\",\"rys\",\"byr\",\"cip\",\"wyi\",\"rtw\",\"wnn\",\"eja\",\"lwt\",\"lnd\",\"chi\",\"hno\",\"rng\",\"nmx\",\"xpq\",\"uba\",\"blp\",\"tja\",\"kaw\",\"zna\",\"pmf\",\"ddw\",\"mhn\",\"ona\",\"nhv\",\"nmt\",\"chg\",\"ahp\",\"isd\",\"mhs\",\"kid\",\"guw\",\"xkp\",\"aii\",\"cpc\",\"ryn\",\"pck\",\"mzl\",\"mxm\",\"bbk\",\"tds\",\"qxa\",\"goo\",\"bzz\",\"lgt\",\"msk\",\"cel\",\"yry\",\"bfm\",\"unk\",\"crn\",\"vau\",\"nij\",\"mgj\",\"hav\",\"sjp\",\"mdb\",\"gku\",\"mdu\",\"btj\",\"crs\",\"ldm\",\"klo\",\"bna\",\"yad\",\"fsl\",\"ehu\",\"gqn\",\"mrg\",\"xmk\",\"gbq\",\"pgz\",\"obk\",\"thu\",\"bzu\",\"wku\",\"mrv\",\"sxc\",\"iru\",\"pll\",\"kwe\",\"hvv\",\"toq\",\"hux\",\"leq\",\"xkj\",\"wbq\",\"ykl\",\"hai\",\"ace\",\"hut\",\"blj\",\"idi\",\"kto\",\"ztp\",\"dbt\",\"apo\",\"hag\",\"rao\",\"bzb\",\"wkb\",\"nrn\",\"kge\",\"hem\",\"kqw\",\"gog\",\"hni\",\"cho\",\"csl\",\"frs\",\"sbw\",\"czk\",\"tbv\",\"lrr\",\"hng\",\"lha\",\"goi\",\"bva\",\"kot\",\"xtm\",\"ver\",\"aio\",\"zhd\",\"sif\",\"ore\",\"fij\",\"yxu\",\"swm\",\"bor\",\"woi\",\"amy\",\"dmu\",\"uji\",\"dim\",\"bkm\",\"tbi\",\"ebg\",\"gov\",\"pdc\",\"wog\",\"dmb\",\"tbg\",\"sgm\",\"nzk\",\"gyo\",\"nsl\",\"baa\",\"ywt\",\"ynd\",\"jbe\",\"nho\",\"lbf\",\"apg\",\"hao\",\"xkc\",\"cpp\",\"rag\",\"ahh\",\"rem\",\"knw\",\"oor\",\"zaa\",\"okm\",\"jka\",\"rut\",\"jei\",\"cbb\",\"ktg\",\"sdx\",\"gvs\",\"wan\",\"lad\",\"eee\",\"wlx\",\"loe\",\"kpd\",\"rai\",\"gvn\",\"was\",\"cic\",\"api\",\"sdt\",\"liw\",\"cbu\",\"cih\",\"tee\",\"blh\",\"blc\",\"mmw\",\"oaa\",\"jam\",\"zor\",\"ckq\",\"kti\",\"klv\",\"pjt\",\"atd\",\"ido\",\"mjm\",\"nic\",\"bhd\",\"nih\",\"suq\",\"nbu\",\"mgc\",\"mgh\",\"kun\",\"gas\",\"vkn\",\"nkq\",\"klg\",\"mro\",\"gef\",\"mek\",\"amr\",\"boy\",\"txa\",\"rav\",\"lkl\",\"apv\",\"bth\",\"mwh\",\"mwc\",\"bre\",\"btc\",\"kli\",\"nbb\",\"fip\",\"ktv\",\"ald\",\"kus\",\"ibn\",\"gan\",\"ykt\",\"odt\",\"aau\",\"nex\",\"kvk\",\"kdy\",\"pef\",\"day\",\"mfj\",\"bjh\",\"pan\",\"bjc\",\"saa\",\"rxw\",\"quy\",\"kjv\",\"plt\",\"drd\",\"pas\",\"ltu\",\"sor\",\"mtm\",\"bwm\",\"asr\",\"num\",\"rbp\",\"dsz\",\"dbl\",\"net\",\"hul\",\"aab\",\"skm\",\"sva\",\"bxp\",\"kak\",\"bif\",\"avu\",\"cet\",\"llb\",\"kji\",\"ptt\",\"llu\",\"taj\",\"mlm\",\"kfe\",\"guk\",\"avb\",\"ecy\",\"kjg\",\"msw\",\"arq\",\"pcw\",\"tcy\",\"bbw\",\"lmq\",\"asy\",\"soy\",\"zbw\",\"mkc\",\"gdc\",\"hib\",\"afn\",\"gdh\",\"mkh\",\"pov\",\"yik\",\"ica\",\"lwl\",\"sre\",\"sth\",\"doa\",\"miq\",\"mzx\",\"ltz\",\"ngq\",\"dar\",\"buq\",\"lgl\",\"kdr\",\"shd\",\"hmm\",\"mzt\",\"qur\",\"dsb\",\"kxo\",\"mfp\",\"afs\",\"aaz\",\"nml\",\"viv\",\"xmw\",\"duh\",\"duc\",\"ecr\",\"bdt\",\"poi\",\"vig\",\"slc\",\"cml\",\"slh\",\"tap\",\"ojc\",\"zgm\",\"pog\",\"tnh\",\"tnc\",\"jiy\",\"kyl\",\"enc\",\"bxj\",\"uli\",\"enh\",\"qvh\",\"bdx\",\"qvc\",\"xwc\",\"xmn\",\"tar\",\"grm\",\"tcp\",\"mqf\",\"vra\",\"tsb\",\"syc\",\"pto\",\"tuc\",\"tuh\",\"lki\",\"wba\",\"lea\",\"crk\",\"tsu\",\"kll\",\"xms\",\"esu\",\"unn\",\"ndd\",\"soj\",\"gba\",\"vmc\",\"asj\",\"nrk\",\"yko\",\"vmh\",\"cdd\",\"plo\",\"nqm\",\"bnq\",\"jel\",\"apl\",\"ral\",\"wrm\",\"acz\",\"qup\",\"mfr\",\"dcr\",\"ktl\",\"ior\",\"kwz\",\"neo\",\"kdp\",\"qxq\",\"bdo\",\"kck\",\"trd\",\"wol\",\"aec\",\"tbl\",\"tsz\",\"jhi\",\"bxr\",\"mnf\",\"awk\",\"pid\",\"nsg\",\"csv\",\"vae\",\"gnk\",\"cfa\",\"xrt\",\"agk\",\"mym\",\"xpa\",\"qya\",\"yob\",\"paw\",\"tay\",\"mde\",\"gke\",\"kmf\",\"the\",\"hok\",\"nsi\",\"uku\",\"bed\",\"daj\",\"bbn\",\"mfy\",\"pcn\",\"kwu\",\"kdj\",\"msn\",\"kgb\",\"kxx\",\"itc\",\"ire\",\"bze\",\"acu\",\"nnm\",\"fam\",\"bqq\",\"csi\",\"jmx\",\"mzo\",\"nyf\",\"mhk\",\"pyx\",\"gol\",\"acb\",\"kgu\",\"asp\",\"nfa\",\"csg\",\"nsv\",\"bbs\",\"tdk\",\"sop\",\"tvy\",\"kxt\",\"kwb\",\"mss\",\"wnk\",\"tkf\",\"pol\",\"kkz\",\"nhx\",\"kns\",\"mmn\",\"tlq\",\"kyg\",\"lin\",\"dee\",\"cmi\",\"iwm\",\"dzd\",\"fir\",\"waw\",\"boj\",\"lis\",\"ijc\",\"igm\",\"gcn\",\"vil\",\"knn\",\"amp\",\"nmv\",\"nht\",\"cmg\",\"snq\",\"ull\",\"ayd\",\"kyi\",\"emz\",\"gaw\",\"lgg\",\"ttq\",\"aix\",\"lcd\",\"tmz\",\"nmg\",\"cht\",\"yle\",\"zca\",\"koo\",\"oca\",\"lgi\",\"chx\",\"ksf\",\"lwg\",\"gid\",\"kuw\",\"byc\",\"pnk\",\"byh\",\"nmi\",\"ait\",\"kyv\",\"twf\",\"kqs\",\"kkb\",\"yud\",\"amj\",\"fwa\",\"kjl\",\"sbs\",\"mrt\",\"npy\",\"bop\",\"sbn\",\"mrx\",\"sed\",\"sze\",\"mpa\",\"kbq\",\"bca\",\"hts\",\"kqn\",\"huv\",\"tgf\",\"kku\",\"dbv\",\"glo\",\"hyw\",\"sqq\",\"xsn\",\"emb\",\"ugb\",\"udj\",\"row\",\"hat\",\"dbg\",\"gwe\",\"tmb\",\"ane\",\"ruo\",\"hug\",\"wlo\",\"adf\",\"ypn\",\"gge\",\"sdo\",\"tmu\",\"akk\",\"prm\",\"cpy\",\"sxr\",\"hui\",\"vep\",\"idt\",\"tim\",\"emu\",\"hax\",\"dbi\",\"rsn\",\"nhf\",\"gyf\",\"nol\",\"urn\",\"myk\",\"agm\",\"xow\",\"cnk\",\"nxi\",\"wsg\",\"xuo\",\"hom\",\"bau\",\"yxa\",\"bab\",\"teq\",\"dma\",\"xqt\",\"kcm\",\"loq\",\"gnm\",\"wsi\",\"bsr\",\"awm\",\"nxg\",\"mbr\",\"juw\",\"pht\",\"smy\",\"lhu\",\"mhm\",\"bvu\",\"sio\",\"nke\",\"aif\",\"jns\",\"wnm\",\"tye\",\"zaz\",\"tdm\",\"kst\",\"sue\",\"mxs\",\"abw\",\"col\",\"ksx\",\"dgo\",\"brq\",\"mxn\",\"ofs\",\"chf\",\"drc\",\"bvb\",\"wsv\",\"yee\",\"gsg\",\"fak\",\"nnk\",\"pmi\",\"tgt\",\"sld\",\"kzw\",\"twx\",\"crm\",\"txu\",\"smr\",\"bfn\",\"mby\",\"cjo\",\"ybi\",\"xcg\",\"spv\",\"bsy\",\"geo\",\"xgd\",\"mrf\",\"nba\",\"egx\",\"lja\",\"end\",\"bfs\",\"grk\",\"xwd\",\"xnt\",\"mal\",\"txb\",\"tgx\",\"tnd\",\"twt\",\"auq\",\"wdd\",\"ldn\",\"zsr\",\"std\",\"wrk\",\"ikk\",\"cba\",\"zab\",\"adt\",\"spi\",\"vlp\",\"lse\",\"gdd\",\"nrm\",\"adx\",\"zau\",\"jku\",\"bvz\",\"shh\",\"shc\",\"ine\",\"haf\",\"njo\",\"nqk\",\"spg\",\"vbk\",\"mvl\",\"xcv\",\"weo\",\"lbo\",\"lla\",\"mie\",\"zpv\",\"xyk\",\"zmr\",\"nge\",\"mow\",\"krn\",\"srq\",\"yap\",\"muo\",\"mqx\",\"svb\",\"kel\",\"ycr\",\"lme\",\"nto\",\"sjd\",\"mqt\",\"old\",\"bio\",\"clo\",\"fox\",\"nwe\",\"krs\",\"xbr\",\"ava\",\"bue\",\"omr\",\"zpi\",\"sab\",\"doz\",\"prk\",\"akm\",\"lnj\",\"xhm\",\"ssr\",\"tik\",\"aor\",\"rrt\",\"nlo\",\"pqm\",\"cwe\",\"aaa\",\"kij\",\"otd\",\"zpg\",\"cto\",\"peo\",\"sau\",\"luy\",\"jgb\",\"bhc\",\"nid\",\"psg\",\"mnx\",\"omy\",\"bhh\",\"mgd\",\"dje\",\"ccs\",\"kmt\",\"khv\",\"xby\",\"bpg\",\"mcg\",\"naw\",\"btd\",\"ccn\",\"xal\",\"yaj\",\"kmx\",\"zmy\",\"alc\",\"alh\",\"mci\",\"bpi\",\"iwk\",\"gmg\",\"psi\",\"saz\",\"sfs\",\"rmn\",\"dob\",\"tkt\",\"kxf\",\"caw\",\"lah\",\"lac\",\"ncn\",\"nyt\",\"kip\",\"lur\",\"fli\",\"xkd\",\"gmv\",\"hia\",\"ght\",\"pnm\",\"khi\",\"wmg\",\"nyx\",\"fun\",\"bld\",\"atc\",\"tzh\",\"are\",\"ath\",\"bpv\",\"mcv\",\"wmi\",\"khg\",\"bmr\",\"oue\",\"rms\",\"tkx\",\"kpc\",\"ncs\",\"kph\",\"qtz\",\"pfa\",\"cid\",\"kfq\",\"ssy\",\"uzs\",\"nef\",\"zkk\",\"isr\",\"lup\",\"kir\",\"one\",\"pex\",\"tfi\",\"rro\",\"jak\",\"mvg\",\"spl\",\"gbb\",\"nan\",\"lcc\",\"lch\",\"efi\",\"mav\",\"gbu\",\"gih\",\"gic\",\"pum\",\"byd\",\"cvn\",\"nlx\",\"nas\",\"thq\",\"plf\",\"bmp\",\"ssj\",\"uzn\",\"zne\",\"mdq\",\"mvi\",\"aoj\",\"ctt\",\"ydk\",\"okk\",\"wih\",\"leu\",\"wic\",\"cas\",\"mem\",\"sfw\",\"tsa\",\"mag\",\"ivb\",\"bix\",\"bzq\",\"xbj\",\"ybl\",\"mut\",\"tou\",\"dkx\",\"tob\",\"mai\",\"bit\",\"yay\",\"ntx\",\"zmj\",\"mux\",\"ayh\",\"ayc\",\"mjk\",\"mqo\",\"can\",\"xcl\",\"mvv\",\"hrv\",\"leb\",\"wbb\",\"clt\",\"rmw\",\"xpz\",\"cog\",\"nov\",\"ssp\",\"mos\",\"eko\",\"kga\",\"scb\",\"nfu\",\"bmj\",\"gsl\",\"nyo\",\"gbz\",\"gho\",\"kiy\",\"kwa\",\"scu\",\"fqs\",\"lqr\",\"aca\",\"luj\",\"zqe\",\"pml\",\"mon\",\"krw\",\"ycp\",\"noi\",\"xpu\",\"bne\",\"lya\",\"yar\",\"kmo\",\"yuc\",\"uka\",\"nzm\",\"nxl\",\"sgk\",\"zmp\",\"toz\",\"omp\",\"mno\",\"cov\",\"nog\",\"swk\",\"oyd\",\"xbp\",\"uam\",\"bkk\",\"seh\",\"sec\",\"cyo\",\"lez\",\"dik\",\"bdf\",\"xpb\",\"cdh\",\"sit\",\"cdc\",\"ocu\",\"hmk\",\"kzs\",\"dgx\",\"kso\",\"dnd\",\"khl\",\"vmd\",\"ymb\",\"kof\",\"fll\",\"dgt\",\"osp\",\"xav\",\"tpn\",\"bfw\",\"six\",\"kbe\",\"kzn\",\"xvi\",\"yim\",\"xdq\",\"anq\",\"syd\",\"rkh\",\"xag\",\"smp\",\"tud\",\"xem\",\"xut\",\"bsj\",\"gml\",\"mbj\",\"eud\",\"zeh\",\"imy\",\"xai\",\"ndh\",\"ndc\",\"psl\",\"bcz\",\"mpz\",\"pho\",\"mcl\",\"bpl\",\"ele\",\"jun\",\"njx\",\"lbx\",\"nuk\",\"tma\",\"bwk\",\"deq\",\"mtk\",\"zpl\",\"ipi\",\"bec\",\"beh\",\"kev\",\"ema\",\"ymz\",\"sdf\",\"itd\",\"uga\",\"ado\",\"sne\",\"ruf\",\"wum\",\"uwa\",\"kvm\",\"xon\",\"njt\",\"urw\",\"lbt\",\"jus\",\"wet\",\"bgk\",\"rsw\",\"erh\",\"gum\",\"imr\",\"pic\",\"pbu\",\"pih\",\"mlk\",\"tgo\",\"mpu\",\"bcu\",\"aze\",\"trh\",\"trc\",\"tte\",\"mxw\",\"kka\",\"ego\",\"kei\",\"umo\",\"abn\",\"aed\",\"ppp\",\"xno\",\"wtf\",\"mbp\",\"gex\",\"abs\",\"cuk\",\"bsp\",\"rtm\",\"ewo\",\"pbb\",\"smj\",\"bcb\",\"keg\",\"mpb\",\"two\",\"kam\",\"zbc\",\"sng\",\"mkw\",\"scx\",\"kxa\",\"eli\",\"hif\",\"ctz\",\"sys\",\"pez\",\"tli\",\"zmm\",\"ttv\",\"stw\",\"tus\",\"jma\",\"doo\",\"syn\",\"tun\",\"xbm\",\"sni\",\"kyq\",\"xmc\",\"tlg\",\"duw\",\"xmh\",\"sct\",\"mej\",\"jle\",\"zib\",\"nlz\",\"sxk\",\"ngl\",\"xgw\",\"aom\",\"mil\",\"vms\",\"ssm\",\"biz\",\"hgm\",\"kee\",\"dyg\",\"xhr\",\"ojw\",\"xpt\",\"tti\",\"akr\",\"tlv\",\"slw\",\"lml\",\"puj\",\"kzd\",\"jgo\",\"xpx\",\"nmq\",\"snv\",\"pny\",\"xra\",\"tnw\",\"dnn\",\"qvw\",\"muz\",\"xww\",\"azg\",\"ttg\",\"lgq\",\"bul\",\"ntz\",\"enw\",\"dyi\",\"gbx\",\"mep\",\"jpn\",\"nlu\",\"zul\",\"ils\",\"aes\",\"kbg\",\"peb\",\"bjw\",\"sao\",\"aaf\",\"arl\",\"ctu\",\"xve\",\"qwe\",\"abd\",\"aen\",\"ziz\",\"kbi\",\"pnr\",\"llf\",\"ntu\",\"muu\",\"lum\",\"its\",\"bib\",\"wbt\",\"let\",\"xod\",\"eot\",\"xae\",\"lex\",\"vrt\",\"jud\",\"pup\",\"huq\",\"tqw\",\"clu\",\"bmm\",\"dbq\",\"pch\",\"mub\",\"pcc\",\"biu\",\"bbh\",\"bbc\",\"aky\",\"tox\",\"msc\",\"msh\",\"kbv\",\"ida\",\"hor\",\"knc\",\"dgz\",\"xkw\",\"gup\",\"bnv\",\"agr\",\"tyl\",\"cad\",\"njb\",\"gcc\",\"yrs\",\"ijn\",\"lbb\",\"mbm\",\"coe\",\"nju\",\"ciw\",\"lbu\",\"lih\",\"lic\",\"weu\",\"ppm\",\"awr\",\"bxk\",\"ijs\",\"siz\",\"bsm\",\"jko\",\"yrn\",\"gnr\",\"kcr\",\"zao\",\"kap\",\"blw\",\"cry\",\"mmc\",\"haa\",\"mmh\",\"yel\",\"dwz\",\"yij\",\"byn\",\"tdr\",\"rbk\",\"niw\",\"bni\",\"mgw\",\"xil\",\"noe\",\"txo\",\"pbt\",\"mhr\",\"mpt\",\"bct\",\"bng\",\"mra\",\"ckl\",\"kvp\",\"mpx\",\"bys\",\"mww\",\"btw\",\"geb\",\"ymx\",\"kaj\",\"kqc\",\"kqh\",\"dwu\",\"ddj\",\"oni\",\"xep\",\"awy\",\"dgb\",\"bqv\",\"aia\",\"smm\",\"zng\",\"sbc\",\"tjg\",\"bvo\",\"siu\",\"sbh\",\"efe\",\"crr\",\"njz\",\"lsl\",\"kcy\",\"lbz\",\"jje\",\"nsq\",\"cha\",\"hoy\",\"ong\",\"sib\",\"gqr\",\"tak\",\"hre\",\"otw\",\"zyn\",\"ubi\",\"guj\",\"tji\",\"mve\",\"inl\",\"agy\",\"xuu\",\"gya\",\"nha\",\"bao\",\"job\",\"csq\",\"zlw\",\"bqi\",\"yph\",\"kvj\",\"zsm\",\"mod\",\"mfk\",\"wny\",\"bqg\",\"tvk\",\"mae\",\"tdy\",\"dmf\",\"mhy\",\"xsc\",\"xsh\",\"nrr\",\"sjw\",\"yip\",\"xub\",\"gez\",\"zts\",\"xnb\",\"quk\",\"gla\",\"umu\",\"ext\",\"otn\",\"pbo\",\"tgu\",\"dak\",\"bco\",\"mpo\",\"kdk\",\"kvy\",\"frc\",\"wuy\",\"txt\",\"twb\",\"kkf\",\"gme\",\"dji\",\"nrp\",\"yir\",\"arv\",\"tdj\",\"twu\",\"tgb\",\"txx\",\"pse\",\"ztn\",\"bpe\",\"ots\",\"mhj\",\"xnu\",\"mce\",\"umb\",\"zls\",\"adu\",\"sjn\",\"gta\",\"roc\",\"roh\",\"arg\",\"gnj\",\"kay\",\"zax\",\"kjq\",\"crp\",\"tmf\",\"khe\",\"xer\",\"pdn\",\"kcj\",\"sda\",\"krd\",\"tck\",\"hoj\",\"wla\",\"ari\",\"sjs\",\"kbl\",\"ksz\",\"agj\",\"zln\",\"oui\",\"wme\",\"zat\",\"adb\",\"xnz\",\"mhp\",\"xks\",\"bln\",\"fud\",\"mii\",\"bat\",\"ywa\",\"ngi\",\"ahs\",\"lmi\",\"sok\",\"yrw\",\"ttl\",\"wnp\",\"axm\",\"nwg\",\"cin\",\"bug\",\"ask\",\"gvc\",\"ngg\",\"mig\",\"rmd\",\"wah\",\"wac\",\"ncd\",\"tgz\",\"phu\",\"xkn\",\"bls\",\"wur\",\"kvr\",\"bui\",\"nwi\",\"yga\",\"ahn\",\"bax\",\"lmg\",\"nis\",\"mgs\",\"mwn\",\"ccd\",\"btn\",\"ksu\",\"gah\",\"ibh\",\"zpe\",\"gac\",\"ngv\",\"cgg\",\"lmv\",\"lht\",\"gur\",\"tll\",\"bvt\",\"koa\",\"hop\",\"oco\",\"poq\",\"kar\",\"snl\",\"ddr\",\"bvx\",\"kcp\",\"hca\",\"mgn\",\"ksb\",\"mws\",\"bts\",\"buv\",\"nin\",\"crj\",\"cwg\",\"ymo\",\"kuc\",\"byw\",\"adz\",\"kuh\",\"bql\",\"aew\",\"mnb\",\"kmu\",\"mqz\",\"mxd\",\"lsv\",\"lfa\",\"ism\",\"kim\",\"cyb\",\"bjs\",\"ybe\",\"jnd\",\"xpo\",\"pnp\",\"pac\",\"bjn\",\"pah\",\"whu\",\"mnu\",\"xce\",\"kmb\",\"goq\",\"lnm\",\"yso\",\"bda\",\"akj\",\"pur\",\"yhl\",\"nyu\",\"itw\",\"tbq\",\"lsi\",\"tqn\",\"kgf\",\"dox\",\"tku\",\"amk\",\"mer\",\"mza\",\"ing\",\"qxl\",\"spe\",\"kwf\",\"tkb\",\"dot\",\"nzy\",\"urd\",\"tjl\",\"sco\",\"acf\",\"nyb\",\"ubl\",\"bnl\",\"too\",\"hbn\",\"xii\",\"akp\",\"hed\",\"nkv\",\"mnz\",\"dun\",\"mks\",\"gds\",\"svx\",\"mqb\",\"puy\",\"suv\",\"sts\",\"tuw\",\"dlg\",\"leo\",\"syw\",\"tyv\",\"nxe\",\"stn\",\"pnj\",\"cea\",\"vro\",\"ktq\",\"lrk\",\"pta\",\"yev\",\"kmz\",\"mqu\",\"gdn\",\"mkn\",\"afh\",\"raq\",\"dus\",\"apq\",\"wja\",\"xiv\",\"enn\",\"vmw\",\"nzr\",\"nea\",\"ffi\",\"usu\",\"nki\",\"dnw\",\"qvn\",\"sui\",\"tnn\",\"sls\",\"yam\",\"ldd\",\"sax\",\"ojs\",\"tyi\",\"gse\",\"sln\",\"tkz\",\"tns\",\"ckv\",\"nkg\",\"sat\",\"klq\",\"uar\",\"pla\",\"oda\",\"yka\",\"yei\",\"pme\",\"mey\",\"bfd\",\"bok\",\"xgn\",\"qvs\",\"dti\",\"gbo\",\"jye\",\"sug\",\"yot\",\"wgi\",\"szg\",\"bgr\",\"rgs\",\"xru\",\"anv\",\"fap\",\"gin\",\"nnp\",\"bhw\",\"hmy\",\"lcs\",\"kfl\",\"skr\",\"gis\",\"wgg\",\"alw\",\"nac\",\"nah\",\"yox\",\"xrb\",\"asm\",\"nur\",\"mtr\",\"axk\",\"bwr\",\"rgn\",\"som\",\"act\",\"ggg\",\"cac\",\"law\",\"cah\",\"pyu\",\"gcd\",\"rhp\",\"kgx\",\"kxb\",\"dso\",\"ani\",\"dhl\",\"knd\",\"kwt\",\"cur\",\"gwi\",\"ayn\",\"oia\",\"dof\",\"jim\",\"kgt\",\"szv\",\"atw\",\"myp\",\"kwx\",\"zia\",\"ays\",\"hio\",\"mmd\",\"acx\",\"kpw\",\"gwg\",\"dbe\",\"win\",\"lid\",\"dzn\",\"xaq\",\"grj\",\"mlr\",\"jmb\",\"cnp\",\"ang\",\"hue\",\"nvh\",\"tsx\",\"cla\",\"moh\",\"avo\",\"sky\",\"moc\",\"cme\",\"bia\",\"mty\",\"bwy\",\"dei\",\"nuy\",\"dka\",\"esx\",\"bgy\",\"hld\",\"wrp\",\"ikp\",\"xsd\",\"mua\",\"faj\",\"est\",\"qum\",\"nnj\",\"hmr\",\"kye\",\"deg\",\"tst\",\"kdm\",\"dam\",\"sbd\",\"sen\",\"keq\",\"cta\",\"tcm\",\"pea\",\"kxz\",\"kqd\",\"yli\",\"srl\",\"lwe\",\"aao\",\"dev\",\"saf\",\"myj\",\"pkg\",\"yun\",\"nla\",\"zhw\",\"cuy\",\"ses\",\"lto\",\"aqg\",\"zgr\",\"nme\",\"ylg\",\"cds\",\"lrm\",\"jay\",\"mru\",\"bzv\",\"mdg\",\"kzc\",\"vai\",\"tpc\",\"nbo\",\"gyz\",\"kkx\",\"vjk\",\"nhz\",\"eip\",\"mdi\",\"mvq\",\"gea\",\"kkt\",\"vut\",\"cja\",\"tip\",\"xtr\",\"vem\",\"cdn\",\"vag\",\"thi\",\"nse\",\"mrb\",\"rey\",\"nja\",\"bzi\",\"wea\",\"bkr\",\"iri\",\"dir\",\"zaf\",\"cse\",\"vav\",\"zen\",\"bom\",\"und\",\"ndn\",\"emx\",\"idb\",\"hau\",\"swr\",\"aul\",\"xmd\",\"idu\",\"hab\",\"mdv\",\"sgr\",\"bzg\",\"nds\",\"maq\",\"chz\",\"mjy\",\"cbo\",\"yak\",\"thv\",\"tmt\",\"shw\",\"ajw\",\"xoc\",\"pgi\",\"tij\",\"okr\",\"bes\",\"dmo\",\"rer\",\"gyb\",\"nhb\",\"pwg\",\"xty\",\"xua\",\"pcd\",\"ben\",\"nhu\",\"bbd\",\"pgg\",\"lke\",\"zom\",\"msd\",\"pwi\",\"brl\",\"zkr\",\"juc\",\"juh\",\"udm\",\"kik\",\"isk\",\"baf\",\"coq\",\"mrz\",\"ers\",\"rnb\",\"rxd\",\"sgy\",\"chu\",\"pis\",\"aib\",\"trs\",\"dga\",\"xyj\",\"noq\",\"mjr\",\"xly\",\"haz\",\"dwa\",\"diy\",\"trn\",\"bky\",\"abc\",\"abh\",\"bvf\",\"lol\",\"swy\",\"sia\",\"chb\",\"amm\",\"tel\",\"wyb\",\"pin\",\"koz\",\"ruu\",\"wlu\",\"sdb\",\"mjp\",\"ajs\",\"sgj\",\"aql\",\"bew\",\"xyy\",\"yll\",\"sri\",\"fmu\",\"ada\",\"ugo\",\"bkj\",\"gtu\",\"ihw\",\"dij\",\"srg\",\"tmo\",\"rub\",\"ute\",\"swj\",\"urc\",\"sdu\",\"ajn\",\"urh\",\"cbt\",\"piw\",\"tiy\",\"bmk\",\"drn\",\"twa\",\"poe\",\"erw\",\"zpq\",\"nbt\",\"xtj\",\"del\",\"xna\",\"trw\",\"ytl\",\"mxh\",\"srv\",\"mxc\",\"glb\",\"ega\",\"kko\",\"luk\",\"glu\",\"ljx\",\"uma\",\"bcf\",\"pbf\",\"zkp\",\"pad\",\"drs\",\"tga\",\"vie\",\"ule\",\"wtb\",\"swp\",\"gmq\",\"sdz\",\"ldh\",\"ril\",\"omk\",\"dip\",\"dhi\",\"bkp\",\"anl\",\"ggl\",\"bpq\",\"mcq\",\"bfh\",\"bfc\",\"xlp\",\"psq\",\"kfv\",\"sgp\",\"zmk\",\"dhg\",\"ruz\",\"kou\",\"mjj\",\"shn\",\"kfg\",\"khq\",\"jaj\",\"pha\",\"rkw\",\"heh\",\"kje\",\"ygu\",\"dhv\",\"xqa\",\"gft\",\"ssk\",\"ndw\",\"aok\",\"afd\",\"tir\",\"xtp\",\"dmx\",\"kfi\",\"sxm\",\"ywu\",\"szl\",\"okj\",\"prr\",\"rej\",\"shs\",\"mzb\",\"nxq\",\"log\",\"scf\",\"myy\",\"rod\",\"teg\",\"hix\",\"jee\",\"ynn\",\"mlj\",\"grr\",\"kwo\",\"nez\",\"tam\",\"nya\",\"loi\",\"smk\",\"hit\",\"gha\",\"ape\",\"eka\",\"kgo\",\"yns\",\"krh\",\"krc\",\"kte\",\"plz\",\"cuj\",\"tka\",\"tei\",\"mzu\",\"brv\",\"skj\",\"wha\",\"frd\",\"tvm\",\"pgl\",\"cya\",\"mna\",\"nuj\",\"brg\",\"zhn\",\"mtj\",\"fbl\",\"bdu\",\"bwj\",\"yuw\",\"bdb\",\"lov\",\"xpf\",\"nny\",\"fay\",\"kma\",\"bgj\",\"pmq\",\"hmp\",\"mfm\",\"kle\",\"zsk\",\"ikr\",\"bri\",\"tev\",\"wrr\",\"sew\",\"cup\",\"mzz\",\"bsk\",\"hne\",\"aug\",\"gbf\",\"bzl\",\"wkl\",\"bxm\",\"ppk\",\"plu\",\"mbk\",\"bhs\",\"odu\",\"yku\",\"aat\",\"cch\",\"rpt\",\"ccc\",\"ibd\",\"aln\",\"lvs\",\"gad\",\"org\",\"fvr\",\"neb\",\"cnr\",\"aax\",\"mlp\",\"bhn\",\"neu\",\"zrg\",\"usa\",\"aui\",\"giw\",\"goe\",\"kud\",\"als\",\"ori\",\"plb\",\"gry\",\"hkk\",\"myr\",\"gju\",\"llx\",\"avt\",\"spq\",\"far\",\"woe\",\"nnr\",\"kpn\",\"ptu\",\"las\",\"bgp\",\"mqa\",\"fuh\",\"wry\",\"tbe\",\"fuc\",\"hmj\",\"atn\",\"val\",\"tzn\",\"eso\",\"bdz\",\"mtp\",\"ufi\",\"bwp\",\"lef\",\"nqy\",\"wbf\",\"mdl\",\"ats\",\"nup\",\"orv\",\"tso\",\"ceb\",\"rmh\",\"jbi\",\"rmc\",\"thl\",\"nch\",\"kps\",\"skp\",\"lan\",\"ncc\",\"wad\",\"tof\",\"nce\",\"zun\",\"gly\",\"rme\",\"sak\",\"kva\",\"wua\",\"bot\",\"ars\",\"pdi\",\"tib\",\"mrp\",\"ygr\",\"arn\",\"emm\",\"box\",\"ywr\",\"tbh\",\"tiu\",\"sjg\",\"fue\",\"tbc\",\"npo\",\"bqw\",\"ael\",\"tmm\",\"woc\",\"pru\",\"sfe\",\"ebc\",\"paq\",\"dda\",\"hap\",\"cpo\",\"kaa\",\"tjw\",\"lrt\",\"xyb\",\"oti\",\"aij\",\"goc\",\"goh\",\"djn\",\"svk\",\"ztg\",\"kkm\",\"zpd\",\"vum\",\"qxw\",\"ruy\",\"wly\",\"chj\",\"cce\",\"gua\",\"lpa\",\"itl\",\"kor\",\"onw\",\"hnh\",\"mwi\",\"bti\",\"blv\",\"yia\",\"klh\",\"klc\",\"prz\",\"dok\",\"amx\",\"bpd\",\"mcd\",\"tiz\",\"mgg\",\"nig\",\"psd\",\"glr\",\"mrj\",\"tul\",\"syl\",\"btg\",\"mwg\",\"nii\",\"xkv\",\"bnw\",\"mgi\",\"gmd\",\"nhp\",\"amt\",\"bli\",\"mwv\",\"wlr\",\"btv\",\"bus\",\"kth\",\"ngn\",\"kre\",\"ktc\",\"jgk\",\"min\",\"igb\",\"koy\",\"ahg\",\"chp\",\"lmn\",\"wmd\",\"rah\",\"xkg\",\"rac\",\"aph\",\"apc\",\"jeh\",\"rnp\",\"sxo\",\"bun\",\"niv\",\"mgv\",\"vml\",\"xki\",\"aip\",\"mis\",\"khd\",\"haj\",\"sdr\",\"ngs\",\"ahi\",\"ins\",\"ikz\",\"gjr\",\"wrz\",\"ptr\",\"acm\",\"nnu\",\"fau\",\"jit\",\"lsn\",\"kwm\",\"tao\",\"lss\",\"bjv\",\"kjh\",\"kjc\",\"nnb\",\"bdy\",\"inn\",\"fab\",\"nxd\",\"vsl\",\"bvk\",\"myb\",\"sox\",\"kbw\",\"plr\",\"yom\",\"tvo\",\"bjg\",\"asx\",\"mzy\",\"ykr\",\"pmd\",\"cnb\",\"bfe\",\"ner\",\"nza\",\"bji\",\"cnu\",\"grz\",\"bak\",\"kif\",\"mfo\",\"sot\",\"kxp\",\"myu\",\"frq\",\"yzg\",\"ast\",\"yes\",\"nkn\",\"ojg\",\"ulc\",\"wrb\",\"tct\",\"qvi\",\"bxo\",\"slg\",\"dts\",\"vic\",\"duv\",\"xgg\",\"tni\",\"sun\",\"tyn\",\"pua\",\"zak\",\"kuq\",\"bdr\",\"cey\",\"ttw\",\"sus\",\"yrl\",\"stv\",\"sli\",\"xwg\",\"pty\",\"mxe\",\"eng\",\"gaq\",\"yaf\",\"tys\",\"oji\",\"wdg\",\"nnz\",\"vbb\",\"faz\",\"nks\",\"tcx\",\"tng\",\"poh\",\"wru\",\"dtn\",\"mkv\",\"xgi\",\"iku\",\"poc\",\"kdx\",\"myz\",\"env\",\"xin\",\"ure\",\"uth\",\"dax\",\"stg\",\"waq\",\"byl\",\"dui\",\"ney\",\"kxj\",\"snw\",\"gdg\",\"mkg\",\"qux\",\"tnv\",\"cks\",\"gru\",\"esm\",\"dug\",\"slv\",\"grb\",\"sti\",\"ply\",\"ckn\",\"ojv\",\"xis\",\"tsm\",\"kdt\",\"spd\",\"yky\",\"dln\",\"mzr\",\"mea\",\"mki\",\"gdi\",\"zxx\",\"zgb\",\"tvt\",\"nej\",\"ogu\",\"kxy\",\"mlz\",\"mfx\",\"bll\",\"zdj\",\"etn\",\"iqw\",\"abe\",\"vmg\",\"dng\",\"azn\",\"ttn\",\"pyy\",\"mzp\",\"xjb\",\"tts\",\"suw\",\"tuv\",\"tvx\",\"ahl\",\"ssf\",\"aso\",\"dni\",\"ogb\",\"ets\",\"xkl\",\"vmi\",\"mft\",\"aof\",\"dyn\",\"nkw\",\"soo\",\"tug\",\"bwz\",\"liq\",\"bdp\",\"xad\",\"hmb\",\"nuz\",\"mwl\",\"lkh\",\"lkc\",\"ymk\",\"tat\",\"sns\",\"skz\",\"mmq\",\"jio\",\"tln\",\"syi\",\"pna\",\"tax\",\"zmf\",\"knq\",\"tls\",\"snn\",\"him\",\"tui\",\"bgz\",\"hmu\",\"dnv\",\"vmv\",\"mgl\",\"nil\",\"mlb\",\"jmr\",\"otl\",\"avm\",\"xsq\",\"bmf\",\"cuu\",\"mzj\",\"cub\",\"csc\",\"csh\",\"lsw\",\"kxr\",\"luf\",\"mpk\",\"nep\",\"kdo\",\"llm\",\"bck\",\"dao\",\"ilv\",\"ztl\",\"iti\",\"mlu\",\"bwb\",\"nsc\",\"mtb\",\"nsh\",\"hmz\",\"bgu\",\"xrr\",\"nub\",\"tco\",\"sjl\",\"kbs\",\"ilg\",\"ptp\",\"skb\",\"sqs\",\"tpe\",\"kqq\",\"pqa\",\"sku\",\"sqn\",\"nuu\",\"sbq\",\"bgb\",\"xha\",\"kze\",\"ked\",\"kbn\",\"itv\",\"mtu\",\"ili\",\"bdj\",\"bwu\",\"aka\",\"dul\",\"lro\",\"wyr\",\"nmc\",\"nmh\",\"byi\",\"bnn\",\"chr\",\"gqa\",\"sgz\",\"mvd\",\"nbm\",\"lgh\",\"mju\",\"rup\",\"mjb\",\"sdp\",\"byg\",\"stl\",\"air\",\"lwh\",\"xzp\",\"tok\",\"veo\",\"mkl\",\"gdl\",\"cra\",\"diz\",\"wbk\",\"hay\",\"lek\",\"cpx\",\"bkz\",\"bns\",\"fmp\",\"rnr\",\"mad\",\"nww\",\"qvl\",\"byv\",\"xwl\",\"oku\",\"enl\",\"buw\",\"kyc\",\"kyh\",\"gbk\",\"xmq\",\"npx\",\"jab\",\"bsf\",\"boo\",\"tnl\",\"mbf\",\"nra\",\"ife\",\"nhr\",\"gyr\",\"cmc\",\"lmw\",\"sll\",\"reb\",\"jau\",\"mry\",\"okb\",\"miw\",\"xgl\",\"zku\",\"ngw\",\"fit\",\"moe\",\"bku\",\"awa\",\"yhs\",\"aiy\",\"huh\",\"huc\",\"gna\",\"hsb\",\"cod\",\"bqn\",\"xlb\",\"diu\",\"dmm\",\"sgb\",\"bbq\",\"kca\",\"har\",\"swu\",\"yxm\",\"sdj\",\"vnm\",\"arw\",\"msq\",\"hoa\",\"chy\",\"mjz\",\"swb\",\"idr\",\"kop\",\"bjl\",\"wyy\",\"zoo\",\"sgu\",\"xpk\",\"dib\",\"xlu\",\"aga\",\"vsv\",\"bqs\",\"cae\",\"onn\",\"mrr\",\"glj\",\"ygp\",\"zkz\",\"gyy\",\"xtb\",\"zns\",\"nhy\",\"tjs\",\"nae\",\"qxn\",\"jaz\",\"zyg\",\"sck\",\"tql\",\"qxs\",\"tda\",\"djw\",\"nod\",\"fkk\",\"smf\",\"okz\",\"ons\",\"amo\",\"xtu\",\"tjn\",\"vsi\",\"mha\",\"dri\",\"cxh\",\"iir\",\"pmh\",\"ksr\",\"pbm\",\"xya\",\"guu\",\"llk\",\"bcm\",\"mpm\",\"kab\",\"avk\",\"kau\",\"gub\",\"ady\",\"drg\",\"sij\",\"wep\",\"cpf\",\"fif\",\"afe\",\"sel\",\"wub\",\"pww\",\"kvb\",\"tia\",\"twy\",\"bsx\",\"vto\",\"xny\",\"cjp\",\"ajg\",\"pra\",\"srn\",\"mbx\",\"yiz\",\"bst\",\"xuj\",\"mbt\",\"egy\",\"yul\",\"phr\",\"ppt\",\"srs\",\"kvu\",\"wuu\",\"aak\",\"tgy\",\"aji\",\"kjd\",\"lbj\",\"adr\",\"shi\",\"upi\",\"rwl\",\"njj\",\"sip\",\"xeu\",\"rmq\",\"ncq\",\"osx\",\"gil\",\"kaz\",\"kfs\",\"ksy\",\"lcl\",\"spc\",\"guz\",\"kfn\",\"ost\",\"shg\",\"xeb\",\"fuq\",\"wkw\",\"bzw\",\"umr\",\"hik\",\"smt\",\"shv\",\"upv\",\"xch\",\"kvz\",\"xcc\",\"imo\",\"ayl\",\"tgr\",\"pae\",\"xup\",\"vid\",\"mdw\",\"dhn\",\"ocm\",\"twr\",\"gej\",\"ybh\",\"yiu\",\"dhs\",\"dzl\",\"smx\",\"ymm\",\"xnr\",\"wil\",\"dsk\",\"nzu\",\"kue\",\"gfk\",\"god\",\"cna\",\"sst\",\"pej\",\"brs\",\"mff\",\"aot\",\"zhi\",\"iso\",\"kio\",\"xpm\",\"eky\",\"nlj\",\"cfm\",\"mez\",\"nyy\",\"hnd\",\"bel\",\"zph\",\"ibe\",\"gae\",\"zpc\",\"vnk\",\"ssx\",\"nzb\",\"ysm\",\"aox\",\"mxq\",\"brn\",\"dmk\",\"vol\",\"tes\",\"clj\",\"los\",\"wae\",\"zmt\",\"for\",\"mny\",\"omx\",\"kmy\",\"zmx\",\"ntj\",\"taf\",\"lon\",\"wod\",\"gve\",\"ytw\",\"omt\",\"nna\",\"trl\",\"muj\",\"faa\",\"yng\",\"mqr\",\"pil\",\"ten\",\"tbd\",\"ghr\",\"tom\",\"wmh\",\"wmc\",\"lut\",\"nyr\",\"rad\",\"kpi\",\"ekr\",\"apd\",\"bmx\",\"lag\",\"ggw\",\"lem\",\"ktd\",\"wbm\",\"meu\",\"alv\",\"ati\",\"tkr\",\"bhv\",\"bmt\",\"gww\",\"flh\",\"kpg\",\"meb\",\"khc\",\"anw\",\"khh\",\"pep\",\"atg\",\"jbn\",\"nbk\",\"ctp\",\"nzz\",\"lai\",\"roe\",\"gra\",\"www\",\"ors\",\"mch\",\"bpc\",\"pub\",\"bph\",\"mcc\",\"bfq\",\"kpv\",\"zrn\",\"psc\",\"bhg\",\"kmr\",\"psh\",\"ndl\",\"mup\",\"yao\",\"szw\",\"cbk\",\"ntp\",\"kld\",\"aus\",\"mqy\",\"atv\",\"ali\",\"fre\",\"bhi\",\"gmh\",\"bip\",\"lvi\",\"ldq\",\"aun\",\"alg\",\"bxf\",\"orn\",\"mnr\",\"zrs\",\"puu\",\"gbm\",\"lav\",\"ntr\",\"pnu\",\"rin\",\"hle\",\"xse\",\"jbw\",\"mur\",\"mqj\",\"mac\",\"dzg\",\"hma\",\"mah\",\"kmp\",\"ann\",\"ayi\",\"zmo\",\"kyd\",\"gwn\",\"wig\",\"omo\",\"ans\",\"akz\",\"xbo\",\"mnp\",\"wii\",\"bir\",\"ayg\",\"pnb\",\"dkr\",\"tkp\",\"mvh\",\"asf\",\"shl\",\"orw\",\"hrc\",\"sso\",\"szs\",\"gii\",\"auw\",\"nmd\",\"ekp\",\"zga\",\"kit\",\"nyp\",\"ist\",\"szn\",\"hwo\",\"sbe\",\"per\",\"zwa\",\"kix\",\"kzq\",\"wiv\",\"tpq\",\"kqe\",\"gig\",\"cly\",\"brw\",\"hps\",\"sei\",\"ska\",\"yug\",\"abq\",\"pkn\",\"emk\",\"nua\",\"aqn\",\"xhu\",\"mnj\",\"yln\",\"biy\",\"yat\",\"mta\",\"aku\",\"bwa\",\"sam\",\"muy\",\"akb\",\"yui\",\"bga\",\"mqp\",\"seg\",\"kmj\",\"yax\",\"nty\",\"tcf\",\"noh\",\"noc\",\"pnz\",\"jdt\",\"pks\",\"tew\",\"cty\",\"pey\",\"mme\",\"kdf\",\"sev\",\"yvt\",\"luo\",\"kkk\",\"drl\",\"quf\",\"hud\",\"low\",\"coh\",\"coc\",\"usp\",\"dbd\",\"lie\",\"den\",\"mla\",\"gce\",\"nyj\",\"bmo\",\"des\",\"nly\",\"svm\",\"cua\",\"kne\",\"irn\",\"bzn\",\"lkd\",\"mse\",\"nrb\",\"xur\",\"phj\",\"pce\",\"tgp\",\"jaa\",\"bbe\",\"all\",\"zka\",\"kfw\",\"xac\",\"ump\",\"ndi\",\"mbo\",\"lvl\",\"bof\",\"caq\",\"ppo\",\"cjy\",\"mhz\",\"oka\",\"yda\",\"bso\",\"jkm\",\"gey\",\"bhl\",\"ndg\",\"zam\",\"rki\",\"jor\",\"bzs\",\"rea\",\"twp\",\"zeg\",\"nru\",\"dwr\",\"kpl\",\"naq\",\"ths\",\"njy\",\"qwc\",\"lby\",\"kcz\",\"esk\",\"qwh\",\"van\",\"tzl\",\"txm\",\"atl\",\"gnz\",\"sir\",\"mds\",\"ehs\",\"tsk\",\"ndv\",\"gkn\",\"mdn\",\"dhw\",\"vas\",\"crb\",\"agz\",\"ksj\",\"gqu\",\"lal\",\"cdi\",\"dgr\",\"hoz\",\"thn\",\"gza\",\"ger\",\"nrz\",\"kgk\",\"wnu\",\"amf\",\"smo\",\"trg\",\"ynl\",\"bvm\",\"pig\",\"nsd\",\"xta\",\"lhm\",\"erg\",\"mhu\",\"xnj\",\"imt\",\"bev\",\"tri\",\"ack\",\"mhb\",\"kec\",\"keh\",\"ryu\",\"kwk\",\"eri\",\"wnb\",\"tgj\",\"tdb\",\"pgn\",\"ukk\",\"bam\",\"kcb\",\"xme\",\"beg\",\"sga\",\"gnb\",\"ksp\",\"xla\",\"agu\",\"awb\",\"ihi\",\"wer\",\"adj\",\"bka\",\"awu\",\"oso\",\"trv\",\"bei\",\"lbr\",\"njr\",\"srw\",\"yok\",\"gnu\",\"crz\",\"axx\",\"agb\",\"dwy\",\"dia\",\"moq\",\"une\",\"pgs\",\"pwn\",\"kcu\",\"csd\",\"piv\",\"swa\",\"siy\",\"hob\",\"zbe\",\"xpr\",\"xht\",\"xmv\",\"mdd\",\"gkd\",\"ulw\",\"yau\",\"akt\",\"wal\",\"byq\",\"thd\",\"gvl\",\"puo\",\"hji\",\"ysr\",\"azc\",\"tth\",\"ttc\",\"tre\",\"ley\",\"hak\",\"pie\",\"bxa\",\"toy\",\"yab\",\"pow\",\"etc\",\"eth\",\"akx\",\"kul\",\"vkl\",\"meo\",\"gby\",\"fom\",\"obi\",\"lks\",\"nfr\",\"ung\",\"lnz\",\"snc\",\"lkn\",\"bzd\",\"wkd\",\"tlh\",\"bee\",\"xmg\",\"tlc\",\"mqm\",\"hdy\",\"mrk\",\"gal\",\"kiz\",\"ibl\",\"uni\",\"elh\",\"hvk\",\"pci\",\"tkm\",\"ler\",\"bbi\",\"wbr\",\"ysy\",\"msi\",\"ekm\",\"pnt\",\"nde\",\"nym\",\"csn\",\"tor\",\"aik\",\"msg\",\"pnx\",\"pgd\",\"taa\",\"faf\",\"css\",\"bbg\",\"xpy\",\"nnf\",\"pcg\",\"czo\",\"eaa\",\"yaz\",\"chk\",\"nss\",\"bbv\",\"mfa\",\"kbh\",\"msv\",\"kiu\",\"isu\",\"kbc\",\"nhk\",\"kmm\",\"lnb\",\"sqh\",\"saj\",\"myf\",\"jhs\",\"dop\",\"iar\",\"gbr\",\"mnm\",\"lnu\",\"nsn\",\"tva\",\"kib\",\"rol\",\"cde\",\"bap\",\"klw\",\"nms\",\"bcr\",\"mpr\",\"wci\",\"pbr\",\"mht\",\"mmv\",\"see\",\"tdt\",\"ksm\",\"liv\",\"lgs\",\"fgr\",\"kxk\",\"tdx\",\"vmq\",\"yue\",\"lws\",\"lgn\",\"txj\",\"nmn\",\"knv\",\"mhx\",\"bnc\",\"awx\",\"ktw\",\"agt\",\"zaj\",\"kys\",\"mmi\",\"ggd\",\"hot\",\"raw\",\"bvp\",\"kcx\",\"apw\",\"kng\",\"cmn\",\"lhp\",\"mmg\",\"kct\",\"cms\",\"euq\",\"and\",\"awt\",\"kni\",\"axb\",\"agx\",\"phm\",\"gwd\",\"gnt\",\"kyn\",\"lig\",\"tuq\",\"fia\",\"hus\",\"xsi\",\"tif\",\"ypg\",\"prf\",\"aye\",\"npa\",\"pal\",\"nrx\",\"sbv\",\"pby\",\"mpy\",\"tbw\",\"bcy\",\"kvo\",\"nrt\",\"kqv\",\"hun\",\"dze\",\"baj\",\"dbn\",\"bqc\",\"bqh\",\"wie\",\"ded\",\"wow\",\"xfa\",\"hti\",\"crt\",\"xsv\",\"guo\",\"lpo\",\"sbg\",\"egm\",\"gie\",\"umm\",\"kqg\",\"gow\",\"qxc\",\"xnm\",\"qxh\",\"sbi\",\"lce\",\"ymr\",\"bvj\",\"jkp\",\"aeq\",\"crx\",\"zap\",\"kqi\",\"rge\",\"aqd\",\"cpa\",\"ddo\",\"twm\",\"kao\",\"tqq\",\"won\",\"kpe\",\"zsu\",\"xet\",\"eze\",\"dbw\",\"uda\",\"ago\",\"huw\",\"oar\",\"ate\",\"arc\",\"arh\",\"rog\",\"ted\",\"tbn\",\"xum\",\"hoo\",\"bvy\",\"bkf\",\"dif\",\"zar\",\"kco\",\"tbs\",\"bdk\",\"gno\",\"zuh\",\"wos\",\"lae\",\"jkr\",\"awo\",\"osu\",\"ymp\",\"swf\",\"mbz\",\"msl\",\"yix\",\"mho\",\"hnn\",\"sim\",\"bbl\",\"pcl\",\"txr\",\"mzk\",\"ama\",\"wno\",\"gos\",\"ale\",\"smu\",\"tdo\",\"brd\",\"gon\",\"mpj\",\"bhe\",\"bcj\",\"djc\",\"smb\",\"bay\",\"yit\",\"hns\",\"gut\",\"obl\",\"kax\",\"tnq\",\"zay\",\"cro\",\"kls\",\"xml\",\"aud\",\"nmw\",\"kui\",\"bvr\",\"cgc\",\"ibg\",\"enq\",\"cjm\",\"gag\",\"wav\",\"gem\",\"kug\",\"lww\",\"lpx\",\"gux\",\"gjk\",\"gai\",\"lra\",\"zbl\",\"cek\",\"kln\",\"mbb\",\"buc\",\"kyw\",\"buh\",\"nwc\",\"bsb\",\"kts\",\"jaf\",\"kuv\",\"nek\",\"mpp\",\"kvx\",\"wux\",\"bcp\",\"jen\",\"pbp\",\"yne\",\"wai\",\"bar\",\"ras\",\"aps\",\"duq\",\"kvt\",\"lmh\",\"wut\",\"boa\",\"lmc\",\"txy\",\"odk\",\"ykk\",\"apn\",\"smz\",\"mkq\",\"gdq\",\"ran\",\"bsu\",\"stq\",\"wag\",\"njm\",\"mic\",\"plk\",\"mbu\",\"mih\",\"ppu\",\"wem\",\"ngc\",\"ngh\",\"ktn\",\"lbm\",\"inc\",\"kda\",\"inh\",\"daa\",\"bmb\",\"met\",\"she\",\"pav\",\"glk\",\"say\",\"mum\",\"qua\",\"luu\",\"mlf\",\"kjn\",\"ntm\",\"lsh\",\"zlq\",\"csw\",\"lsc\",\"aoz\",\"lub\",\"ssz\",\"kjs\",\"bim\",\"sbl\",\"clm\",\"wtk\",\"bmu\",\"icr\",\"srd\",\"gbp\",\"dor\",\"kql\",\"skf\",\"ako\",\"top\",\"sdk\",\"xsl\",\"nsw\",\"xho\",\"dva\",\"pai\",\"nuf\",\"pux\",\"mtf\",\"otq\",\"bwf\",\"nlm\",\"wbp\",\"lep\",\"ztq\",\"xpj\",\"put\",\"wxw\",\"bgf\",\"wlk\",\"ctm\",\"tca\",\"ruk\",\"pem\",\"zmz\",\"pag\",\"yec\",\"mml\",\"soa\",\"gbj\",\"dre\",\"bmz\",\"dth\",\"pon\",\"vis\",\"asa\",\"doy\",\"mwq\",\"btq\",\"ssu\",\"lil\",\"aou\",\"niq\",\"suc\",\"mgq\",\"aob\",\"gcl\",\"ssb\",\"luz\",\"tyh\",\"uln\",\"nkc\",\"scp\",\"nkh\",\"vin\",\"knl\",\"dhd\",\"sar\",\"pos\",\"omb\",\"zmu\",\"xbb\",\"afi\",\"kok\",\"kfd\",\"blq\",\"hmf\",\"xpp\",\"pno\",\"qyp\",\"ckh\",\"zim\",\"xkq\",\"afg\",\"jia\",\"wbj\",\"lej\",\"zmb\",\"omu\",\"ysp\",\"czt\",\"toj\",\"kwr\",\"gro\",\"rbb\",\"cut\",\"ndq\",\"wmw\",\"mfz\",\"mas\",\"hij\",\"acr\",\"bfl\",\"cag\",\"nav\",\"gjm\",\"mlx\",\"aap\",\"khw\",\"anc\",\"anh\",\"man\",\"gwc\",\"mlt\",\"gek\",\"cai\",\"kgr\",\"ldl\",\"yva\",\"cjk\",\"mef\",\"cux\",\"tsy\",\"psw\",\"mvs\",\"skx\",\"bpw\",\"jda\",\"mcw\",\"tfn\",\"ykm\",\"cvg\",\"lbk\",\"szc\",\"nux\",\"bxu\",\"esy\",\"yor\",\"nqo\",\"bgt\",\"nai\",\"fni\",\"lzz\",\"bwx\",\"xdc\",\"mtx\",\"puf\",\"nut\",\"iko\",\"wro\",\"bxb\",\"bgx\",\"yaa\",\"mtt\",\"gmw\",\"bnd\",\"bwt\",\"ukr\",\"mvn\",\"fng\",\"cav\",\"nag\",\"skt\",\"llp\",\"nem\",\"taz\",\"bdm\",\"xjt\",\"url\",\"non\",\"rsl\",\"lna\",\"bye\",\"mfb\",\"tvu\",\"kgy\",\"znd\",\"hhi\",\"myo\",\"ues\",\"aqc\",\"mfu\",\"kia\",\"kwy\",\"isa\",\"trq\",\"nos\",\"acy\",\"zpw\",\"iou\",\"cno\",\"pkc\",\"pkh\",\"mov\",\"llj\",\"tau\",\"dgk\",\"yhd\",\"ije\",\"nno\",\"mog\",\"hmx\",\"fao\",\"cos\",\"uky\",\"mzm\",\"mxl\",\"yre\",\"hmt\",\"sik\",\"bqd\",\"bxz\",\"deh\",\"dec\",\"con\",\"tsr\",\"beq\",\"dwk\",\"jnl\",\"yoy\",\"moi\",\"tab\",\"ygm\",\"snd\",\"swt\",\"ayq\",\"cbj\",\"sxb\",\"sgx\",\"bkt\",\"xas\",\"iwo\",\"bkx\",\"tpv\",\"ccl\",\"igo\",\"xan\",\"tmr\",\"dix\",\"sxu\",\"ite\",\"bzh\",\"bzc\",\"irh\",\"swx\",\"tld\",\"kzv\",\"sgt\",\"giq\",\"yif\",\"xcw\",\"ncl\",\"tpg\",\"rml\",\"thh\",\"xvs\",\"qws\",\"kzg\",\"mdh\",\"mdc\",\"vnp\",\"dyd\",\"kom\",\"nbj\",\"xtt\",\"vah\",\"ful\",\"tpi\",\"ima\",\"zik\",\"ile\",\"aee\",\"azd\",\"xvn\",\"epi\",\"ttd\",\"kzi\",\"lcq\",\"kkr\",\"tmy\",\"bik\",\"kaf\",\"pmw\",\"mjt\",\"krl\",\"dkk\",\"emy\",\"seq\",\"ugy\",\"cpu\",\"abi\",\"ken\",\"kbd\",\"clk\",\"wtm\",\"cpb\",\"kes\",\"gsw\",\"vme\",\"ntk\",\"guf\",\"yuq\",\"abg\",\"mjx\",\"dne\",\"muk\",\"jls\",\"jui\",\"ret\",\"jax\",\"npb\",\"kky\",\"kvf\",\"abv\",\"pek\",\"ljp\",\"rum\",\"xog\",\"nbp\",\"wlm\",\"zkt\",\"tue\",\"fiu\",\"xzm\",\"okx\",\"pro\",\"nlk\",\"npu\",\"xoi\",\"tio\",\"jat\",\"mxg\",\"qve\",\"xwe\",\"nbr\",\"urv\",\"pms\",\"jng\",\"chm\",\"gsn\",\"amb\",\"wym\",\"kkj\",\"kew\",\"dmy\",\"tdf\",\"gss\",\"yxy\",\"sle\",\"xto\",\"mxi\",\"jct\",\"amu\",\"aim\",\"ghk\",\"nyk\",\"sma\",\"boz\",\"pmn\",\"mhf\",\"jni\",\"ekk\",\"swo\",\"due\",\"mnk\",\"wss\",\"tmj\",\"uri\",\"ynq\",\"ind\",\"kcf\",\"whk\",\"nxn\",\"dio\",\"bko\",\"osa\",\"urg\",\"zsa\",\"lrz\",\"cbr\",\"xlo\",\"udu\",\"ste\",\"src\",\"srh\",\"mxv\",\"agf\",\"lsd\",\"mke\",\"gde\",\"gym\",\"kmk\",\"nhm\",\"bou\",\"ham\",\"dmr\",\"tix\",\"atq\",\"heg\",\"xaw\",\"oko\",\"bsa\",\"uhn\",\"spn\",\"amz\",\"nal\",\"prx\",\"nrf\",\"kpq\",\"mba\",\"kfh\",\"kfc\",\"hei\",\"hka\",\"kkp\",\"prt\",\"sps\",\"zko\",\"nby\",\"laq\",\"eit\",\"usk\",\"jao\",\"bob\",\"tit\",\"mjo\",\"mqk\",\"cby\",\"alq\",\"ybn\",\"bfi\",\"ldg\",\"dtd\",\"mrm\",\"xyt\",\"nkd\",\"emp\",\"bje\",\"bhq\",\"crf\",\"cal\",\"bfg\",\"ldi\",\"xcn\",\"krv\",\"zpn\",\"brh\",\"mwe\",\"bte\",\"jul\",\"izh\",\"brc\",\"drq\",\"bgo\",\"tcu\",\"wrx\",\"nqt\",\"ikx\",\"mto\",\"bwo\",\"fpe\",\"nuo\",\"tcb\",\"ikt\",\"zps\",\"tsp\",\"nie\",\"akf\",\"sko\",\"djd\",\"mge\",\"now\",\"kri\",\"ard\",\"cuo\",\"tec\",\"ble\",\"teh\",\"grt\",\"qub\",\"xnk\",\"bma\",\"loc\",\"cow\",\"loh\",\"kgj\",\"cie\",\"asz\",\"abl\",\"soz\",\"kwj\",\"kdu\",\"tgk\",\"mlo\",\"dau\",\"hir\",\"lua\",\"gpa\",\"xke\",\"grx\",\"ole\",\"jiu\",\"rmi\",\"wms\",\"sje\",\"nci\",\"tsj\",\"pym\",\"fln\",\"bud\",\"shq\",\"maw\",\"oma\",\"nnt\",\"khn\",\"fat\",\"zma\",\"fui\",\"tpl\",\"ncg\",\"mid\",\"khs\",\"hmo\",\"rmg\",\"ukp\",\"tcz\",\"ngd\",\"fax\",\"kxm\",\"kzl\",\"wmn\",\"lmd\",\"zle\",\"ksk\",\"jib\",\"ote\",\"orc\",\"hrw\",\"bps\",\"mcs\",\"orh\",\"aar\",\"cnx\",\"rmv\",\"ccg\",\"quz\",\"pss\",\"acp\",\"mvw\",\"gmn\",\"asb\",\"xrm\",\"phk\",\"auh\",\"ogo\",\"myx\",\"kdz\",\"auc\",\"kwp\",\"daz\",\"hwa\",\"sob\",\"sou\",\"fuv\",\"kgp\",\"asu\",\"bpn\",\"mcn\",\"cwd\",\"cnt\",\"ssa\",\"zte\",\"psn\",\"aoa\"]",""]

Data::Sah can also create validator that returns a hash of detailed error
message. Data::Sah can even create validator that targets other language, like
JavaScript, from the same schema. Other things Data::Sah can do: show source
code for validator, generate a validator code with debug comments and/or log
statements, generate human text from schema. See its documentation for more
details.

=head2 Using with Params::Sah

To validate function parameters against this schema (requires L<Params::Sah>):

 use Params::Sah qw(gen_validator);

 sub myfunc {
     my @args = @_;
     state $validator = gen_validator("language::code*");
     $validator->(\@args);
     ...
 }

=head2 Using with Perinci::CmdLine::Lite

To specify schema in L<Rinci> function metadata and use the metadata with
L<Perinci::CmdLine> (L<Perinci::CmdLine::Lite>) to create a CLI:

 # in lib/MyApp.pm
 package
   MyApp;
 our %SPEC;
 $SPEC{myfunc} = {
     v => 1.1,
     summary => 'Routine to do blah ...',
     args => {
         arg1 => {
             summary => 'The blah blah argument',
             schema => ['language::code*'],
         },
         ...
     },
 };
 sub myfunc {
     my %args = @_;
     ...
 }
 1;

 # in myapp.pl
 package
   main;
 use Perinci::CmdLine::Any;
 Perinci::CmdLine::Any->new(url=>'/MyApp/myfunc')->run;

 # in command-line
 % ./myapp.pl --help
 myapp - Routine to do blah ...
 ...

 % ./myapp.pl --version

 % ./myapp.pl --arg1 ...

=head2 Using on the CLI with validate-with-sah

To validate some data on the CLI, you can use L<validate-with-sah> utility.
Specify the schema as the first argument (encoded in Perl syntax) and the data
to validate as the second argument (encoded in Perl syntax):

 % validate-with-sah '"language::code*"' '"data..."'

C<validate-with-sah> has several options for, e.g. validating multiple data,
showing the generated validator code (Perl/JavaScript/etc), or loading
schema/data from file. See its manpage for more details.


=head2 Using with Type::Tiny

To create a type constraint and type library from a schema (requires
L<Type::Tiny> as well as L<Type::FromSah>):

 package My::Types {
     use Type::Library -base;
     use Type::FromSah qw( sah2type );

     __PACKAGE__->add_type(
         sah2type('language::code*', name=>'LanguageCode')
     );
 }

 use My::Types qw(LanguageCode);
 LanguageCode->assert_valid($data);

=head1 DESCRIPTION

Accept only current (not retired) codes. Alpha-2 or alpha-3 codes are accepted.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Sah-Schemas-Language>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Sah-Schemas-Language>.

=head1 SEE ALSO

L<Sah::Schema::language::code::alpha2>

L<Sah::Schema::language::code::alpha3>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTING


To contribute, you can send patches by email/via RT, or send pull requests on
GitHub.

Most of the time, you don't need to build the distribution yourself. You can
simply modify the code, then test via:

 % prove -l

If you want to build the distribution (e.g. to try to install it locally on your
system), you can install L<Dist::Zilla>,
L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
that are considered a bug and can be reported to me.

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2023, 2020, 2019, 2018 by perlancar <perlancar@cpan.org>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Sah-Schemas-Language>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=cut
